#ifndef BATCHFORM_H
#define BATCHFORM_H


#include "workerthread.h"
#include <gtkmm.h>
#include "GladeUtils.h"
#include <iostream>
#include <memory>
#include <batchsettingswindow.h>
#include "photoprocessor.h"
#include "preprocessingoptionsform.h"
#include "../PhotomatixExternalLibs/stlsoft/include/stlsoft/string/simple_string.hpp"


namespace Photomatix
{
	// Forward declaration
	class BatchForm;
	
	
	class BatchProcessor: public PhotomatixMisc::WorkerThread
	{
	protected:
		/// Standard output channel for PhotomatixCL process
		Glib::RefPtr<Glib::IOChannel> ch_stdout;
		Glib::RefPtr<Gtk::TextBuffer> textBuffer;
		
		int doneSteps;
		int totalSteps;
		int totalNumberOfSets;
		int bracketSize;
		
		std::vector <std::string> command;
		std::vector <std::string> fileNames;
		
		std::string firstFile;
		
		BatchForm* batchForm;
		
		
		virtual void Process();
		bool TreatBatchOutput(Glib::IOCondition cond);
		
	public:
		BatchProcessor(int setsNum, int setSize, std::vector <std::string>& cmd, std::vector <std::string>& files, Glib::RefPtr<Gtk::TextBuffer> textBuf,
			BatchForm* form, int totalSteps);
	};
	
	class BatchForm: public Gtk::Window
	{
	private:
	  std::shared_ptr<BatchForm> myRef;
	  
	  struct FileListEntry
	  {
		  std::string shortName;
		  std::string fullPath;
		  
		  bool operator < (const FileListEntry& rhs) const
		  {
			  return (shortName < rhs.shortName);
		  }
	  };
	  
	  int activeFilesCount;
	  std::string outputPath;
	  std::string numberOfMergedFilesNote;
	  
	  std::vector <FileListEntry> allSourceFiles;
	  std::vector <std::string> tmpFiles;

		void OpenPath(std::string& path);
	  
	protected:
		Gtk::Button *btnCancel;
		Gtk::Button *btnDestChoose;
		Gtk::Button *btnNaming;
		Gtk::Button *btnRemove;
		Gtk::Button *btnRun;
		Gtk::Button *btnSelect;
		Gtk::Button *btnStop;
		Gtk::Button *btnTM1;
		Gtk::Button *btnTM2;
		Gtk::Button *btnTM3;
		Gtk::Button *btnToneBalancer;
		Gtk::Button *btnAdjust;
		Gtk::CheckButton *chkAdjust;
		Gtk::CheckButton *chkAlign;
		Gtk::CheckButton *chkAuto;
		Gtk::CheckButton *chkAverage;
		Gtk::CheckButton *chkCrop;
		Gtk::CheckButton *chkIntensive;
		Gtk::CheckButton *chkPerspectiveCorrection;
		Gtk::CheckButton *chkRealistic;
		Gtk::CheckButton *chkRemoveHDR;
		Gtk::CheckButton *chkTM1;
		Gtk::CheckButton *chkTM2;
		Gtk::CheckButton *chkTM3;
		Gtk::CheckButton *chkToneBalancer;
		Gtk::ComboBox *cbxHdrSavedAs;
		Gtk::ComboBox *cbxSaveAs;
		Gtk::ComboBox *cmbBracket;
		Gtk::ComboBox *cmbFilter;
		Gtk::Entry *txtDest;
		Gtk::Label *label1;
		Gtk::Label *label2;
		Gtk::Label *label3;
		Gtk::Label *label4;
		Gtk::Label *label5;
		Gtk::Label *label6;
		Gtk::Label *label7;
		Gtk::Label *label8;
		Gtk::Label *labelDest;
		Gtk::Label *lblNotice;
		//Gtk::RadioButton *rbnAdvanced;
		Gtk::RadioButton *rbnAlignRotation;
		Gtk::RadioButton *rbnAlignTranslation;
		Gtk::RadioButton *rbnDestCustom;
		Gtk::RadioButton *rbnDestUnderSource;
		//Gtk::RadioButton *rbnSelect;
		Gtk::RadioButton *rbnSelectionFiles;
		Gtk::RadioButton *rbnSelectionFolder;
		Gtk::SpinButton *nudJpgQuality;
		Gtk::TreeView* lstFileList;
		Gtk::TextView* txtBatchLog;
		Gtk::ProgressBar* prgGlobal;
		
		Gtk::CheckButton* chkReduceNoise;
		//Gtk::CheckButton *chkForceSpacing;
		Gtk::CheckButton *chkRemoveGhosts;
		//Gtk::ComboBoxText *cbxColorProfile;
		//Gtk::ComboBoxText *cbxEVSpacing;
		Gtk::ComboBoxText *cbxReduceNoiseTarget;
		//Gtk::ComboBoxText *cbxWhiteBalance;
		//Gtk::Entry *txtWhiteBalanceTemperature;
		Gtk::Scale* sldReduceNoiseStrength;
		Gtk::Scale* sldDeghosting;
		
		Glib::RefPtr<Gtk::TextBuffer> textBuffer;
		
		/** file list data model. This data model is instantiated in the
		 * class constructor and attached to the treeview laid out in
		 * glade
		 */
		Glib::RefPtr<Gtk::ListStore> sourceFileList;
		
		/**
		 * Data model for our file list.  The data model for this file
		 * list contains two fields: a full path to the location of the
		 * file, and the short filename that is displayed to the user
		 */
		class SourceFileColumns : public Gtk::TreeModel::ColumnRecord
		{
		public:
			
			SourceFileColumns() {add(shortName); add(fullPath);}
			
			/** short name that is displayed to the user */
			Gtk::TreeModelColumn<Glib::ustring> shortName;
			/** full path that is stored hidden to access the file */
			Gtk::TreeModelColumn<Glib::ustring> fullPath;
		} sourceFileColumns;
		
		/// Standard output channel for PhotomatixCL process
		Glib::RefPtr<Glib::IOChannel> ch_stdout;
		
		int totalNumberOfSets;
		int bracketSize;
		
		BatchSettingsWindow* deSettings;
		BatchSettingsWindow* tcSettings;
		BatchSettingsWindow* tbSettings;
		BatchSettingsWindow* coSettings;
		BatchSettingsWindow* efSettings;
		
		std::shared_ptr<BatchProcessor> processor;
		
		std::string sourcePath;
		

		//PhotomatixImaging::HdrGenerator* hdrGenerator;
		
		//Gtk::CheckButton* chkAlignImages;
		//Gtk::CheckButton* chkCrop;
		//Gtk::RadioButton* rbnAlignShifts;
		//Gtk::RadioButton* rbnAlignFeatures;
		//Gtk::CheckButton* chkPerspective;
		//Gtk::Label* lblPerspective;
		//Gtk::Label* lblMaximumShift;
		//Gtk::Label* lblMaximumShiftPercentage;
		//Gtk::HScale* sldMaximumShift;
		
		
		//Gtk::CheckButton* chkReduceChromaticAberrations;
		//Gtk::CheckButton* chkReduceNoise;
		//Gtk::ComboBox* cbxReduceNoiseType;
		//Gtk::Label* lblReduceNoiseStrength;
		//Gtk::Label* lblReduceNoiseStrengthPercentage;
		//Gtk::HScale* sldReduceNoiseStrength;
		
		//@{
		/** Signal Handlers */
		void OnBtnDetailsEnhancerSettingsClicked();
		void OnBtnToneCompressorSettingsClicked();
		void OnBtnToneBalancerSettingsClicked();
		void OnBtnContrastOptimizerSettingsClicked();
		void OnBtnExposureFusionSettingsClicked();
		void OnBtnDestChooseClicked();
		
		void OnListSelectedItemsChanged();
		
		void OnBtnSelectClicked();
		void OnBtnRunClicked();
		void OnBtnRemoveClicked();
		
		void OnCmbBracketChanged();
		void OnCbxFileFormatChanged();
		
		void OnFilterSelectedIndexChanged();
		
		void OnReduceNoiseCheckedChanged();
		void OnRemoveGhostsCheckedChanged();
		void OnCustomPathCheckedChanged();
		void OnSelectFilesCheckedChanged();
		
		bool TreatBatchOutput(Glib::IOCondition cond);
		void OnBatchFinished();
		//@}
		
		void SetBatchFileNamesFromForm();
		
		void SaveStickySettings();
		void ReadStickySettings();
		
		bool GetRemoveGhosts() const { return this->chkRemoveGhosts->get_active(); }
		bool GetReduceNoise() const { return this->chkReduceNoise->get_active(); }
		PhotomatixImaging::ENoiseReductionType GetNoiseReductionType() const {return (PhotomatixImaging::ENoiseReductionType)this->cbxReduceNoiseTarget->get_active_row_number();}
		int GetNoiseReductionStrength() const {return this->sldReduceNoiseStrength->get_value();}
		int GetDeghostingStrength() const {return this->sldDeghosting->get_value();}
		
		std::string FindNextAvailableSubfolder(std::string& folder, std::string& prefix) const;
		
		void UpdateFilesList();
		
	public:
		BatchForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
		virtual ~BatchForm();
		static std::shared_ptr<BatchForm> GetInstance()
		{
		    BatchForm *window=0;
		    PhotomatixUtils::get_widget_derived("batch-form", window);
		    std::cout << "CREATING TEST2" << std::endl;
		    window->myRef = std::shared_ptr<BatchForm>(window);
		    return window->myRef;
		}
		
		void SetProgress(double progress);
		void ScrollLogToEnd();

		bool GetAdjust() const { return this->chkAdjust->get_active(); }
//bool GetAdvanced() const { return this->rbnAdvanced->get_active(); }
bool GetAlign() const { return this->chkAlign->get_active(); }
bool GetAlignRotation() const { return this->rbnAlignRotation->get_active(); }
bool GetAlignTranslation() const { return this->rbnAlignTranslation->get_active(); }
bool GetAuto() const { return this->chkAuto->get_active(); }
bool GetAverage() const { return this->chkAverage->get_active(); }
bool GetCrop() const { return this->chkCrop->get_active(); }
bool GetDestCustom() const { return this->rbnDestCustom->get_active(); }
bool GetDestUnderSource() const { return this->rbnDestUnderSource->get_active(); }
bool GetIntensive() const { return this->chkIntensive->get_active(); }
bool GetPerspectiveCorrection() const { return this->chkPerspectiveCorrection->get_active(); }
bool GetRealistic() const { return this->chkRealistic->get_active(); }
bool GetRemoveHDR() const { return this->chkRemoveHDR->get_active(); }
//bool GetSelect() const { return this->rbnSelect->get_active(); }
bool GetSelectionFiles() const { return this->rbnSelectionFiles->get_active(); }
bool GetSelectionFolder() const { return this->rbnSelectionFolder->get_active(); }
bool GetTM1() const { return this->chkTM1->get_active(); }
bool GetTM2() const { return this->chkTM2->get_active(); }
bool GetTM3() const { return this->chkTM3->get_active(); }
bool GetToneBalancer() const {return this->chkToneBalancer->get_active();}
double GetJpgQuality() const { return this->nudJpgQuality->get_value(); }
//int Getbobox2() const { return this->combobox2->get_active_row_number(); }
int GetBracket() const { return this->cmbBracket->get_active_row_number(); }
int GetFilter() const { return this->cmbFilter->get_active_row_number(); }
int GetHdrSavedAs() const { return this->cbxHdrSavedAs->get_active_row_number(); }
int GetSaveAs() const { return this->cbxSaveAs->get_active_row_number(); }
std::string GetDest() const { return this->txtDest->get_text(); }

void EnableDisableControls();


virtual bool on_delete_event(GdkEventAny* event)
{
  bool result = Widget::on_delete_event(event);
  std::cout << "ON DESTROY" << std::endl;
  return result;
}

	void on_hide()
	{
	  Widget::on_hide();
	  std::cout << "ON HIDE1" << std::endl;
	  //std::cout << "Use count: " << myRef.use_count() << std::endl;
	  myRef.reset();
	  //delete this;
	}


		
		//void SetHdrGenerator(PhotomatixImaging::HdrGenerator* generator) {this->hdrGenerator = generator;}
		
// 		void GenerateHDRChecked(Gtk::Object *object, gpointer user_data)
// 		{
// 		  this->btnGenerateHDR->set_sensitive(GetGenerateHDR());
// 		}
	};
}

	
#endif // BATCHFORM_H
