#include "batchsettingswindow.h"
#include "GladeUtils.h"


using namespace PhotomatixImaging;


namespace Photomatix
{
	BatchSettingsWindow::BatchSettingsWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		Gtk::Window(cobject)
	{
		this->set_resizable(false);
		this->set_default_size(400, 300);
		this->set_position(Gtk::WIN_POS_CENTER);
		this->set_title("Details Enhancer");
#ifdef Release
		this->set_icon_from_file("/usr/share/photomatix/icon.png");
#endif
		
		builder->get_widget("box-settings", this->box);
		
		builder->get_widget("btnOk", this->btnOk);
		if (this->btnOk) {
			this->btnOk->signal_clicked().connect( sigc::mem_fun(*this, &BatchSettingsWindow::OnBtnOkClicked) ); 
		}
	}

	BatchSettingsWindow::~BatchSettingsWindow()
	{

	}
	
	void BatchSettingsWindow::InitForm(int method)
	{
		switch(method) {
			case 1:
			{
				Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
				refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("details-enhancer-settings").c_str());
				refBuilder->get_widget_derived("contents", this->deSettingsControl);
				this->deSettingsControl->reparent(*this);
				
				this->box->pack_start(*this->deSettingsControl);
				this->box->reorder_child(*this->deSettingsControl, 0);
				
				this->settingsControl = this->deSettingsControl;
				
				this->set_title("Details Enhancer");
				
				break;
			}	
			case 2:
			{
				Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
				refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("tone-compressor-settings").c_str());
				refBuilder->get_widget_derived("contents", this->tcSettingsControl);
				this->tcSettingsControl->reparent(*this);
				
				this->box->pack_start(*this->tcSettingsControl);
				this->box->reorder_child(*this->tcSettingsControl, 0);
				
				this->settingsControl = this->tcSettingsControl;
				
				this->set_title("Tone Compressor");
				
				break;
			}	
			case 3:
			{
				Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
				refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("contrast-optimizer-settings").c_str());
				refBuilder->get_widget_derived("contents", this->coSettingsControl);
				this->coSettingsControl->reparent(*this);
				
				this->box->pack_start(*this->coSettingsControl);
				this->box->reorder_child(*this->coSettingsControl, 0);
				
				this->settingsControl = this->coSettingsControl;
				
				this->set_title("Contrast Optimizer");
				
				break;
			}
			case 4:
			{
				Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
				refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("tone-balancer-settings").c_str());
				refBuilder->get_widget_derived("contents", this->tbSettingsControl);
				this->tbSettingsControl->reparent(*this);
				
				this->box->pack_start(*this->tbSettingsControl);
				this->box->reorder_child(*this->tbSettingsControl, 0);
				
				this->settingsControl = this->tbSettingsControl;
				
				this->set_title("Tone Balancer");
				
				break;
			}	
			case 10:
			{
				Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
				refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("exposure-fusion-settings").c_str());
				refBuilder->get_widget_derived("contents", this->efSettingsControl);
				this->efSettingsControl->reparent(*this);
				
				this->box->pack_start(*this->efSettingsControl);
				this->box->reorder_child(*this->efSettingsControl, 0);
				
				this->settingsControl = this->efSettingsControl;
				
				this->set_title("Exposure Fusion");
				
				break;
			}
		}
	}

	void BatchSettingsWindow::OnBtnOkClicked()
	{
		this->hide();
	}
	
	XmpSettings* BatchSettingsWindow::GetCurrentSettings()
	{
		return this->settingsControl->GetCurrentSettings();
	}
}