#include "finishingtouchcontroller.h"
#include "finishingtouch.h"
#include "displayableimageform.h"
#include "../PhotomatixMisc/GladeUtils.h"
#include "../PhotomatixMisc/appproperties.h"
#include "../PhotomatixImaging/photoimage.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"
#include "../Photomatix32/Utils.h"

#include <cassert>
#include <math.h>


using namespace std;
using namespace PhotomatixImaging;


namespace Photomatix
{
	FinishingTouchController::FinishingTouchController(std::shared_ptr <PhotomatixImaging::PhotoImage> inputImage, DisplayableImageForm* parentWindow):
		parentWindow(parentWindow),
		originalPhoto(inputImage),
		isCropped(false),
		inCropMode(false),
		inDragMode(false),
		startDragPosition(ERectangleArea::Undefined),
		startDragX(-1),
		startDragY(-1),
		cropRect(0, 0, 0, 0),
		startDragCropRect(0, 0, 0, 0),
		cropRectangleImageUnits(0, 0, 0, 0),
		finalCropRectangle(0, 0, 0, 0),
		cropRatio(0.0f)
	{
	}

	FinishingTouchController::~FinishingTouchController()
	{

	}
	
	void FinishingTouchController::Init()
	{
		memset(this->saturationLUT, 0, 360 * sizeof(float));
		
		
		// Prepare preview image
		this->photoForProcessing = shared_ptr <PhotoImage>(new PhotoImage(*this->originalPhoto));
		int bps = this->photoForProcessing->GetBitsPerSample();
		if(bps == 8) {
			shared_ptr <vector <unsigned char>> tmpBuffer = shared_ptr <vector <unsigned char>> (new vector <unsigned char>((long)this->originalPhoto->GetImageBuffer()->size()));
			memcpy(tmpBuffer->data(), this->originalPhoto->GetImageBuffer()->data(), this->originalPhoto->GetImageBuffer()->size());
			
			this->photoForProcessing->SetImageBuffer(tmpBuffer);
			
		} else if(bps == 16) {
			
			shared_ptr <vector <unsigned char>> tmpBuffer = shared_ptr <vector <unsigned char>> (new vector <unsigned char>((long)this->originalPhoto->GetImageBuffer()->size() / 2));
			from16to8copy(photoForProcessing->GetWidth(), photoForProcessing->GetHeight(), photoForProcessing->GetBitsPerSample(),
				      photoForProcessing->GetWidth() * photoForProcessing->GetSamplesPerPixel() * 2, this->originalPhoto->GetImageBuffer()->data(), tmpBuffer->data());
			this->photoForProcessing->SetImageBuffer(tmpBuffer);
			this->photoForProcessing->SetBitsPerSample(8);
		}
		
		this->signal_finishingtouch_preview_updated.emit(this->photoForProcessing);
		
		
		// Create Finishing Touch window
		PhotomatixUtils::get_widget_derived("finishing-touch", this->finishingTouchWindow);
		assert(this->finishingTouchWindow);
		this->finishingTouchWindow->set_position(Gtk::WIN_POS_CENTER);
		this->finishingTouchWindow->set_keep_above(true);
		this->finishingTouchWindow->present();
		
		this->finishingTouchWindow->signal_finishingtouch_save_requested.connect(sigc::mem_fun(*this, &FinishingTouchController::OnFinishingTouchSaved));
		this->finishingTouchWindow->signal_finishingtouch_closed.connect(sigc::mem_fun(*this, &FinishingTouchController::OnFinishingTouchClosed));
		this->finishingTouchWindow->signal_settings_changed.connect(sigc::mem_fun(*this, &FinishingTouchController::OnSettingsChanged));
		this->finishingTouchWindow->signal_settings_reset.connect(sigc::mem_fun(*this, &FinishingTouchController::OnSettingsReset));
		this->finishingTouchWindow->signal_crop_applied.connect(sigc::mem_fun(*this, &FinishingTouchController::OnCropApplied));
		this->finishingTouchWindow->signal_process_changed.connect(sigc::mem_fun(*this, &FinishingTouchController::OnProcessChanged));
		this->finishingTouchWindow->signal_crop_ratio_changed.connect(sigc::mem_fun(*this, &FinishingTouchController::OnCropRatioChanged));
		
		this->finishingTouchWindow->show();
	}
	
	shared_ptr <PhotoImage> FinishingTouchController::ProcessImage(shared_ptr< PhotoImage > image)
	{
#ifdef _WIN32
		SYSTEM_INFO sysinfo;
		GetSystemInfo( &sysinfo );
		int processorsNum = sysinfo.dwNumberOfProcessors;
#else
		unsigned int processorsNum = PhotomatixMisc::AppProperties::GetNumberOfProcessors();
#endif
		// Crop image first
		if(!this->finalCropRectangle.has_zero_area()) {
			image = this->CropImage(image, finalCropRectangle.get_x(), finalCropRectangle.get_y(), finalCropRectangle.get_width(), finalCropRectangle.get_height());
		}
		
		
		if(this->hslMap.empty()) {
			long mapSize = (long)image->GetWidth() * image->GetHeight() * image->GetSamplesPerPixel();
			this->hslMap.resize(mapSize);
			
			get_hsl(processorsNum, image->GetWidth(), image->GetHeight(), image->GetSamplesPerPixel(), 1, image->GetRowStride(),
				image->GetBitsPerSample(), image->GetImageBuffer()->data(), hslMap.data());
		}
		
		shared_ptr <std::vector<unsigned char>> tmpBuffer(new vector <unsigned char>(image->GetImageBuffer()->size()));
		memcpy(tmpBuffer->data(), image->GetImageBuffer()->data(), image->GetImageBuffer()->size());
		post_adj(processorsNum, image->GetWidth(), image->GetHeight(), image->GetSamplesPerPixel(), 1, image->GetRowStride(),
			 image->GetBitsPerSample(), this->finishingTouchWindow->GetContrastCurveLUT().data(), this->saturationLUT, this->finishingTouchWindow->GetSharpenParams(),
			 hslMap.data(), tmpBuffer->data());
		
		
		
		shared_ptr <PhotoImage> resultImage = shared_ptr <PhotoImage>(new PhotoImage(*image));
		resultImage->SetImageBuffer(tmpBuffer);
		//this->signal_finishingtouch_preview_updated(image);
		//
		
		return resultImage;
	}
	
	void FinishingTouchController::OnProcessChanged(int index)
	{
		if(index == 2) {
			this->StartCropMode();
		} else {
			this->StopCropMode();
		}
	}
	
	void FinishingTouchController::OnCropRatioChanged(float newRatio)
	{
		if (!this->cropRectangleImageUnits.has_zero_area() && !FloatEquals(newRatio, 0))
		{
			int cropW = this->cropRectangleImageUnits.get_width();
			int cropH = this->cropRectangleImageUnits.get_height();
			int cropY = this->cropRectangleImageUnits.get_y();
			int cropX = this->cropRectangleImageUnits.get_x();

			bool isPortrait = cropH > cropW;
			cropH = (int)(cropW * newRatio);
			if (cropY + cropH > this->photoForProcessing->GetWidth())
			{
				cropY = this->photoForProcessing->GetHeight() - cropH;
				if (cropY < 0)
				{
					cropH = cropH + cropY;
					cropY = 0;
					cropW = (int)(cropH / newRatio);
				}
			}
			this->RefreshCropRect(Gdk::Rectangle(cropX, cropY, cropW, cropH));

		}
		this->cropRatio = newRatio;
	}
	
	void FinishingTouchController::RefreshCropRect(Gdk::Rectangle newCropRect)
	{
		// Return, as crop coordinates are invalid
		if (newCropRect.get_x() < 0 || newCropRect.get_y() < 0 || newCropRect.get_x() + newCropRect.get_width() > this->photoForProcessing->GetWidth() || newCropRect.get_y() + newCropRect.get_height() > this->photoForProcessing->GetHeight())
			return;
		
		this->cropRectangleImageUnits = newCropRect;
		float factor = this->parentWindow->GetZoomFactor();
		this->cropRect = Gdk::Rectangle(newCropRect.get_x() * factor, newCropRect.get_y() * factor, newCropRect.get_width() * factor, newCropRect.get_height() * factor);// newCropRect;//.ctrlImage.PixelRectToScreen(newCropRect);
		this->parentWindow->ShowCropRectangle(newCropRect);
	}

	void FinishingTouchController::OnCropApplied()
	{
		std::shared_ptr <PhotomatixImaging::PhotoImage> curImage = this->photoForProcessing;
		
		int cropRectX = cropRectangleImageUnits.get_x(), cropRectY = cropRectangleImageUnits.get_y();
		int cropRectWidth = cropRectangleImageUnits.get_width(), cropRectHeight = cropRectangleImageUnits.get_height();
		
		cropRectX += finalCropRectangle.get_x();
		cropRectY += finalCropRectangle.get_y();
		

		if (cropRectX + cropRectWidth > curImage->GetWidth())
			cropRectWidth = max <int>(0, curImage->GetWidth() - cropRectX);
		if (cropRectY + cropRectHeight > curImage->GetHeight())
			cropRectHeight = max <int>(0, curImage->GetHeight() - cropRectY);
		
		// No need to crop
		if ((cropRectWidth == 0 && cropRectHeight == 0) || (cropRectWidth == curImage->GetWidth() && cropRectHeight == curImage->GetHeight()))
			return;
		
		
		// Clear HSL map as it needs to be recalculated since image size changed
		this->hslMap.clear();
		
		this->finalCropRectangle = Gdk::Rectangle(cropRectX, cropRectY, cropRectWidth, cropRectHeight);
		
		shared_ptr<PhotoImage> outputImage = this->ProcessImage(this->photoForProcessing);
		
		this->signal_finishingtouch_preview_updated.emit(outputImage);
		
		
		this->isCropped = true;
		this->finishingTouchWindow->SetIsCropped(isCropped);
	}
	
	shared_ptr <PhotoImage> FinishingTouchController::CropImage(shared_ptr<PhotoImage> image, int cropRectX, int cropRectY, int cropRectWidth, int cropRectHeight)
	{
		shared_ptr <PhotoImage> resultImage = shared_ptr <PhotoImage>(new PhotoImage(*image));
		
		int outStride = PhotoImage::CalculateRowStride(cropRectWidth, image->GetSamplesPerPixel());
		
		size_t bufferSize = (size_t)outStride * cropRectHeight * image->GetBytesPerPixel();
		shared_ptr <vector <unsigned char>> outBuff(new vector <unsigned char>(bufferSize));
		
		crop_ldr(image->GetBitsPerSample(), image->GetRowStride(), cropRectWidth, cropRectHeight, outStride, image->GetSamplesPerPixel(),
				cropRectX, cropRectY, image->GetImageBuffer()->data(), outBuff->data());
		

		resultImage->SetWidth(cropRectWidth);
		resultImage->SetHeight(cropRectHeight);
		resultImage->SetImageBuffer(outBuff);
		
		
		this->cropRect = Gdk::Rectangle(0, 0, 0, 0);
		this->cropRectangleImageUnits = Gdk::Rectangle(0, 0, 0, 0);


		isCropped = true;
		this->FireCropStateChanged();
		
		return resultImage;
	}
	
	void FinishingTouchController::OnSettingsChanged()
	{
		shared_ptr<PhotoImage> outputImage = this->ProcessImage(this->photoForProcessing);
		
		this->signal_finishingtouch_preview_updated.emit(outputImage);
	}
	
	void FinishingTouchController::OnSettingsReset(bool resetCrop)
	{
		this->hslMap.clear();
		
		if(resetCrop) {
			this->isCropped = false;
			this->cropRect = Gdk::Rectangle(0, 0, 0, 0);
			this->cropRectangleImageUnits = Gdk::Rectangle(0, 0, 0, 0);
			this->finalCropRectangle = Gdk::Rectangle(0, 0, 0, 0);
			
			this->finishingTouchWindow->SetIsCropped(isCropped);
		}
		
		shared_ptr<PhotoImage> outputImage = this->ProcessImage(this->photoForProcessing);
		
		this->signal_finishingtouch_preview_updated.emit(outputImage);
	}
	
	void FinishingTouchController::OnFinishingTouchSaved()
	{
		this->parentWindow->ShowCropRectangle(Gdk::Rectangle(0, 0, 0, 0));
		
		if(this->finishingTouchWindow->HasAdjustments() || this->isCropped) {
			this->hslMap.clear();
			shared_ptr <PhotoImage> resultImage = this->ProcessImage(this->originalPhoto);
			
			this->signal_finishingtouch_finished.emit(resultImage);
		} else {
			this->signal_finishingtouch_finished.emit(this->originalPhoto);
		}
	}

	void FinishingTouchController::OnFinishingTouchClosed()
	{		
		this->parentWindow->ShowCropRectangle(Gdk::Rectangle(0, 0, 0, 0));
		
		if(this->finishingTouchWindow->HasAdjustments() || this->isCropped) {
			this->hslMap.clear();
			shared_ptr <PhotoImage> resultImage = this->ProcessImage(this->originalPhoto);
			this->signal_finishingtouch_closed.emit(resultImage);
		} else {
			this->signal_finishingtouch_closed.emit(this->originalPhoto);
		}
		
		this->parentWindow->SetSaveMenuEnability(true);
	}
	
	ERectangleArea FinishingTouchController::GetMousePositionInCropBox(int pointerX, int pointerY)
	{
		ERectangleArea mousePos = ERectangleArea::Outside;

		if (this->cropRect.has_zero_area()) return mousePos; //If crop rectangle not shown then we're obviously outside of it.
		int cursorSize = Gdk::Display::get_default()->get_default_cursor_size();
		int dx = cursorSize / 2, dy = cursorSize / 2;
		
		int x = this->cropRect.get_x();
		int right = x + this->cropRect.get_width();
		int y = this->cropRect.get_y();
		int bottom = y + this->cropRect.get_height();
		
		Gdk::Rectangle upperLeft(x - dx, y - dy, dx * 2, dy * 2);
		Gdk::Rectangle upperCenter(x + dx, y - dy, cropRect.get_width() - dx * 2, dy * 2);
		Gdk::Rectangle upperRight(right - dx, y - dy, dx * 2, dy * 2);
		Gdk::Rectangle centerLeft(x - dx, y + dy, dx * 2, cropRect.get_height() - dy * 2);
		Gdk::Rectangle centerRight(right - dx, y + dy, dx * 2, cropRect.get_height() - dy * 2);
		Gdk::Rectangle bottomLeft(x - dx, bottom - dy, dx * 2, dy * 2);
		Gdk::Rectangle bottomCenter(x + dx, bottom - dy, cropRect.get_width() - dx * 2, dy * 2);
		Gdk::Rectangle bottomRight(right - dx, bottom - dy, dx * 2, dy * 2);
		Gdk::Rectangle innerRect(x + dx, y + dy, cropRect.get_width() - dx * 2, cropRect.get_height() - dy * 2);

		std::vector <Gdk::Rectangle> posRecs = { upperLeft, upperCenter, upperRight, centerLeft, centerRight, bottomLeft, bottomCenter, bottomRight, innerRect };
		std::vector <ERectangleArea> posAlign = { ERectangleArea::TopLeft, ERectangleArea::TopCenter, ERectangleArea::TopRight, ERectangleArea::MiddleLeft, ERectangleArea::MiddleRight, ERectangleArea::BottomLeft, ERectangleArea::BottomCenter, ERectangleArea::BottomRight, ERectangleArea::MiddleCenter };

		for (int i = 0; i < posRecs.size(); i++)
		{
			Gdk::Rectangle curRect = posRecs[i];
			// Does current rectangle contain mouse point?
			if (pointerX >= curRect.get_x() && pointerX < curRect.get_x() + curRect.get_width() &&
				pointerY >= curRect.get_y() && pointerY < curRect.get_y() + curRect.get_height())
			{
				mousePos = posAlign[i];
				break;
			}
		}
		
		return mousePos;
	}
	
	Gdk::Rectangle FinishingTouchController::GetImageUnitsRectangle(Gdk::Rectangle screenRect)
	{
		//return this.ctrlImage.GetImageRectangle(screenRect.Location, new Size(screenRect.Width, screenRect.Height));
		Gdk::Rectangle rect;
		
		int imgWidth = this->parentWindow->GetPhoto()->GetWidth();
		int imgHeight = this->parentWindow->GetPhoto()->GetHeight();

		int imX = (int)((screenRect.get_x()) / this->parentWindow->GetZoomFactor());
		int imY = (int)((screenRect.get_y()) / this->parentWindow->GetZoomFactor());
		if (imX > 0 && imX < imgWidth) rect.set_x(imX);
		if (imY > 0 && imY < imgHeight) rect.set_y(imY);


		rect.set_width(screenRect.get_width());
		rect.set_height(screenRect.get_height());
		
		rect.set_width((int)(rect.get_width() / this->parentWindow->GetZoomFactor()));
		rect.set_height((int)(rect.get_height() / this->parentWindow->GetZoomFactor()));
		
		if (imX < 0) rect.set_width(rect.get_width() - fabs(imX));
		if (imY < 0) rect.set_height(rect.get_height() - fabs(imY));
		if (rect.get_x() + rect.get_width() >= imgWidth) rect.set_width(rect.get_width() - (rect.get_x() + rect.get_width() - imgWidth));
		if (rect.get_y() + rect.get_height() >= imgHeight) rect.set_height(rect.get_height() - (rect.get_y() + rect.get_height() - imgHeight));
		

		return rect;
	}
	
	void FinishingTouchController::OnImageMouseMoved(GdkEventMotion* evt)
	{
		if (!this->inCropMode) return;
		
		const int MinCropRectWidth = 10;
		const int MinCropRectHeight = 10;
		
		int mouseX = evt->x;
		int mouseY = evt->y;
		
		if (this->inDragMode) {
			Gdk::Rectangle imgUnitRect = this->GetImageUnitsRectangle(Gdk::Rectangle(this->startDragX, this->startDragY, mouseX - this->startDragX, mouseY - this->startDragY));
			float x = imgUnitRect.get_x(); float y = imgUnitRect.get_y();
			float w = imgUnitRect.get_width(); float h = imgUnitRect.get_height();

			if (this->startDragPosition == ERectangleArea::Outside) {
				if (fabs(w) > MinCropRectWidth && fabs(h) > MinCropRectHeight) {
					// For negative width and height we need to fix x and y location
					if (w < 0) {
						w = fabs(w); x -= w;
						if (x < 0) {
							w += x;
							x = 0;
						}
					}
					if (h < 0) {
						h = fabs(h); y -= h;
						if (y < 0) {
							h += y;
							y = 0;
						}
					}
					if (!FloatEquals(this->cropRatio, 0.0f)) {
						h = (w * this->cropRatio);
					}
					this->RefreshCropRect(Gdk::Rectangle(x, y, w, h));
				}
				else {
					this->cropRect = this->cropRectangleImageUnits = Gdk::Rectangle(0, 0, 0, 0);
					//this->ctrlImage.HideRectangleBox();
				}
			}
			else {
				Gdk::Rectangle imgUnitStartRect = this->startDragCropRect;
				float cx = imgUnitStartRect.get_x();
				float cy = imgUnitStartRect.get_y();
				float cw = imgUnitStartRect.get_width();
				float ch = imgUnitStartRect.get_height();

				float dx = -imgUnitRect.get_width();
				float dy = -imgUnitRect.get_height();

				float lockedH = dx;
				float lockedW = dy;
				if (!FloatEquals(this->cropRatio, 0.0f)) {
					lockedH = (dx * this->cropRatio);
					lockedW = (dy / this->cropRatio);
				}
				float dw = dx, dh = dy;

				float dirx = 1, diry = 1;

				if (this->startDragPosition == ERectangleArea::MiddleCenter) { dw = 0; dh = 0; dirx = -1; diry = -1; } //We just move the box around so only Location changes
				if (this->startDragPosition == ERectangleArea::MiddleLeft || this->startDragPosition == ERectangleArea::MiddleRight)
					dy = dh = 0;  //Since we're sizing horizontally only, ignore any mouse delta along Y-axis
				if (this->startDragPosition == ERectangleArea::TopCenter || this->startDragPosition == ERectangleArea::BottomCenter)
					dx = dw = 0; //Since we're sizing vertically only, ignore any mouse delta along X-axis
				
				if (this->startDragPosition == ERectangleArea::TopRight || this->startDragPosition == ERectangleArea::MiddleRight || this->startDragPosition == ERectangleArea::BottomRight)
						dx = 0; //When sizing from the right part of the rectangle we just increase the size so we don't change the location.
				if (this->startDragPosition == ERectangleArea::BottomLeft || this->startDragPosition == ERectangleArea::BottomCenter || this->startDragPosition == ERectangleArea::BottomRight) 
					dy = 0; //When sizing from the bottom part of the rectangle we just increase the size so we don't change the location.

				if (this->startDragPosition == ERectangleArea::TopLeft || this->startDragPosition == ERectangleArea::MiddleLeft || this->startDragPosition == ERectangleArea::BottomLeft)
					dirx = -1; //If we're sizing the left side, inverse the X-axis direction
				if (this->startDragPosition == ERectangleArea::TopLeft || this->startDragPosition == ERectangleArea::TopCenter || this->startDragPosition == ERectangleArea::TopRight)
					diry = -1; //If we're sizing the bottom side, inverse the Y-axis direction


				if (!FloatEquals(this->cropRatio, 0.0f))
				{
					//Adjust width or height so that it is proportional to the ratio of the other dimension dragged, when a corner is resized. 
					if (this->startDragPosition == ERectangleArea::BottomRight)
					{
						dw = lockedW;
					}
					else if (this->startDragPosition == ERectangleArea::TopRight)
					{
						dw = lockedW;
						dirx = -1;
					}
					else if (this->startDragPosition == ERectangleArea::BottomLeft)
					{
						dx = dw = lockedW;
						dirx = 1;
						diry = 1;
					}
					else if (this->startDragPosition == ERectangleArea::TopLeft)
					{
						dx = dw = lockedW;
						diry = -1;
					}
				}

				//Calculate corrdinates of new crop rectangle:
				float xf = cx + dirx * dx;
				float yf = cy + diry * dy;
				float wf = cw + -dirx * dw;
				float hf = ch + -diry * dh;

				if (!FloatEquals(this->cropRatio, 0.0f)) {
					//Adjust width or height so that it is evenly based on ratio, when one of the border is resized.
					if (this->startDragPosition == ERectangleArea::MiddleLeft || this->startDragPosition == ERectangleArea::MiddleRight) {
						
						yf -= -dirx*lockedH/2;
						hf += -dirx*lockedH;
					}
					else if (this->startDragPosition == ERectangleArea::TopCenter || this->startDragPosition == ERectangleArea::BottomCenter) {
						xf -= -diry * lockedW / 2;
						wf += -diry * lockedW;
					}
				}					
				x = xf; y = yf; w = wf; h = hf;
				if (x < 0) x = 0;
				if (y < 0) y = 0;
				if (x + w > this->photoForProcessing->GetWidth()) x = this->photoForProcessing->GetWidth() - w;
				if (y + h > this->photoForProcessing->GetHeight()) y = this->photoForProcessing->GetHeight() - h;
				
				this->RefreshCropRect(Gdk::Rectangle(x, y, w, h));
			}
			this->FireCropStateChanged();
		}
		else
		{
			int cursorSize = Gdk::Display::get_default()->get_default_cursor_size();
			int dx = cursorSize, dy = cursorSize;
			//int dx = this->ctrlImage.Cursor.Size.Width, dy = this->ctrlImage.Cursor.Size.Height;

			vector <ERectangleArea> posAlign = { ERectangleArea::TopLeft, ERectangleArea::TopCenter, ERectangleArea::TopRight, ERectangleArea::MiddleLeft, ERectangleArea::MiddleCenter, ERectangleArea::MiddleRight, ERectangleArea::BottomLeft, ERectangleArea::BottomCenter, ERectangleArea::BottomRight, ERectangleArea::Outside };
			//Cursor[] posCurs = new Cursor[] { Cursors.SizeNWSE, Cursors.SizeNS, Cursors.SizeNESW, Cursors.SizeWE, Cursors.Hand, Cursors.SizeWE, Cursors.SizeNESW, Cursors.SizeNS, Cursors.SizeNWSE, Cursors.Default };

			//System.Collections.Generic.Dictionary<ERectangleArea, Cursor> pos2Cursor = new System.Collections.Generic.Dictionary<ERectangleArea, Cursor>();
			/*for (int i = 0; i < posAlign.size(); i++)
			{
				pos2Cursor.Add(posAlign[i], posCurs[i]);
			}
			this->ctrlImage.Cursor = pos2Cursor[GetMousePositionInCropBox(e.Location)];*/
		}
	}
	
	void FinishingTouchController::FireCropStateChanged()
	{
		bool canCrop = !cropRect.has_zero_area() && this->inCropMode && !this->inDragMode;
		this->finishingTouchWindow->SetCropControlEnability(canCrop);
	}

	void FinishingTouchController::OnImageMouseDown(GdkEventButton* evt)
	{
		if (!this->inCropMode) return;
		int x = evt->x, y = evt->y;
		this->startDragPosition = this->GetMousePositionInCropBox(x, y);
		
		if (this->inCropMode || this->startDragPosition != ERectangleArea::Outside)
		{
			this->inDragMode = true;
			this->startDragX = x;
			this->startDragY = y;
			this->startDragCropRect = this->cropRectangleImageUnits;
		}
	}
	
	void FinishingTouchController::OnImageMouseUp(GdkEventButton* evt)
	{
		if (!this->inCropMode) return;
		this->inDragMode = false;
		this->FireCropStateChanged();
	}
	
	void FinishingTouchController::StartCropMode()
	{
		cropRect = Gdk::Rectangle(0, 0, 0, 0);
		cropRectangleImageUnits = Gdk::Rectangle(0, 0, 0, 0);
		
		this->inDragMode = false;
		
		this->inCropMode = true;
		this->FireCropStateChanged();
	}

	void FinishingTouchController::StopCropMode()
	{
		this->inCropMode = false;
		this->parentWindow->ShowCropRectangle(Gdk::Rectangle(0, 0, 0, 0));
		this->FireCropStateChanged();
	}
	
	bool FinishingTouchController::FloatEquals(float val1, float val2)
	{
		return fabs(val2 - val1) < .0000001f; 
	}

}
