#ifndef FINISHINGTOUCHCONTROLLER_H
#define FINISHINGTOUCHCONTROLLER_H

#include <memory>
#include <vector>
#include "../PhotomatixMisc/photomatixconstants.h"
#include <sigc++/connection.h>
#include <gtkmm.h>
#include "presetthumbnail.h"


namespace PhotomatixImaging
{
	class PhotomatixImaging;
}

namespace Photomatix
{
	// Forward declarations
	class FinishingTouch;
	class DisplayableImageForm;
	
	
	enum ERectangleArea
	{
		Undefined = -1,
		TopLeft = 1,
		TopCenter = 2,
		TopRight = 4,
		MiddleLeft = 16,
		MiddleCenter = 32,
		MiddleRight = 64,
		BottomLeft = 256,
		BottomCenter = 512,
		BottomRight = 1024,
		Outside = 2048
	};
	
	/** Controller handling Finishing Touch adjustments.
	 */
	class FinishingTouchController
	{
	protected:
		DisplayableImageForm* parentWindow;
		FinishingTouch* finishingTouchWindow;				///< Finishing Touch window
		std::shared_ptr <PhotomatixImaging::PhotoImage> photoForProcessing;
		std::shared_ptr <PhotomatixImaging::PhotoImage> originalPhoto;
		
		std::vector <unsigned char> hslMap;
		
		Gdk::Rectangle cropRect;
		Gdk::Rectangle startDragCropRect;
		Gdk::Rectangle cropRectangleImageUnits;
		
		Gdk::Rectangle finalCropRectangle;
		
		ERectangleArea startDragPosition;
		int startDragX;
		int startDragY;
		
		float cropRatio;
	
		
		
		float saturationLUT[360];
		bool isCropped;
		
		bool inCropMode;
		bool inDragMode;
		
		
		void OnFinishingTouchSaved();
		void OnFinishingTouchClosed();
		void OnProcessChanged(int index);
		void OnSettingsChanged();
		void OnSettingsReset(bool resetCrop);
		void OnCropApplied();
		void OnCropRatioChanged(float newRatio);
		
		
		void RefreshCropRect(Gdk::Rectangle newCropRect);
		Gdk::Rectangle GetImageUnitsRectangle(Gdk::Rectangle screenRect);
		
		void StartCropMode();
		void StopCropMode();
		
		bool FloatEquals(float val1, float val2);
		
		void FireCropStateChanged();
		
		ERectangleArea GetMousePositionInCropBox(int x, int y);
		
		
		std::shared_ptr <PhotomatixImaging::PhotoImage> CropImage(std::shared_ptr<PhotomatixImaging::PhotoImage> image, int cropRectX, int cropRectY, int cropRectWidth, int cropRectHeight);
		
		std::shared_ptr <PhotomatixImaging::PhotoImage> ProcessImage(std::shared_ptr<PhotomatixImaging::PhotoImage> image);
	
	public:
		FinishingTouchController(std::shared_ptr <PhotomatixImaging::PhotoImage> inputImage, DisplayableImageForm* parentWindow);
		~FinishingTouchController();
		
		void Init();
		
		void OnImageMouseMoved(GdkEventMotion* evt);
		void OnImageMouseDown(GdkEventButton* evt);
		void OnImageMouseUp(GdkEventButton* evt);
		
		sigc::signal <void, std::shared_ptr<PhotomatixImaging::PhotoImage> > signal_finishingtouch_finished;
		sigc::signal <void, std::shared_ptr<PhotomatixImaging::PhotoImage> > signal_finishingtouch_closed;
		sigc::signal <void, std::shared_ptr<PhotomatixImaging::PhotoImage> > signal_finishingtouch_preview_updated;
	};
}

#endif // PREVIEWCONTROLLER_H
