#ifndef HDRCONTROLLER_H
#define HDRCONTROLLER_H

#include "displayableimageform.h"
#include <memory>


namespace PhotomatixImaging
{
	class HdrGenerator;
}

namespace Photomatix
{
	class LoadBracketedDialog;
	class PreprocessingOptionsForm;
	class ProgressForm;
	
	/**
	 * Controller for merging bracketed photos to HDR.
	 */
	class HdrController
	{
	private:
		LoadBracketedDialog* loadBracketed;
		PreprocessingOptionsForm* preprocessingOptions;
		ProgressForm* progressForm;
		DisplayableImageForm* imageWindow;

		std::shared_ptr <PhotomatixImaging::HdrGenerator> hdrGenerator;		///< HDR generator instance
		
		
		//@{
		/** Event handlers */
		void OnLoadBracketedOk();
		void GenerateHDR();
		void OnGeneratingHDRDone();
		void OnGeneratingHDRCanceled();
		//@}
		
	public:
		HdrController();
		virtual ~HdrController();
		
		//@{
		/** Initializes controller. Needs to be called before using any other method. */
		void Init();
		void Init(const std::vector <std::string>& files);
		//@}
		
		/// Signal emitted when merging to HDR finishes.
		sigc::signal<void> signal_hdr_generated;
		
		std::shared_ptr <PhotomatixImaging::HdrGenerator> GetHdrGenerator() {return this->hdrGenerator;}
		
		void SetImageWindow(DisplayableImageForm* form) {this->imageWindow = form;}
	};
}

#endif // HDRCONTROLLER_H
