#ifndef LOAD_BRACKETED_DIALOG_H
#define LOAD_BRACKETED_DIALOG_H

#include <gtkmm.h>


namespace Photomatix
{
	/**
	* The dialog box for selecting and loading a set of bracketed images.
	*******************************************************************************/
	class LoadBracketedDialog : public Gtk::Dialog
	{
	protected:

		Gtk::Button* btnOk;
		Gtk::TreeView* m_treeview;

		/** glade builder with which to access widgets */
		Glib::RefPtr<Gtk::Builder> m_builder;

		/** file list data model. This data model is instantiated in the
		* class constructor and attached to the treeview laid out in
		* glade
		*/
		Glib::RefPtr<Gtk::ListStore> m_filelist;

		/**
		* Data model for our file list.  The data model for this file
		* list contains two fields: a full path to the location of the
		* file, and the short filename that is displayed to the user
		*/
		class FileListModelColumns : public Gtk::TreeModel::ColumnRecord
		{
		public:

			FileListModelColumns()
			{ add(m_col_short_name); add(m_col_full_path);}

			/** short name that is displayed to the user */
			Gtk::TreeModelColumn<Glib::ustring> m_col_short_name;
			/** full path that is stored hidden to access the file */
			Gtk::TreeModelColumn<Glib::ustring> m_col_full_path;
		} m_columns;

		//@{
		/** Signal Handlers */
		void on_button_ok();
		void on_button_cancel();
		void on_button_add();
		void on_button_remove();
		bool on_window_delete(GdkEventAny *event);
		void on_treeview_dnd_data_received(
			const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
			const Gtk::SelectionData& selection_data, guint info, guint time);
		//@}
		
		void ReadStickySettings();
		void SaveStickySettings();
		
	public:
		/**
		 * This dialog laid out in glade and accessed as a derived type
		 */
		LoadBracketedDialog(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
		virtual ~LoadBracketedDialog();
		
		/* given path to a file, add it to the list of files to open */
		void add_path_to_list(const std::string& full_path);
		
		/// Returns list of the file names
		std::vector <std::string> GetFileNames();
		
		/// Signal emitted when the ok button was clicked.
		sigc::signal<void> signal_ok;

	};
}


#endif //LOAD_BRACKETED_DIALOG_H
