#include <gtkmm.h>
#include <iostream>
#include <cassert>

#include <locale.h>
#include <libintl.h>

#include "GladeUtils.h"
#include "processphotocontroller.h"
#include "trialform.h"
#include "../PhotomatixMisc/appproperties.h"
#include "localtext.h"
#include "dbgprintf.h"

using namespace Photomatix;
using namespace PhotomatixMisc;
using namespace std;


/*void sighandler(int signal, siginfo_t *si, void *arg)
{
	printf("Handler executed for signal");
	//context->uc_mcontext.gregs[REG_RAX] = &n;
}*/


int main (int argc, char **argv)
{
	setlocale(LC_ALL, "POSIX");
	
	bindtextdomain("photomatix", "/usr/share/locale");
        textdomain("photomatix");
	
	locale l("");
	cout<<"Locale by C++: "<<l.name()<<endl;
	cout << "LC_ALL: " << setlocale(LC_ALL, NULL) << endl;
	cout << "LC_CTYPE: " << setlocale(LC_CTYPE, NULL) << endl;
	
	string localeName = l.name();
	cout << localeName<< endl;
	string type = "LC_CTYPE";
	if(localeName.find(type) != std::string::npos) {
		int index = localeName.find(type) + type.length() + 1;
		localeName = localeName.substr(index, localeName.length() - index);
		int indexEnd = localeName.find(";");
		localeName = localeName.substr(0, indexEnd);
	}
#ifdef Release
	freopen("/dev/null", "w", stderr);
	freopen("/dev/null", "w", stdout);
	
	if(localeName.find("de") != std::string::npos)
		LocalText::LoadResourceFile("/usr/share/photomatix/DynamicText.de.resx");
	else
		LocalText::LoadResourceFile("/usr/share/photomatix/DynamicText.resx");
#else
	if(localeName.find("de") != std::string::npos)
		LocalText::LoadResourceFile("DynamicText.de.resx");
	else
		LocalText::LoadResourceFile("DynamicText.resx");
#endif
	
	dbgprintf("Photomatix for Linux started\n");
	
	Gtk::Main kit(argc, argv);
	
	if(!AppProperties::Init()) {
		Gtk::MessageDialog dialog("Properties file not found", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK);
		dialog.set_secondary_text("Make sure it was properly installed.");
		
		int result = dialog.run();
		return 0;
	}
	
	Glib::RefPtr<Gtk::CssProvider> m_refCssProvider;
	m_refCssProvider = Gtk::CssProvider::create();
	auto refStyleContext = Gtk::StyleContext::create();//this->get_style_context();//get_style_context();
	//refStyleContext->add_provider(m_refCssProvider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	refStyleContext->add_provider_for_screen(Gdk::Screen::get_default(), m_refCssProvider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	//m_refCssProvider->load_from_data("GtkWidget { padding: 0; }");
#ifdef Release
	m_refCssProvider->load_from_path("/usr/share/photomatix/photomatix.css");
#else
	m_refCssProvider->load_from_path("photomatix.css");
#endif

	// Create ProcessPhotoController and show Image Window which will serve as the main window
	ProcessPhotoController controller;
	controller.ShowImageWindow();
	
	// Show Trial form if necessary
	if(!AppProperties::IsRegistered()) {
		TrialForm* trialForm;
		PhotomatixUtils::get_widget_derived("trial-form", trialForm);
		int result = trialForm->run();
	}
	
	kit.run();	

	return 0;
}
