#include "openimageform.h"

#include "appproperties.h"
#include "../PhotomatixMisc/localtext.h"

#include <gtkmm/stock.h>
#include <vector>
#include <string>

using namespace std;
using namespace PhotomatixMisc;


namespace Photomatix
{
	OpenImageForm::OpenImageForm():
		FileChooserDialog(LocalText::GetOpenDialogTitle(), Gtk::FILE_CHOOSER_ACTION_OPEN)
	{
		this->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		this->add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
		
		vector <string> jpeg = {"*.jpg", "*.jpe", "*.jpeg"};
		vector <string> tiff = {"*.tif", "*.tiff"};
		
		vector <string> radianceRGBE = {"*.hdr"};
		
		vector <string> canonRAW = {"*.crw", "*.cr2"};
		vector <string> nikonRAW = {"*.nef", "*.nrw"};
		vector <string> fujiRAW = {"*.raf"};
		vector <string> olympusRAW = {"*.orf"};
		vector <string> sonyRAW = {"*.srf", "*.sr2", "*.arw"};
		vector <string> pentaxRAW = {"*.pef"};
		vector <string> minoltaRAW = {"*.mrw"};
		vector <string> kodakRAW = {"*.dcr", "*.kdc"};
		vector <string> sigmaRAW = {"*.x3f"};
		vector <string> panasonicRAW = {"*.raw", "*.rw2", "*.rwl"};
		vector <string> adobeRAW = {"*.dng"};
		vector <string> hasselbladRAW = {"*.fff", "*.3fr"};
		
		vector <string> allImages;
		allImages.insert(allImages.begin(), canonRAW.begin(), canonRAW.end());
		allImages.insert(allImages.begin(), nikonRAW.begin(), nikonRAW.end());
		allImages.insert(allImages.begin(), fujiRAW.begin(), fujiRAW.end());
		allImages.insert(allImages.begin(), olympusRAW.begin(), olympusRAW.end());
		allImages.insert(allImages.begin(), sonyRAW.begin(), sonyRAW.end());
		allImages.insert(allImages.begin(), pentaxRAW.begin(), pentaxRAW.end());
		allImages.insert(allImages.begin(), minoltaRAW.begin(), minoltaRAW.end());
		allImages.insert(allImages.begin(), kodakRAW.begin(), kodakRAW.end());
		allImages.insert(allImages.begin(), sigmaRAW.begin(), sigmaRAW.end());
		allImages.insert(allImages.begin(), panasonicRAW.begin(), panasonicRAW.end());
		allImages.insert(allImages.begin(), adobeRAW.begin(), adobeRAW.end());
		allImages.insert(allImages.begin(), hasselbladRAW.begin(), hasselbladRAW.end());
		allImages.insert(allImages.begin(), jpeg.begin(), jpeg.end());
		allImages.insert(allImages.begin(), tiff.begin(), tiff.end());
		allImages.insert(allImages.begin(), radianceRGBE.begin(), radianceRGBE.end());
		
		this->AddFilter(LocalText::GetTextAllImagesFilter(), allImages);
		
		this->AddFilter("JPEG", jpeg);
		this->AddFilter("TIFF", tiff);
		
		this->AddFilter("Radiance RGBE <HDR>", radianceRGBE);
		
		this->AddFilter("Canon RAWs <CRW, CR2>", canonRAW);
		this->AddFilter("Nikon RAWs <NEF, NRW>", nikonRAW);
		this->AddFilter("Fuji RAWs <RAF>", fujiRAW);
		this->AddFilter("Olympus RAWs <ORF>", olympusRAW);
		this->AddFilter("Sony RAWs <SRF, SR2, ARW>", sonyRAW);
		this->AddFilter("Pentax RAWs <PEF>", pentaxRAW);
		this->AddFilter("Minolta RAWs <MRW>", minoltaRAW);
		this->AddFilter("Kodak RAWs <DCR, KDC>", kodakRAW);
		this->AddFilter("Sigma RAWs <X3F>", sigmaRAW);
		this->AddFilter("Panasonic RAWs <RAW, RW2, RWL>", panasonicRAW);
		this->AddFilter("Digital Negative <DNG>", adobeRAW);
		this->AddFilter("Hasselblad RAWs <FFF, 3FR>", hasselbladRAW);
		
		string prevFolder = this->get_current_folder();
		this->set_current_folder(PhotomatixMisc::AppProperties::GetLastOpenDirectory());
		if(PhotomatixMisc::AppProperties::GetLastOpenDirectory().empty())
			this->set_current_folder(prevFolder);
	}
	
	OpenImageForm::~OpenImageForm()
	{
		
	}
	
	void OpenImageForm::AddFilter(const std::string& filterName, const vector<string>& patterns)
	{
		Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
		filter->set_name(filterName);
		for(uint i = 0; i < patterns.size(); ++i)
		{
			stringstream pattern;
			for(unsigned char c : patterns[i])
			{
				if(c >= 97 && c <= 122)
					pattern << "[" << (char)(c - 32) << c << "]";
				else
					pattern << c;
			}
			filter->add_pattern(pattern.str());
		}
		this->add_filter(filter);
	}

}
