#include "preprocessingoptionsform.h"
#include "displayableimageform.h"
#include "../PhotomatixMisc/appproperties.h"

#include <iostream>
#include <iomanip>

using namespace std;
using namespace PhotomatixMisc;


namespace Photomatix
{
	PreprocessingOptionsForm::PreprocessingOptionsForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		Gtk::Dialog(cobject)
	{
		this->set_resizable(false);
		
		whiteBalanceSettings[0] = EWhiteBalance::WBAsShot;
		whiteBalanceSettings[1] = EWhiteBalance::WBAuto;
		whiteBalanceSettings[2] = EWhiteBalance::WBDaylight;
		whiteBalanceSettings[3] = EWhiteBalance::WBCloudy;
		whiteBalanceSettings[4] = EWhiteBalance::WBShade;
		whiteBalanceSettings[5] = EWhiteBalance::WBTungsten;
		whiteBalanceSettings[6] = EWhiteBalance::WBFluorescent;
		whiteBalanceSettings[7] = EWhiteBalance::WBCustom;
		
		
		Gtk::Button* button;
		
		////////////////////////////////////////////
		// Bind signals to controls
		////////////////////////////////////////////
		
		// General controls
		builder->get_widget("btnOK", button);
		if (button)
			button->signal_clicked().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnBtnOkClicked)); 
		
		/*refStyleContext = button->get_style_context();//get_style_context();
	refStyleContext->add_provider(m_refCssProvider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	//m_refCssProvider->load_from_data("GtkWidget { padding: 0; }");
	m_refCssProvider->load_from_path("photomatix.css");*/
		
		builder->get_widget("btnCancel", button);
		if (button)
			button->signal_clicked().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnBtnCancelClicked));
		
		// Controls related to image alignment
		builder->get_widget("chkAlign", this->chkAlignImages);
		if (this->chkAlignImages)
			this->chkAlignImages->signal_toggled().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnAlignImagesCheckedChanged));
		builder->get_widget("chkCrop", this->chkCrop);
		builder->get_widget("rbnHorizVertShifts", this->rbnAlignShifts);
		builder->get_widget("rbnMatchingFeatures", this->rbnAlignFeatures);
		builder->get_widget("chkPerspective", this->chkPerspective);
		builder->get_widget("lblMaximumShift", this->lblMaximumShift);
		builder->get_widget("lblMaximumShiftPercentage", this->lblMaximumShiftPercentage);
		builder->get_widget("sldMaximumShift", this->sldMaximumShift);
		
		this->sldMaximumShift->set_value(12.0);
		
		// Controls related to ghost removal
		builder->get_widget("chkDeghost", this->chkDeghost);
		if (this->chkDeghost)
			this->chkDeghost->signal_toggled().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnDeghostCheckedChanged));
		builder->get_widget("lblGhostDetection", this->lblGhostDetection);
		builder->get_widget("lblGhostDetectionValue", this->lblGhostDetectionValue);
		//builder->get_widget("cbxGhostDetection", this->cbxGhostDetection);
		//if(this->cbxGhostDetection)
		//	this->cbxGhostDetection->set_active(0);
		
		
		// Controls related to noise reduction
		builder->get_widget("lblReduceNoiseStrength", this->lblReduceNoiseStrength);
		builder->get_widget("lblReduceNoiseStrengthPercentage", this->lblReduceNoiseStrengthPercentage);
		builder->get_widget("cbxReduceNoise", this->cbxReduceNoiseType);
		if(this->cbxReduceNoiseType)
			this->cbxReduceNoiseType->set_active(0);
		/*this->cbxReduceNoiseType->set_popup_fixed_width(true);
		Gtk::CellRendererText* aa = (Gtk::CellRendererText*)this->cbxReduceNoiseType->get_cells()[0];//->set_visible(false);
		aa->property_ellipsize().set_value(Pango::EllipsizeMode::ELLIPSIZE_MIDDLE);
		//aa->property_max_width_chars().set_value(10);
		aa->property_width() = 150;*/
		
		builder->get_widget("sldReduceNoiseStrength", this->sldReduceNoiseStrength);
		this->sldReduceNoiseStrength->set_range(50.0, 150.0);
		this->sldReduceNoiseStrength->set_value(100.0);
		
		builder->get_widget("chkReduceNoise", this->chkReduceNoise);
		if (this->chkReduceNoise)
			this->chkReduceNoise->signal_toggled().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnReduceNoiseCheckedChanged));
		
		// Other controls		
		builder->get_widget("frmRawConversion", this->frmRawConversion);
		builder->get_widget("cbxWB", this->cbxWhiteBalance);
		if (this->cbxWhiteBalance)
		{
			this->cbxWhiteBalance->set_active(0);
			this->cbxWhiteBalance->signal_changed().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnWhiteBalanceSelectedIndexChanged));
		}
		builder->get_widget("txtWB", this->txtWhiteBalance);
		this->txtWhiteBalance->set_editable(false);
		this->txtWhiteBalance->set_sensitive(false);
		this->txtWhiteBalance->signal_focus_out_event().connect(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnTxtWhiteBalanceLostFocus));
		
		builder->get_widget("sldDeghosting", this->sldDeghosting);
		this->sldDeghosting->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &PreprocessingOptionsForm::OnSliderValueChanging), this->sldDeghosting));
		
		this->sldDeghosting->set_digits(0);
		this->sldDeghosting->set_range(1.0, 100.0);
		this->sldDeghosting->set_value(50.0);
		
		builder->get_widget("cbxColorSpace", this->cbxColorSpace);
		if(this->cbxColorSpace)
			this->cbxColorSpace->set_active(0);
		
		this->ReadStickySettings();
		
		// Set initial enability of controls
		this->OnAlignImagesCheckedChanged();
		this->OnDeghostCheckedChanged();
		this->OnReduceNoiseCheckedChanged();
	}

	PreprocessingOptionsForm::~PreprocessingOptionsForm()
	{

	}
	
	void PreprocessingOptionsForm::OnBtnOkClicked()
	{
		this->SaveStickySettings();
		
		this->hide();
		this->signal_ok.emit();
	}
	
	void PreprocessingOptionsForm::OnBtnCancelClicked()
	{
		this->hide();
	}
	
	void PreprocessingOptionsForm::OnReduceNoiseCheckedChanged()
	{
		bool enabled = this->chkReduceNoise->get_active();
		this->cbxReduceNoiseType->set_sensitive(enabled);
		this->lblReduceNoiseStrength->set_sensitive(enabled);
		this->lblReduceNoiseStrengthPercentage->set_sensitive(enabled);
		this->sldReduceNoiseStrength->set_sensitive(enabled);
	}
	
	void PreprocessingOptionsForm::OnAlignImagesCheckedChanged()
	{
		bool enabled = this->chkAlignImages->get_active();
		this->chkCrop->set_sensitive(enabled);
		this->rbnAlignShifts->set_sensitive(enabled);
		this->rbnAlignFeatures->set_sensitive(enabled);
		this->chkPerspective->set_sensitive(enabled);
		this->lblMaximumShift->set_sensitive(enabled);
		this->lblMaximumShiftPercentage->set_sensitive(enabled);
		this->sldMaximumShift->set_sensitive(enabled);
	}
	
	void PreprocessingOptionsForm::OnDeghostCheckedChanged()
	{
		bool enabled = this->chkDeghost->get_active();
		this->lblGhostDetection->set_sensitive(enabled);
		this->lblGhostDetectionValue->set_sensitive(enabled);
		this->sldDeghosting->set_sensitive(enabled);
	}
	
	void PreprocessingOptionsForm::SetSourceImagesRaw(bool raw)
	{
		this->frmRawConversion->set_visible(raw);
	}
	
	void PreprocessingOptionsForm::OnWhiteBalanceSelectedIndexChanged()
	{
		int index = this->cbxWhiteBalance->get_active_row_number();
		EWhiteBalance curWB = this->whiteBalanceSettings[index];
		if(curWB != EWhiteBalance::WBAsShot && curWB != EWhiteBalance::WBAuto)
		{
			stringstream whiteBalance;
			whiteBalance << curWB;
			this->txtWhiteBalance->set_text(whiteBalance.str());
		}
		else if(curWB == EWhiteBalance::WBAsShot || curWB == EWhiteBalance::WBAuto)
		{
			this->txtWhiteBalance->set_text("");
		}
		
		if(curWB == EWhiteBalance::WBCustom)
		{
			stringstream whiteBalance;
			whiteBalance << (int)EWhiteBalance::WBDaylight;
			this->txtWhiteBalance->set_text(whiteBalance.str());
			this->txtWhiteBalance->set_editable(true);
			this->txtWhiteBalance->set_sensitive(true);
		}
		else
		{
			this->txtWhiteBalance->set_editable(false);
			this->txtWhiteBalance->set_sensitive(false);
		}
	}
	
	bool PreprocessingOptionsForm::OnTxtWhiteBalanceLostFocus(GdkEventFocus* evt)
	{
		if(this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()] != EWhiteBalance::WBCustom)
			return false;
		
		int curTemperature = atoi(this->txtWhiteBalance->get_text().c_str());
		// Limit available temperature to the range of 2000 - 50000 (one used by Lightroom 4)
		if(curTemperature < 2000 || curTemperature > 50000)
		{
			if(curTemperature < 2000) curTemperature = 2000;
			if(curTemperature > 50000) curTemperature = 50000;
			
			stringstream temperature;
			temperature << curTemperature;
			this->txtWhiteBalance->set_text(temperature.str());
		}
	
		return false;
	}
	
	void PreprocessingOptionsForm::ReadStickySettings()
	{
		this->chkAlignImages->set_active(AppProperties::GetGenerateHdrAlignmentEnabled());
		this->chkCrop->set_active(AppProperties::GetGenerateHdrAlignmentCrop());
		this->chkPerspective->set_active(AppProperties::GetGenerateHdrAlignmentPerspective());
		this->sldMaximumShift->set_value(AppProperties::GetGenerateHdrAlignmentMaxShiftPercent() * 100.0f);
		if(AppProperties::GetGenerateHdrAlignmentType() == 1)
			this->rbnAlignFeatures->set_active(true);
		else
			this->rbnAlignShifts->set_active(true);
		this->chkReduceNoise->set_active(AppProperties::GetGenerateHdrNoiseReductionEnabled());
		this->cbxReduceNoiseType->set_active(AppProperties::GetGenerateHdrNoiseReductionType());
		this->sldReduceNoiseStrength->set_value(AppProperties::GetGenerateHdrNoiseReductionStrength());
		this->chkDeghost->set_active(AppProperties::GetGenerateHdrDeghostingEnabled());
		this->sldDeghosting->set_value(AppProperties::GetGenerateHdrDeghostingStrength());
		this->cbxWhiteBalance->set_active(AppProperties::GetGenerateHdrWhiteBalance());
		this->cbxColorSpace->set_active(AppProperties::GetGenerateHdrColorProfile());
	}

	void PreprocessingOptionsForm::SaveStickySettings()
	{
		AppProperties::SetGenerateHdrAlignmentEnabled(this->chkAlignImages->get_active());
		AppProperties::SetGenerateHdrAlignmentCrop(this->chkCrop->get_active());
		AppProperties::SetGenerateHdrAlignmentPerspective(this->chkPerspective->get_active());
		AppProperties::SetGenerateHdrAlignmentMaxShiftPercent(this->sldMaximumShift->get_value() * 0.01f);
		AppProperties::SetGenerateHdrAlignmentType(this->rbnAlignFeatures->get_active() ? 1 : 0);
		AppProperties::SetGenerateHdrNoiseReductionEnabled(this->chkReduceNoise->get_active());
		AppProperties::SetGenerateHdrNoiseReductionType(this->cbxReduceNoiseType->get_active_row_number());
		AppProperties::SetGenerateHdrNoiseReductionStrength(this->sldReduceNoiseStrength->get_value());
		AppProperties::SetGenerateHdrDeghostingEnabled(this->chkDeghost->get_active());
		AppProperties::SetGenerateHdrDeghostingStrength(this->sldDeghosting->get_value());
		AppProperties::SetGenerateHdrWhiteBalance(this->cbxWhiteBalance->get_active_row_number());
		AppProperties::SetGenerateHdrColorProfile(this->cbxColorSpace->get_active_row_number());
	}
	
	void PreprocessingOptionsForm::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldDeghosting) this->lblGhostDetectionValue->set_text(str.str());
	}

}