#ifndef PREPROCESSINGOPTIONSFORM_H
#define PREPROCESSINGOPTIONSFORM_H


#include <gtkmm.h>
#include "photoprocessor.h"


namespace Photomatix
{
	/// White balance for RAW conversion
	enum EWhiteBalance
	{
		WBAsShot = 0,
		WBAuto = -1,
		WBDaylight = 5500,
		WBCloudy = 6500,
		WBShade = 7500,
		WBTungsten = 2850,
		WBFluorescent = 3800,
		WBCustom = -2
	};
	
	/** Preprocessing options form.
	 */
	class PreprocessingOptionsForm: public Gtk::Dialog
	{
	protected:
		Gtk::CheckButton* chkAlignImages;
		Gtk::CheckButton* chkCrop;
		Gtk::RadioButton* rbnAlignShifts;
		Gtk::RadioButton* rbnAlignFeatures;
		Gtk::CheckButton* chkPerspective;
		Gtk::Label* lblPerspective;
		Gtk::Label* lblMaximumShift;
		Gtk::Label* lblMaximumShiftPercentage;
		Gtk::HScale* sldMaximumShift;
		
		Gtk::CheckButton* chkDeghost;
		Gtk::Label* lblGhostDetection;
		Gtk::Label* lblGhostDetectionValue;
		//Gtk::ComboBox* cbxGhostDetection;
		Gtk::HScale* sldDeghosting;
		
		Gtk::CheckButton* chkReduceNoise;
		Gtk::ComboBox* cbxReduceNoiseType;
		Gtk::Label* lblReduceNoiseStrength;
		Gtk::Label* lblReduceNoiseStrengthPercentage;
		Gtk::HScale* sldReduceNoiseStrength;
		
		Gtk::Frame* frmRawConversion;
		Gtk::ComboBox* cbxWhiteBalance;
		Gtk::Entry* txtWhiteBalance;
		
		Gtk::ComboBox* cbxColorSpace;
		
		EWhiteBalance whiteBalanceSettings[8];
		
		
		//@{
		/** Signal Handlers */
		void OnBtnOkClicked();
		void OnBtnCancelClicked();
		
		void OnSliderValueChanging(Gtk::Scale* sender);
		void OnAlignImagesCheckedChanged();
		void OnDeghostCheckedChanged();
		void OnReduceNoiseCheckedChanged();
		void OnWhiteBalanceSelectedIndexChanged();
		bool OnTxtWhiteBalanceLostFocus(GdkEventFocus* evt);
		//@}
		
		void ReadStickySettings();
		void SaveStickySettings();
			
	public:
		PreprocessingOptionsForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
		virtual ~PreprocessingOptionsForm();
		
		/// Signal emitted when the ok button was clicked.
		sigc::signal<void> signal_ok;
		
		void SetSourceImagesRaw(bool raw);
		
		//@{
		/** Getters returning values of controls */
		bool GetReduceChromaticAberrations() const {return false;}
		bool GetAlignImages() const {return this->chkAlignImages->get_active();}
		bool GetCropAlignedImages() const {return this->chkCrop->get_active();}
		bool GetAlignByShifts() const {return this->rbnAlignShifts->get_active();}
		bool GetPerspectiveCorrection() const {return this->chkPerspective->get_active();}
		
		float GetGhostingDetectionStrength() const {return this->sldDeghosting->get_value();}
		PhotomatixImaging::ENoiseReductionType GetNoiseReductionType() const {return (PhotomatixImaging::ENoiseReductionType)this->cbxReduceNoiseType->get_active_row_number();}
		
		float GetMaximumShift() const {return this->sldMaximumShift->get_value();}
		
		bool GetDeghost() const {return this->chkDeghost->get_active();}
		bool GetReduceNoise() const {return this->chkReduceNoise->get_active();}
		
		int GetNoiseReductionStrength() const {return this->sldReduceNoiseStrength->get_value();}
		
		int GetColorTemperature() const
		{
			if(this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()] != EWhiteBalance::WBCustom)
				return this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()];
			else
				return atoi(this->txtWhiteBalance->get_text().c_str());
		}
		
		unsigned int GetColorSpace() const {return this->cbxColorSpace->get_active_row_number();}
		//@}
	};
}

	
#endif // PREPROCESSINGOPTIONSFORM_H
