#ifndef PRESETSWINDOW_H
#define PRESETSWINDOW_H


#include <gtkmm-3.0/gtkmm/scrolledwindow.h>
#include <gtkmm-3.0/gtkmm/box.h>
#include <vector>
#include <memory>
#include <queue>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <gtkmm.h>
#include "presetthumbnail.h"


// Forward declarations
namespace PhotomatixImaging
{
	class PhotoImage;
}

namespace Photomatix
{
	class PresetsWindow;
	
	
	typedef std::pair<std::shared_ptr <PhotomatixImaging::PhotoImage>, PhotomatixImaging::Preset> Thumbnail;
	
	
	
	template <typename T>
	class Queue
	{
	public:
	
		T pop()
		{
			std::unique_lock<std::mutex> mlock(mutex_);
			while (queue_.empty())
			{
				cond_.wait(mlock);
			}
			auto item = queue_.front();
			queue_.pop();
			return item;
		}
		
		void pop(T& item)
		{
			std::unique_lock<std::mutex> mlock(mutex_);
			while (queue_.empty())
			{
				cond_.wait(mlock);
			}
			item = queue_.front();
			queue_.pop();
		}
		
		void push(const T& item)
		{
			std::unique_lock<std::mutex> mlock(mutex_);
			queue_.push(item);
			mlock.unlock();
			cond_.notify_one();
		}
		
		void push(T&& item)
		{
			std::unique_lock<std::mutex> mlock(mutex_);
			queue_.push(std::move(item));
			mlock.unlock();
			cond_.notify_one();
		}
	
	private:
		std::queue<T> queue_;
		std::mutex mutex_;
		std::condition_variable cond_;
	};

	class PresetThumbnailsGenerator
	{
	private:
		mutable std::mutex mutexStopped_;
		
	protected:
		std::shared_ptr <PhotomatixImaging::PhotoImage> photo;
		std::vector <PhotomatixImaging::Preset> presetsList;
		//std::vector <std::shared_ptr <PhotomatixImaging::PhotoImage> > outputImages;
		
		Queue <Thumbnail>& outputQueue;
		
		PresetsWindow* caller;
		
		bool stopped;
		bool shallStop;
		
	public:
		PresetThumbnailsGenerator(PresetsWindow* presetsWindow, Queue <Thumbnail>& queue);
		
		
		void Process();
		
		
		void SetSourcePhoto(std::shared_ptr <PhotomatixImaging::PhotoImage> sourcePhoto) {this->photo = sourcePhoto;}
		void SetPresetsList(std::vector <PhotomatixImaging::Preset> presets) {this->presetsList = presets;}
		
		bool HasStopped() const;
		void Stop();
		
		//std::vector <std::shared_ptr <PhotomatixImaging::PhotoImage> > GetOutputPhotos() {return this->outputImages;}
	};
	
	/*class PresetThumbnailsGeneratorConsumer
	{
	protected:
		Queue <PresetThumbnail*>& outputQueue;
		int presetsNum;
		
	public:
		PresetThumbnailsGeneratorConsumer(Queue <PresetThumbnail*>& queue, int presetsCount):
			outputQueue(queue),
			presetsNum(presetsCount)
		{}
		
		void operator()();
	};*/
	
	
	class PresetsWindow: public Gtk::ScrolledWindow
	{
	friend class PresetThumbnailsGenerator;
	protected:
		Gtk::Box boxPresetsBuiltIn;
		Gtk::Box boxPresetsCustom;
		
		Gtk::Label lblNoCustomPresets;
		
		
		Glib::Dispatcher dispatcher;
		Glib::Dispatcher dispatcherFinished;
		
		std::vector <PhotomatixImaging::Preset> presetsList;
		std::vector <PhotomatixImaging::Preset> presetsListCustom;
		
		std::shared_ptr <PhotomatixImaging::PhotoImage> inputPhoto;
		std::vector <std::shared_ptr <PhotomatixImaging::PhotoImage>> sourcePhotos;
		std::shared_ptr <std::vector <unsigned char>> imageBuffer;
		std::shared_ptr <std::vector <unsigned char> > ghostMask;
		
		bool refreshBuiltInPresets;
		bool refreshCustomPresets;
		bool isShowingBuiltIn;
		
		Queue <Thumbnail> thumbnailGeneratorQueue;
		
		PresetThumbnail* previouslySelectedThumb;
		
		
		//PresetThumbnailsGenerator thumbnailGenerator;
		PresetThumbnailsGenerator thumbnailGenerator;
		std::thread* thumbnailGeneratorThread;
		
		
		void GetPresetsList(bool builtIn);
		void RenderPresets(bool builtIn);
		
		void OnThumbnailClicked(PresetThumbnail* thumb);
		void OnThumbnailCreated();
		void OnThumbnailsCreated();
		
	public:
		PresetsWindow();
		virtual ~PresetsWindow();
		
		void InitPresetsThumbnails(std::shared_ptr <PhotomatixImaging::PhotoImage> inputPhoto, std::vector <std::shared_ptr <PhotomatixImaging::PhotoImage>> sourcePhotos, std::shared_ptr <std::vector <unsigned char>> imageBuffer, std::shared_ptr <std::vector <unsigned char> > ghostMask);
		void CalculatePresetsThumbnails() {}
		
		void RefreshCustomPresets();
		void ShowPresets(bool builtIn);
		
		void UnselectThumbnail();
		
		
		// Called from the worker thread.
		void NotifyThumbnailCreated();
		void NotifyFinished();
		
		void StopThumbnailsRendering();
		
		
		sigc::signal <void, PhotomatixImaging::Preset*> signal_settings_changed;
		sigc::signal <void, bool> signal_thumbnails_rendering_changed;
	};
}
	
#endif // PRESETSWINDOW_H
