#ifndef PRESETTHUMBNAIL_H
#define PRESETTHUMBNAIL_H


#include <memory>
#include <gtkmm-3.0/gtkmm/box.h>
#include <gtkmm-3.0/gtkmm/label.h>

#include "preset.h"


// Some forward declarations
namespace PhotomatixControls
{
	class DisplayableImageControl;
}

namespace PhotomatixImaging
{
	class PhotoImage;
}


namespace Photomatix
{
	/** A single preset thumbnail.
	 */
	class PresetThumbnail: public Gtk::Box
	{
	protected:
		PhotomatixControls::DisplayableImageControl* presetImage;
		Gtk::Label* presetName;
		PhotomatixImaging::Preset preset;		///< Tone-mapping/fusion settings used by this thumbnail
		
		
		void get_preferred_height_vfunc(int& minimum_height, int& natural_height) const;
		void OnThumbnailClicked(GdkEventButton* evt);
		
	public:
		PresetThumbnail();
		~PresetThumbnail();
		
		void SetSettings(const PhotomatixImaging::Preset& preset);
		void SetThumbnailImage(std::shared_ptr <PhotomatixImaging::PhotoImage> image);
		void SetSelected(bool selected);
		
		PhotomatixImaging::Preset& GetPreset() {return this->preset;}
		PhotomatixControls::DisplayableImageControl* GetImage() const {return this->presetImage;}
		
		
		sigc::signal <void, PresetThumbnail*> signal_thumbnail_clicked;
	};
}


#endif // PRESETTHUMBNAIL_H
