#include "previewcontroller.h"
#include "presetswindow.h"
#include "previewwindow.h"
#include "settingswindow.h"
#include "../PhotomatixImaging/photoimage.h"
#include "../PhotomatixImaging/hdrgenerator.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"
#include "../Photomatix32/PhotomatixMetaLib.h"
#include "../PhotomatixMisc/localtext.h"
#include "exposurefusionprocessor.h"
#include "tonemapper.h"
#include "GladeUtils.h"
#include "presetthumbnail.h"
#include "progressform.h"
#include "displayableimageform.h"

#include "../Photomatix32/tmlib.h"
#include "../Photomatix32/pmjp/pmjp.h"
#include "../PhotomatixExternalLibs/stlsoft/include/rangelib/filtered_range.hpp"
#include "../PhotomatixMisc/appproperties.h"
#include <gtkmm/window.h>
#include "detailsenhancersettings.h"
#include "progressnotifier.h"
#include "linuxprogressnotifier.h"
#include "dbgprintf.h"

/// TODO: remove this later
#include "ResourceUtils.h"


using namespace std;
using namespace PhotomatixImaging;
using namespace PhotomatixControls;
using namespace PhotomatixMisc;


namespace Photomatix
{
	PreviewController::PreviewController(std::shared_ptr <PhotomatixImaging::PhotoImage> photo, shared_ptr <HdrGenerator> generator):
		currentZoomLevel(0),
		currentMethod(1),
		hdrGenerator(generator),
		isPreviewUpdating(false),
		initialUpdate(true),
		isChangingThumbnail(false),
		basePreviewContrastOptimizerW(0),
		basePreviewContrastOptimizerH(0),
		basePreviewW(0),
		basePreviewH(0)
	{		
		
		this->builtInPresetsSelected = AppProperties::GetBuiltInPresetsSelected();
		
		this->originalPhoto = photo;
		
		this->mapper = shared_ptr <ToneMapper>(new ToneMapper());
		// Create blender when needed
		if(generator != NULL && generator->GetInputPhotosCount() > 1) {
			this->blender = shared_ptr <ExposureFusionProcessor>(new ExposureFusionProcessor());
		}
		
		//int result = tmparam_default(1, 0, tmParams);
		shared_ptr<char> oldLocale(strdup(setlocale(LC_ALL, NULL)), free);
                setlocale(LC_ALL, "POSIX");

		// 1. Get method
		string path = AppProperties::GetUserPath() + "sticky_settings.xmp";
		this->currentMethod = read_xmp(path.c_str(), -1, NULL);
		bool stickySettingsInvalid = (this->currentMethod != 1 && this->currentMethod != 2 && this->currentMethod != 3 && this->currentMethod != 4 && this->currentMethod != 10);
		// 2. Read real parameters
		read_xmp(path.c_str(), this->currentMethod, this->currentSettings);
                setlocale(LC_ALL, oldLocale.get());
		if(stickySettingsInvalid || !AppProperties::GetUseStickyTmSettings() || (currentMethod == 10 && (generator == NULL || generator->GetInputPhotosCount() == 1)) || currentMethod == -1) {
			currentMethod = 1;
			Preset::GetMethodDefault(1, 0, this->currentSettings);
			//tmparam_default(1, 0, this->currentSettings);
		}
		XmpSettings xmpSettings;
		xmpSettings.method = this->currentMethod;
		memcpy(xmpSettings.parameters, this->currentSettings, sizeof(this->currentSettings));
		if(this->currentMethod <= 4) {
			this->mapper->SetToneMappingMethod((PhotomatixImaging::EToneMappingMethod)this->currentMethod);
			this->mapper->SetProcessingSettings(this->currentSettings);
		} else if(this->currentMethod == 10) {
			this->blender->SetProcessingSettings(this->currentSettings);
		}
		
		this->presetsForm = shared_ptr <PresetsWindow>(new PresetsWindow());
		this->presetsForm->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewController::OnThumbnailChanged));
		this->presetsForm->signal_thumbnails_rendering_changed.connect(sigc::mem_fun(*this, &PreviewController::OnThumbnailsRenderingChanged));
		
		this->settingsForm = shared_ptr <SettingsWindow>(new SettingsWindow());
		//this->settingsForm->ReadSettingsFromSticky();
		
		//this->previewForm = shared_ptr <PreviewWindow>(new PreviewWindow());
		
		PhotomatixMisc::dbgprintf("num photos = %d\n", generator != NULL && generator->GetInputPhotosCount() > 1 ? generator->GetInputPhotosCount() : 1);
		
		PhotomatixUtils::get_widget_derived("preview-window", this->previewForm);
		this->previewForm->SetImagesNum(generator != NULL && generator->GetInputPhotosCount() > 1 ? generator->GetInputPhotosCount() : 1);
		this->previewForm->Init();
		this->previewForm->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewController::OnSettingsChanged));
		this->previewForm->signal_preset_changed.connect(sigc::mem_fun(*this, &PreviewController::OnPresetChanged));
		
		this->InitPreviewSizes();
		this->previewForm->SetZoomLevels(this->zoomInLevels, this->zoomOutLevels);
		this->basePreviewPhotos.resize(this->zoomInLevels + this->zoomOutLevels + 1);
		this->basePreviewBuffers.resize(this->zoomInLevels + this->zoomOutLevels + 1);
		this->ghostMasks.resize(this->zoomInLevels + this->zoomOutLevels + 1);
		
		/// TODO: we should read it from sticky settings!
		this->currentZoomLevel = this->zoomInLevels;
		if(AppProperties::GetPreviewZoomLevel() != -1) {
			this->previewForm->SetPreviewZoomLevel(AppProperties::GetPreviewZoomLevel());
			this->currentZoomLevel = this->previewForm->GetPreviewZoomLevel();
		}
		
		this->InitPreviewImage();
		
		//this->UpdatePreviewImage();
		// Set sliders
		this->previewForm->SetBlockUpdates(true);
		this->OnSettingsChanged(&xmpSettings);
		this->previewForm->SetBlockUpdates(false);
		
		this->InitThumbnails();
		
	//	this->preprocessingOptions->SetSourceImagesRaw(this->hdrGenerator->GetInputPhoto(0)->GetImageFormat() == EImageFormatType::Raw);
	//	this->preprocessingOptions->signal_ok.connect(sigc::mem_fun(*this, &HdrController::GenerateHDR));
		
		if(this->previewForm)
			this->previewForm->show();
		this->previewForm->AddControl("presets-thumbnails-box", presetsForm.get());
		this->previewForm->signal_zoom_level_changed.connect(sigc::mem_fun(*this, &PreviewController::OnPreviewZoomLevelChanged));
		this->previewForm->signal_preview_closed.connect(sigc::mem_fun(*this, &PreviewController::OnPreviewClosed));
		this->previewForm->signal_final_processing.connect(sigc::mem_fun(*this, &PreviewController::OnFinalProcessing));
		this->previewForm->signal_presets_selected.connect(sigc::mem_fun(*this, &PreviewController::OnPresetsCategorySelected));
	}

	PreviewController::~PreviewController()
	{

	}
	
	void PreviewController::OnThumbnailsRenderingChanged(bool done)
	{
		this->previewForm->SetThumbnailsTabsEnability(done);
	}
	
	void PreviewController::OnPresetsCategorySelected(bool builtIn)
	{
		AppProperties::SetBuiltInPresetsSelected(builtIn);
		
		this->builtInPresetsSelected = builtIn;
		this->presetsForm->ShowPresets(builtIn);
	}
	
	void PreviewController::OnPresetChanged(int preset, std::string fileName)
	{
		// Save preset
		if(preset == 0)
		{
			int prevIndex = this->previewForm->GetPresetIndex();
			
			Gtk::FileChooserDialog savePresetDialog(LocalText::GetSaveDialogTitle(), Gtk::FILE_CHOOSER_ACTION_SAVE);
			savePresetDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			savePresetDialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
			
			// Create custom presets path if it doesn't exist yet
			struct stat sb;
			string folder = PhotomatixMisc::AppProperties::GetUserPath() + "custom_presets/";
			if (!stat(folder.c_str(), &sb) == 0 || !S_ISDIR(sb.st_mode))
				mkdir(folder.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
			
			Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
			filter->set_name("XMP");
			filter->add_pattern("*.[Xx][Mm][Pp]");
			savePresetDialog.add_filter(filter);
			savePresetDialog.set_current_folder(folder);
			savePresetDialog.set_do_overwrite_confirmation(true);
			savePresetDialog.set_current_name("preset.xmp");
			
			savePresetDialog.set_filter(filter);
			
			int result = savePresetDialog.run();
			savePresetDialog.set_visible(false);
			if(result == Gtk::RESPONSE_OK)
			{
				write_xmp(savePresetDialog.get_filename().c_str(), this->currentMethod, this->currentSettings);
			}
			
			this->previewForm->SetPresetIndex(-1, true);
			
			// Update preset thumbnails list
			this->presetsForm->RefreshCustomPresets();
			if(!this->builtInPresetsSelected)
				this->presetsForm->ShowPresets(false);
		}
		// Load preset
		if(preset == 1)
		{
			int prevIndex = this->previewForm->GetPresetIndex();
			
			Gtk::FileChooserDialog openPresetDialog(LocalText::GetOpenDialogTitle(), Gtk::FILE_CHOOSER_ACTION_OPEN);
			openPresetDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			openPresetDialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
			
			Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
			filter->set_name("XMP");
			filter->add_pattern("*.[Xx][Mm][Pp]");
			openPresetDialog.add_filter(filter);
			openPresetDialog.set_current_folder(PhotomatixMisc::AppProperties::GetLastOpenDirectory());
			openPresetDialog.set_select_multiple(true);
			
			int result = openPresetDialog.run();
			if(result == Gtk::RESPONSE_OK)
			{
				shared_ptr<char> oldLocale(strdup(setlocale(LC_ALL, NULL)), free);
				setlocale(LC_ALL, "POSIX");
				string path = openPresetDialog.get_filename();
				int method = read_xmp(path.c_str(), -1, NULL);
				// 2. Read real parameters
				XmpSettings xmpSettings;
				xmpSettings.method = method;
				
				read_xmp(path.c_str(), method, xmpSettings.parameters);
				setlocale(LC_ALL, oldLocale.get());
				
				//memcpy(xmpSettings.parameters, this->currentSettings, sizeof(this->currentSettings));
				bool exposureFusionEnabled = (this->hdrGenerator != NULL && this->hdrGenerator->GetInputPhotosCount() > 1);
				if(method <= 4 || (method == 10 && exposureFusionEnabled))
				{
					/*if(this->currentMethod == 10) {
						this->blender->SetProcessingSettings(this->currentSettings);
					}
					else {
						this->mapper->SetToneMappingMethod((PhotomatixImaging::EToneMappingMethod)this->currentMethod);
						this->mapper->SetProcessingSettings(this->currentSettings);
					}*/
					
					// Set sliders
					this->previewForm->SetBlockUpdates(true);
					this->OnSettingsChanged(&xmpSettings);
					this->previewForm->SetBlockUpdates(false);
				}
			}
			
			this->previewForm->SetPresetIndex(-1, true);
		}
		else if(!fileName.empty())
		{
			shared_ptr<char> oldLocale(strdup(setlocale(LC_ALL, NULL)), free);
			setlocale(LC_ALL, "POSIX");
			string path = fileName;
			// Default settings
			int method = 0;
			XmpSettings xmpSettings;
			if(path[0] == '-')
			{
				method = std::stoi(path.substr(1));
				Preset::GetMethodDefault(method, 0, xmpSettings.parameters);
			}
			else
			{
				method = read_xmp(path.c_str(), -1, NULL);
				// 2. Read real parameters
				read_xmp(path.c_str(), method, xmpSettings.parameters);
			}
			setlocale(LC_ALL, oldLocale.get());
			xmpSettings.method = method;
			
			//memcpy(xmpSettings.parameters, this->currentSettings, sizeof(this->currentSettings));
			if(method <= 4 || method == 10)
			{
				/*if(this->currentMethod == 10) {
					this->blender->SetProcessingSettings(this->currentSettings);
				}
				else {
					this->mapper->SetToneMappingMethod((PhotomatixImaging::EToneMappingMethod)this->currentMethod);
					this->mapper->SetProcessingSettings(this->currentSettings);
				} */
				
				// Set sliders
				this->previewForm->SetBlockUpdates(true);
				this->OnSettingsChanged(&xmpSettings);
				this->previewForm->SetBlockUpdates(false);
			}
		}
	}
	
	void PreviewController::OnFinalProcessing()
	{
		this->presetsForm->StopThumbnailsRendering();
		
		// Reset locale temporarily so we can write decimal points properly
		shared_ptr<char> oldLocale(strdup(setlocale(LC_ALL, NULL)), free);
		setlocale(LC_ALL, "POSIX");

		// Save tone mapping settings
		string path = AppProperties::GetUserPath() + "sticky_settings.xmp";
		int result = write_xmp(path.c_str(), this->currentMethod, this->currentSettings);
		setlocale(LC_ALL, oldLocale.get());
		
		
		// 1. Prepare processor
		if(this->currentMethod <= 4)
		{
			processor = this->mapper.get();
			std::vector <std::shared_ptr <PhotoImage>> photos;
			//if(this->basePreviewPhotos[currentZoomLevel] != NULL)
				//photos.push_back(this->basePreviewPhotos[this->currentZoomLevel]);
			photos.push_back(this->hdrGenerator ? this->hdrGenerator->GetOutputPhoto() : this->originalPhoto);
			this->mapper->SetIsPreview(false);
			this->mapper->SetInputPhotos(photos);
		}
		else if(this->currentMethod == 10)
		{
			processor = this->blender.get();
			std::vector <std::shared_ptr <PhotoImage>> photos = this->hdrGenerator->GetInputPhotos();
			this->blender->SetInputPhotos(photos);
			
			this->blender->SetIsPreview(false);
			this->blender->SetGhostMask(this->hdrGenerator->GetGhostMask());
			this->blender->SetImagesBuffer(this->hdrGenerator->GetImagesBuffer());
		}
		
		processor->SetBitsPerSampleOut(16);
		
		
		this->finalProcessorDone = processor->sig_done.connect(sigc::mem_fun(*this, &PreviewController::OnFinalProcessingDone));
		
		// 2. Hide preview window
		this->previewForm->set_visible(false);
		
		// 3. Create progress notifier
		PhotomatixUtils::get_widget_derived("progress-dialog", this->progressForm);
		std::string title;
		if(this->hdrGenerator) {
			title = this->hdrGenerator->GetOutputFileName(AppProperties::GetConcatenateSourceFilenames());
		} else {
			title = this->originalPhoto->GetFileNameShortNoExtension();
		}
		//std::string title = this->hdrGenerator->GetOutputFileName(true);
		this->progressForm->set_title(title.substr(title.find_last_of("//") + 1));
		this->finalProcessorCancelled = this->progressForm->signal_stop_button_clicked.connect(sigc::mem_fun(*this, &PreviewController::OnFinalProcessingCancelled));
		processor->progressNotifier = std::dynamic_pointer_cast<PhotomatixImaging::ProgressNotifier>(this->progressForm->GetProgressNotifier());
		if(this->progressForm)
			this->progressForm->show();	
		
		// 4. Process
		processor->Start();
	}
	
	void PreviewController::OnFinalProcessingCancelled()
	{
		processor->Stop();
	}
	
	void PreviewController::OnFinalProcessingDone()
	{
		this->progressForm->set_visible(false);
		
		
		if(processor->Cancelled()) {
			this->processor->Reset();
			
			this->finalProcessorCancelled.disconnect();
			this->finalProcessorDone.disconnect();
			
			this->processor->progressNotifier = NULL;
			
			this->UpdatePreviewImage();
			this->previewForm->present();
			
			return;
		}
		
		
		// Write photo to a file
		shared_ptr<PhotoImage> outputImage = processor->GetOutputPhoto();
		 
		if(this->currentMethod != 10 && originalPhoto->GetSamplesPerPixel() == 3 && originalPhoto->GetImageFormat() != EImageFormatType::Raw &&
			originalPhoto->GetImageFormat() != EImageFormatType::Hdr) {
			bgr2rgb(outputImage->GetWidth(), outputImage->GetHeight(), outputImage->GetRowStride(), outputImage->GetSamplesPerPixel(), outputImage->GetBitsPerSample(), outputImage->GetImageBuffer()->data());
		}
		
		
		this->signal_preview_finished.emit(outputImage);
	}
	
	void PreviewController::OnPreviewClosed()
	{
		this->signal_preview_closed.emit();
	}
	
	void PreviewController::GetBasePreviewSizeForContrastOptimizer(int wPreview, int hPreview, int wOrig, int hOrig, int& basePreviewW, int& basePreviewH)
	{
		int majorDimOrig = wOrig, minorDimOrig = hOrig, majorDimPreview = wPreview, minorDimPreview = hPreview, iLevel = 0;
		if (hOrig > wOrig)
		{
			majorDimOrig = hOrig; majorDimPreview = hPreview;
			minorDimOrig = wOrig; minorDimPreview = wPreview;
		}
		int levelMaxOrig = (int)(logf((float)minorDimOrig) / logf(2.0) + 1.0), levelMaxPreview = (int)(logf((float)minorDimPreview) / logf(2.0) + 1.0);
		
		std::vector <int> arrMajorDimOrig(levelMaxOrig);
		std::vector <int> arrMajorDimPreview(levelMaxOrig);
		
		arrMajorDimOrig[0] = majorDimOrig;
		for (iLevel = 1; iLevel < levelMaxOrig; iLevel++) arrMajorDimOrig[iLevel] = arrMajorDimOrig[iLevel - 1] / 2 + arrMajorDimOrig[iLevel - 1] % 2;
		
		arrMajorDimPreview[0] = majorDimPreview;
		for (iLevel = 1; iLevel < levelMaxPreview; iLevel++) arrMajorDimPreview[iLevel] = arrMajorDimPreview[iLevel - 1] / 2 + arrMajorDimPreview[iLevel - 1] % 2;
		
		if ((arrMajorDimOrig[levelMaxOrig - 1] == arrMajorDimPreview[levelMaxPreview - 1]) && (arrMajorDimOrig[levelMaxOrig - 2] == arrMajorDimPreview[levelMaxPreview - 2]))
			//#endif
		{
			basePreviewW = wPreview; basePreviewH = hPreview;
			return;
		}
		
		//Changing sampling of the two lowest frequencies to the sampling of image at original resolution
		//#ifdef EqualiseLowestFrequencyOnly
		//	arrMajorDimPreview[levelMaxPreview-1]=arrMajorDimOrig[levelMaxOrig-1];
		
		//Calculating the size needed for the Preview to have sampling of two lowest frequencies the same as for image at original resolution
		//	for (iLevel=levelMaxPreview-1; iLevel>0; iLevel--) {
			//#else
			arrMajorDimPreview[levelMaxPreview - 2] = arrMajorDimOrig[levelMaxOrig - 2];
			
			//Calculating the size needed for the Preview to have sampling of two lowest frequencies the same as for image at original resolution
			for (iLevel = levelMaxPreview - 2; iLevel > 0; iLevel--)
			{
				//#endif	
				arrMajorDimPreview[iLevel - 1] = arrMajorDimPreview[iLevel] * 2;
				if ((arrMajorDimPreview[iLevel - 1] % 2) == 0) arrMajorDimPreview[iLevel - 1]--;   //we try to avoid readjusting at a too bigger size by taking advantage of the fact that we can subtract 1 in the case of even size		
			}
			
			if (arrMajorDimPreview[0] < majorDimPreview)
			{
				arrMajorDimPreview[0] *= 2; if ((arrMajorDimPreview[0] % 2) == 0) arrMajorDimPreview[0]--;
			}
			
			int adjustedMajorDim = arrMajorDimPreview[0], adjustedW = adjustedMajorDim, adjustedH = adjustedMajorDim;
			
			if (hOrig > wOrig) adjustedW = (int)((float)adjustedH * (float)wOrig / (float)hOrig);
			else adjustedH = (int)((float)adjustedW * (float)hOrig / (float)wOrig);
			
			basePreviewW = adjustedW;
			basePreviewH = adjustedH;
			//	return new Size(adjustedW, adjustedH);
		}

	void PreviewController::OnPreviewZoomLevelChanged(int zoomLevel)
	{		
		PhotomatixMisc::dbgprintf("Start PreviewController::OnPreviewZoomLevelChanged()\n");
		
		this->currentZoomLevel = zoomLevel;
		AppProperties::SetPreviewZoomLevel(zoomLevel);
		this->mapper->SetContrastOptimizerAlreadyMapped(false);
		this->UpdatePreviewImage();
		
		PhotomatixMisc::dbgprintf("End PreviewController::OnPreviewZoomLevelChanged()\n");
	}
	
	void PreviewController::InitPreviewSizes()
	{
		std::vector <Size> zoomInSizes;
		std::vector <Size> zoomOutSizes;
		Size proportionalSize;
		PhotoImage::GetProportionalSize(800, 600, this->originalPhoto->GetWidth(), this->originalPhoto->GetHeight(), proportionalSize.width, proportionalSize.height);
		if(proportionalSize.width > this->originalPhoto->GetWidth() || proportionalSize.height > this->originalPhoto->GetHeight()) {
			proportionalSize.width = this->originalPhoto->GetWidth();
			proportionalSize.height = this->originalPhoto->GetHeight();
		}
		
		while(proportionalSize.width % 4) ++proportionalSize.width;
		
		zoomInLevels = 0;
		zoomOutLevels = 0;
		
		Size maxSize = proportionalSize;
		Size minSize = proportionalSize;
		for (int i = 0; i < 4; i++) {
			maxSize.width = maxSize.width * 1.2f;
			maxSize.height = maxSize.height * 1.2f;
			while(maxSize.width % 4) ++maxSize.width;
			if(maxSize.width <= this->originalPhoto->GetWidth() && maxSize.height <= this->originalPhoto->GetHeight()) {
				zoomInSizes.insert(zoomInSizes.begin(), maxSize);
				++zoomInLevels;
			}
			else
				break;
		}
		
		for (int i = 0; i < 3; i++) {
			minSize.width = minSize.width / 1.2f;
			minSize.height = minSize.height / 1.2f;
			while(minSize.width % 4) ++minSize.width;
			if(minSize.width >= 100 && minSize.height >= 100)	{
				zoomOutSizes.push_back(minSize);
				++zoomOutLevels;
			}
			else
				break;
		}
		
		this->previewSizes.insert(this->previewSizes.begin(), zoomInSizes.begin(), zoomInSizes.end());
		this->previewSizes.push_back(proportionalSize);
		this->previewSizes.insert(this->previewSizes.begin() + this->previewSizes.size(), zoomOutSizes.begin(), zoomOutSizes.end());
	}
	
	void PreviewController::GetPresetThumbnailsSize(int inWidth, int inHeight, int& outWidth, int& outHeight)
	{
		int wCalcul = inWidth, hCalcul = inHeight, wPreview = inWidth, hPreview = inHeight, result = 0, indexThumb = 0;
		const int minimumDim = 200;
		int nivo = (int)(log((float)(inWidth + inHeight) * 0.5) / log(2.0)), prevW = wCalcul, prevH = hCalcul;
		
		for (int k = 0; k < nivo; k++)
		{
			prevW = wCalcul; prevH = hCalcul;
			if ((wCalcul > 3) && (hCalcul > 3))
			{
				wCalcul = (wCalcul >> 1) + (wCalcul % 2); hCalcul = (hCalcul >> 1) + (hCalcul % 2);
			}
			if (((wCalcul + hCalcul) / 2) <= minimumDim)
			{
				if (((wCalcul + hCalcul) / 2) < minimumDim)
				{
					wCalcul = prevW;
					hCalcul = prevH;
				}
				break;
			}
		}
		
		outWidth = wCalcul;
		while(outWidth % 4) outWidth++;
		outHeight = hCalcul;
	}
	
	void PreviewController::InitThumbnails()
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::InitThumbnails()\n");
		
		// First we need to downsample source images
		int previewWidth = 0; int previewHeight = 0;
		this->GetPresetThumbnailsSize(this->basePreviewPhotos[0]->GetWidth(), this->basePreviewPhotos[0]->GetHeight(), previewWidth, previewHeight);
		
		int rowStride = PhotoImage::CalculateRowStride(previewWidth, this->basePreviewPhotos[0]->GetSamplesPerPixel());
		size_t bufferSize = this->originalPhoto->IsHDR() ? previewWidth * previewHeight * 4 : rowStride * previewHeight * (this->basePreviewPhotos[0]->GetBitsPerSample() >> 3);
		imageBuffer = std::shared_ptr<std::vector<unsigned char>>(new std::vector <unsigned char>(bufferSize));
		int result = pm_downsample(this->basePreviewPhotos[0]->GetWidth(), this->basePreviewPhotos[0]->GetHeight(), this->basePreviewPhotos[0]->GetRowStride(), previewWidth, previewHeight,
								   previewWidth * this->basePreviewPhotos[0]->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetBitsPerSample(),
								   this->basePreviewPhotos[0]->GetImageBuffer()->data(), imageBuffer->data());
		
		thumbnailSourcePhoto = shared_ptr <PhotoImage>(new PhotoImage(*this->basePreviewPhotos[0]));
		
		thumbnailSourcePhoto->SetImageBuffer(imageBuffer);
		thumbnailSourcePhoto->SetSize(previewWidth, previewHeight);
		
		thumbnailImageBuffer = std::shared_ptr <std::vector <unsigned char>>(new vector <unsigned char>());
		if(this->hdrGenerator)
		{
			int imagesNum = this->hdrGenerator->GetInputPhotosCount();
			int sourceImageStride = this->previewSizes[0].width * this->previewSizes[0].height * this->hdrGenerator->GetSamplesPerPixel();
			int destImageStride = previewWidth * previewHeight * 3;
			thumbnailImageBuffer->resize(destImageStride * imagesNum);
			for(int i = 0; i < imagesNum; ++i)	{
				result = downsample_decimate(this->previewSizes[0].width, this->previewSizes[0].height, previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), 8, 8, 0, 0, this->basePreviewBuffers[0]->data() + i * sourceImageStride, thumbnailImageBuffer->data() + i * destImageStride);
				
				shared_ptr<PhotoImage> img(new PhotoImage(*this->hdrGenerator->GetInputPhoto(i)));
				img->SetWidth(previewWidth);
				img->SetHeight(previewHeight);
				img->SetBitsPerSample(8);
				
				this->fusionThumbImages.push_back(img);
			}
			
			if(this->hdrGenerator->IsGhostRemovalEnabled()) {
				ghostMask = shared_ptr<vector <unsigned char>> (new vector <unsigned char>(ceil((previewWidth * previewHeight) >> 3)));
				result = downsample_decimate(this->previewSizes[0].width, this->previewSizes[0].height, previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), 1, 1, 0, 0, this->ghostMasks[0]->data(), ghostMask->data());
				this->ghostMasks[this->currentZoomLevel] = ghostMask;
			}
		}

		
		// And then calculate preset thumbnails
		this->presetsForm->InitPresetsThumbnails(thumbnailSourcePhoto, this->fusionThumbImages, thumbnailImageBuffer, ghostMask);
		
		PhotomatixMisc::dbgprintf("End PreviewController::InitThumbnails()\n");
	}
	
	void PreviewController::InitPreviewImage()
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::InitPreviewImage()\n");
		
		// Downsample the image first
		if(!this->basePreviewPhotos[0])
		{
			int previewWidth = this->previewSizes[0].width;
			int previewHeight= this->previewSizes[0].height;
			
			
			int rowStride = PhotoImage::CalculateRowStride(previewWidth, this->originalPhoto->GetSamplesPerPixel());
			
			this->basePreviewPhotos[0] = shared_ptr <PhotoImage>(new PhotoImage(*this->originalPhoto));
			size_t bufferSize = this->originalPhoto->IsHDR() ? previewWidth * previewHeight * 4 : rowStride * previewHeight * (this->originalPhoto->GetBitsPerSample() >> 3);
			std::shared_ptr<std::vector<unsigned char>> imageBuffer(new std::vector <unsigned char>(bufferSize));
			int result = pm_downsample(this->originalPhoto->GetWidth(), this->originalPhoto->GetHeight(), this->originalPhoto->GetRowStride(), previewWidth, previewHeight,
									   previewWidth * this->originalPhoto->GetSamplesPerPixel(), this->originalPhoto->GetSamplesPerPixel(), this->originalPhoto->GetBitsPerSample(),
									   this->originalPhoto->GetImageBuffer()->data(), imageBuffer->data());
			
			this->basePreviewPhotos[0]->SetImageBuffer(imageBuffer);
			this->basePreviewPhotos[0]->SetSize(previewWidth, previewHeight);
			
			if(originalPhoto->GetSamplesPerPixel() == 3 && originalPhoto->GetImageFormat() != EImageFormatType::Raw && originalPhoto->GetImageFormat() != EImageFormatType::Hdr)
				bgr2rgb(basePreviewPhotos[0]->GetWidth(), basePreviewPhotos[0]->GetHeight(), basePreviewPhotos[0]->GetRowStride(), basePreviewPhotos[0]->GetSamplesPerPixel(), /*photo->GetBitsPerSample()*/originalPhoto->GetBitsPerSample(), basePreviewPhotos[0]->GetImageBuffer()->data());
			
			if(this->hdrGenerator)
			{
				int imagesNum = this->hdrGenerator->GetInputPhotosCount();
				int sourceImageStride = this->hdrGenerator->GetWidth() * this->hdrGenerator->GetHeight() * this->hdrGenerator->GetSamplesPerPixel() * (this->hdrGenerator->GetInputPhoto(0)->GetBitsPerSample() >> 3);
				int destImageStride = previewWidth * previewHeight * 3;
				this->basePreviewBuffers[0] = shared_ptr <vector <unsigned char> >(new vector <unsigned char>(destImageStride * imagesNum));
				for(int i = 0; i < imagesNum; ++i) {
					result = downsample_decimate(this->hdrGenerator->GetWidth(), this->hdrGenerator->GetHeight(), previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetInputPhoto(0)->GetBitsPerSample(), 8, 0, 0, this->hdrGenerator->GetImagesBuffer()->data() + i * sourceImageStride, this->basePreviewBuffers[0]->data() + i * destImageStride);
				}
				
				if(this->hdrGenerator->IsGhostRemovalEnabled()) {
					shared_ptr <vector <unsigned char> > ghostMask(new vector <unsigned char>(ceil((previewWidth * previewHeight) >> 3)));
					result = downsample_decimate(this->hdrGenerator->GetWidth(), this->hdrGenerator->GetHeight(), previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), 1, 1, 0, 0, this->hdrGenerator->GetGhostMask()->data(), ghostMask->data());
					this->ghostMasks[0] = ghostMask;
				}
			}
		}
		
		PhotomatixMisc::dbgprintf("End PreviewController::InitPreviewImage()\n");
	}
	
	void PreviewController::UpdatePreviewImage()
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::UpdatePreviewImage\n");
		
		try
		{
			// Don't run 2 processes at a time
			// TODO: maybe semaphore would be more appropriate here?
			if(this->isPreviewUpdating)
				return;
			
			this->isPreviewUpdating = true;
			
			this->previewForm->SetPreviewUpdating(true);
			
			PhotomatixMisc::dbgprintf("PreviewController::UpdatePreviewImage before InitPreviewImage()\n");
			
			this->InitPreviewImage();
			
			if(!this->basePreviewPhotos[currentZoomLevel])
			{
				int previewWidth = this->previewSizes[this->currentZoomLevel].width;
				int previewHeight= this->previewSizes[this->currentZoomLevel].height;
				
				
				int rowStride = PhotoImage::CalculateRowStride(previewWidth, this->originalPhoto->GetSamplesPerPixel());
				
				PhotomatixMisc::dbgprintf("prevW=%d prevH=%d r=%d\n", previewWidth, previewHeight, rowStride);
				
				shared_ptr <PhotoImage> basePreviewPhoto(new PhotoImage(*this->originalPhoto));
				size_t bufferSize = this->originalPhoto->IsHDR() ? previewWidth * previewHeight * 4 : rowStride * previewHeight * (this->originalPhoto->GetBitsPerSample() >> 3);
				std::shared_ptr<std::vector<unsigned char>> imageBuffer(new std::vector <unsigned char>(bufferSize));
				int result = pm_downsample(this->basePreviewPhotos[0]->GetWidth(), this->basePreviewPhotos[0]->GetHeight(), this->basePreviewPhotos[0]->GetRowStride(), previewWidth, previewHeight,
										previewWidth * this->basePreviewPhotos[0]->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetBitsPerSample(),
										this->basePreviewPhotos[0]->GetImageBuffer()->data(), imageBuffer->data());
				
				PhotomatixMisc::dbgprintf("dwn = %d\n", result);
				
				basePreviewPhoto->SetImageBuffer(imageBuffer);
				basePreviewPhoto->SetSize(previewWidth, previewHeight);
				
				this->basePreviewPhotos[this->currentZoomLevel] = basePreviewPhoto;
				
				if(this->hdrGenerator)
				{
					PhotomatixMisc::dbgprintf("PreviewController::UpdatePreviewImage before dd\n");
					
					int imagesNum = this->hdrGenerator->GetInputPhotosCount();
					int sourceImageStride = this->previewSizes[0].width * this->previewSizes[0].height * this->hdrGenerator->GetSamplesPerPixel();
					int destImageStride = previewWidth * previewHeight * 3;
					this->basePreviewBuffers[this->currentZoomLevel] = shared_ptr <vector <unsigned char> >(new vector <unsigned char>(destImageStride * imagesNum));
					for(int i = 0; i < imagesNum; ++i)
					{
						result = downsample_decimate(this->previewSizes[0].width, this->previewSizes[0].height, previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), 8, 8, 0, 0, this->basePreviewBuffers[0]->data() + i * sourceImageStride, this->basePreviewBuffers[this->currentZoomLevel]->data() + i * destImageStride);
						PhotomatixMisc::dbgprintf("ddr = %d\n", result);
					}
					
					if(this->hdrGenerator->IsGhostRemovalEnabled())
					{
						shared_ptr <vector <unsigned char> > ghostMask(new vector <unsigned char>(ceil((previewWidth * previewHeight) >> 3)));
						result = downsample_decimate(this->previewSizes[0].width, this->previewSizes[0].height, previewWidth, previewHeight, this->hdrGenerator->GetSamplesPerPixel(), this->hdrGenerator->GetSamplesPerPixel(), 1, 1, 0, 0, this->ghostMasks[0]->data(), ghostMask->data());
						this->ghostMasks[this->currentZoomLevel] = ghostMask;
						
						PhotomatixMisc::dbgprintf("ddr2 = %d\n", result);
					}
				}
			}
			
			bool needNewTm3InputBuffer = true;
			this->basePreviewW = this->previewSizes[this->currentZoomLevel].width; this->basePreviewH = this->previewSizes[this->currentZoomLevel].height;
			this->GetBasePreviewSizeForContrastOptimizer(this->basePreviewW, this->basePreviewH, this->originalPhoto->GetWidth(), this->originalPhoto->GetHeight(), basePreviewContrastOptimizerW, basePreviewContrastOptimizerH);
			//basePreviewContrastOptimizerW = basePreviewSizeTm3.Width; basePreviewContrastOptimizerH = basePreviewSizeTm3.Height;
			if (this->basePreviewContrastOptimizerW > this->previewSizes[0].width || this->basePreviewContrastOptimizerH > this->previewSizes[0].height)
			{
				this->basePreviewContrastOptimizerW = this->previewSizes[0].width;
				this->basePreviewContrastOptimizerH = this->previewSizes[0].height;
			}
			
			bool isSingleFile = this->hdrGenerator == NULL;
			long hdrBufSizeTm3 = isSingleFile ? (PhotoImage::CalculateRowStride(this->basePreviewContrastOptimizerW, this->basePreviewPhotos[0]->GetSamplesPerPixel())) * basePreviewContrastOptimizerH * (this->basePreviewPhotos[0]->GetBitsPerSample() >> 3) : basePreviewContrastOptimizerW * basePreviewContrastOptimizerH * 4;
			if (this->basePreviewInputContrastOptimizer != NULL && this->basePreviewInputContrastOptimizer->GetImageBuffer()->size() == hdrBufSizeTm3)
				needNewTm3InputBuffer = false;
			if (needNewTm3InputBuffer)
			{
				PhotomatixMisc::dbgprintf("PreviewController::UpdatePreviewImage before d\n");
				
				this->basePreviewInputContrastOptimizer = shared_ptr <PhotoImage>(new PhotoImage(this->basePreviewContrastOptimizerW, this->basePreviewContrastOptimizerH, this->originalPhoto->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetBitsPerSample(),
					this->basePreviewPhotos[0]->GetImageFormat()));
				this->basePreviewInputContrastOptimizer->SetImageFormat(this->originalPhoto->GetImageFormat());
				//this->basePreviewInputContrastOptimizer.ImageFormat = masterPhoto.ImageFormat;
				//this->basePreviewInputContrastOptimizer.ProfileLen = masterPhoto.ProfileLen;
				//this.basePreviewInputContrastOptimizer.IccProfile = masterPhoto.IccProfile;
				//this.basePreviewInputContrastOptimizer.FileName = photo.FileName;
				
				shared_ptr <vector <unsigned char>> imageBuffer(new vector <unsigned char>(hdrBufSizeTm3));
				int previewWidth = this->basePreviewContrastOptimizerW;
				int previewHeight = this->basePreviewContrastOptimizerH;
				int result = pm_downsample(this->basePreviewPhotos[0]->GetWidth(), this->basePreviewPhotos[0]->GetHeight(), this->basePreviewPhotos[0]->GetRowStride(), previewWidth, previewHeight,
							PhotoImage::CalculateRowStride(previewWidth, this->basePreviewPhotos[0]->GetSamplesPerPixel()), this->basePreviewPhotos[0]->GetSamplesPerPixel(), this->basePreviewPhotos[0]->GetBitsPerSample(),
							this->basePreviewPhotos[0]->GetImageBuffer()->data(), imageBuffer->data());
				this->basePreviewInputContrastOptimizer->SetImageBuffer(imageBuffer);
			}
		
			if(this->currentMethod <= 4) {
				this->mapper->SetDownsampleSize(0, 0);
				std::vector <std::shared_ptr <PhotoImage>> photos;
				if(this->currentMethod == 3)
				{
					//this.CheckIfOptimizerIsAlreadyMapped();
					photos.push_back(this->basePreviewInputContrastOptimizer);
					if (this->basePreviewContrastOptimizerW != this->basePreviewW && this->basePreviewContrastOptimizerH != this->basePreviewH)
					{
						this->mapper->SetWidth(this->basePreviewContrastOptimizerW);
						this->mapper->SetHeight(this->basePreviewContrastOptimizerH);
						//this.mapper.stride = this.basePreviewInputContrastOptimizer.RowStride;
						this->mapper->SetDownsampleSize(this->basePreviewW, this->basePreviewH);
					}
				}
				else
				{
					if(this->basePreviewPhotos[currentZoomLevel] != NULL)
						photos.push_back(this->basePreviewPhotos[this->currentZoomLevel]);
				}
				mapper->SetInputPhotos(photos);
				mapper->SetIsPreview(true);
				mapper->SetLoadImages(false);
				mapper->SetBitsPerSampleOut(8);
				if(mapperDone.empty())
					mapperDone = mapper->sig_done.connect(sigc::mem_fun(*this, &PreviewController::OnPreviewToneMappingDone));
				
				PhotomatixMisc::dbgprintf("PreviewController::UpdatePreviewImage before ms\n");
				
				mapper->Start();
			} else if(this->currentMethod == 10) {
				std::vector <std::shared_ptr <PhotoImage>> photos = this->hdrGenerator->GetInputPhotos();
				//if(this->basePreviewPhotos[currentZoomLevel] != NULL)
				//	photos.push_back(this->basePreviewPhotos[currentZoomLevel]);
				blender->SetInputPhotos(photos);
				blender->SetIsPreview(true);
				blender->SetWidth(this->previewSizes[this->currentZoomLevel].width);
				blender->SetHeight(this->previewSizes[this->currentZoomLevel].height);
				blender->SetBitsPerSample(8);
				blender->SetImagesBuffer(this->basePreviewBuffers[this->currentZoomLevel]);
				blender->SetLoadImages(false);
				blender->SetBitsPerSampleOut(8);
				blender->SetGhostMask(this->ghostMasks[this->currentZoomLevel]);
				blender->SetGhostRemovalEnabled(this->hdrGenerator->IsGhostRemovalEnabled());
				blender->SetGhostRemovalDetectionStrength(this->hdrGenerator->GetGhostRemovalDetectionStrength());
				if(blenderDone.empty())
					blenderDone = blender->sig_done.connect(sigc::mem_fun(*this, &PreviewController::OnPreviewFusingDone));
				
				PhotomatixMisc::dbgprintf("PreviewController::UpdatePreviewImage before bs\n");
				
				blender->Start();
			}
		} catch(std::exception& exc) {
			PhotomatixMisc::dbgprintf("Error in PreviewController::UpdatePreviewImage: %s\n", exc.what());
			
			Gtk::MessageDialog dialog("Error", false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
			dialog.set_secondary_text(exc.what());
			dialog.run();
		}
		
		PhotomatixMisc::dbgprintf("End PreviewController::UpdatePreviewImage\n");
	}
	
	void PreviewController::OnPreviewToneMappingDone()
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::OnPreviewToneMappingDone()\n");
		
		shared_ptr <PhotoImage> photo = this->mapper->GetOutputPhoto();
		
		PhotomatixMisc::dbgprintf("PreviewController::OnPreviewToneMappingDone() before previewForm->SetPreviewImage()\n");
		
		this->previewForm->SetPreviewImage(photo);
		
		PhotomatixMisc::dbgprintf("PreviewController::OnPreviewToneMappingDone() before previewForm->SetPreviewUpdating()\n");
		
		this->previewForm->SetPreviewUpdating(false);
		
		this->isPreviewUpdating = false;
		
		PhotomatixMisc::dbgprintf("End PreviewController::OnPreviewToneMappingDone()\n");
	}
	
	void PreviewController::OnPreviewFusingDone()
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::OnPreviewFusingDone()\n");
		
		shared_ptr <PhotoImage> photo = this->blender->GetOutputPhoto();
		
		PhotomatixMisc::dbgprintf("PreviewController::OnPreviewFusingDone() before previewForm->SetPreviewImage()\n");
		
		this->previewForm->SetPreviewImage(photo);
		
		PhotomatixMisc::dbgprintf("PreviewController::OnPreviewFusingDone() before previewForm->SetPreviewUpdating()\n");
		
		this->previewForm->SetPreviewUpdating(false);
		
		this->isPreviewUpdating = false;
		
		PhotomatixMisc::dbgprintf("End PreviewController::OnPreviewFusingDone()\n");
	}
	
	void PreviewController::OnThumbnailChanged(Preset* preset)
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::OnThumbnailChanged()\n");
	
		this->isChangingThumbnail = true;
		
		XmpSettings* settings = preset->GetSettings();
		
		this->previewForm->SetPreset(preset);
		this->previewForm->UpdateSliders(settings);
		this->previewForm->SetBlockUpdates(true);
		
		this->previewForm->SetProcessingMethod((PhotomatixImaging::EToneMappingMethod)settings->method);
		if(settings->method < 10)
		{	
			this->mapper->SetToneMappingMethod((PhotomatixImaging::EToneMappingMethod)settings->method);
			this->mapper->SetProcessingSettings(settings->parameters);
			
			if(settings->method == 3)
			{
				this->mapper->SetContrastOptimizerAlreadyMapped(false);
				
				if(fabsf(this->previousContrastOptimizerSettings.parameters[0] - settings->parameters[0]) < 0.0001f &&
					fabsf(this->previousContrastOptimizerSettings.parameters[1] - settings->parameters[1]) < 0.0001f &&
					fabsf(this->previousContrastOptimizerSettings.parameters[2] - settings->parameters[2]) < 0.0001f)
				{
					this->mapper->SetContrastOptimizerAlreadyMapped(true);
				}
				
				this->previousContrastOptimizerSettings.method = 3;
				memcpy(this->previousContrastOptimizerSettings.parameters, settings, sizeof(float) * PhotomatixMisc::XmpParamsLength);
			}
		}
		else
		{
			this->blender->SetExposureFusionMethod(EFusionMethod::FusionAdjust);
			this->blender->SetProcessingSettings(settings->parameters);
		}
		
		int oldMethod = this->currentMethod;
		
		this->currentMethod = settings->method;
		memcpy(this->currentSettings, settings->parameters, PhotomatixMisc::XmpParamsLength * sizeof(float));
		//if(this->currentMethod == oldMethod || oldMethod == 10)
		this->previewForm->SetBlockUpdates(false);
			this->UpdatePreviewImage();
			
		PhotomatixMisc::dbgprintf("End PreviewController::OnThumbnailChanged()\n");
		
		this->isChangingThumbnail = false;
	}
	
	void PreviewController::OnSettingsChanged(XmpSettings* settings)
	{
		PhotomatixMisc::dbgprintf("Start PreviewController::OnSettingsChanged()\n");
		
		bool sameSettings = true;
		if(!initialUpdate && settings->method == this->currentMethod) {
			for(int i = 0; i < 60; ++i) {
				if(this->currentSettings[i] != settings->parameters[i]) {
					sameSettings = false;
					break;
				}
			}
		} else
			sameSettings = false;
		if(initialUpdate) initialUpdate = false;
		if(sameSettings) return;
		
		if(!this->isChangingThumbnail)
			this->presetsForm->UnselectThumbnail();
		
		this->previewForm->UpdateSliders(settings);
		
		this->previewForm->SetProcessingMethod((PhotomatixImaging::EToneMappingMethod)settings->method);
		if(settings->method < 10)
		{	
			this->mapper->SetToneMappingMethod((PhotomatixImaging::EToneMappingMethod)settings->method);
			this->mapper->SetProcessingSettings(settings->parameters);
			
			if(settings->method == 3)
			{
				this->mapper->SetContrastOptimizerAlreadyMapped(false);
				
				if(fabsf(this->previousContrastOptimizerSettings.parameters[0] - settings->parameters[0]) < 0.0001f &&
					fabsf(this->previousContrastOptimizerSettings.parameters[1] - settings->parameters[1]) < 0.0001f &&
					fabsf(this->previousContrastOptimizerSettings.parameters[2] - settings->parameters[2]) < 0.0001f)
				{
					this->mapper->SetContrastOptimizerAlreadyMapped(true);
				}
				
				this->previousContrastOptimizerSettings.method = 3;
				memcpy(this->previousContrastOptimizerSettings.parameters, settings, sizeof(float) * PhotomatixMisc::XmpParamsLength);
			}
		}
		else
		{
			this->blender->SetExposureFusionMethod(EFusionMethod::FusionAdjust);
			this->blender->SetProcessingSettings(settings->parameters);
		}
		
		this->currentMethod = settings->method;
		memcpy(this->currentSettings, settings->parameters, PhotomatixMisc::XmpParamsLength * sizeof(float));
		this->UpdatePreviewImage();
		
		PhotomatixMisc::dbgprintf("End PreviewController::OnSettingsChanged()\n");
	}

}
