#include "previewwindow.h"
#include "presetthumbnail.h"
#include "../PhotomatixImaging/photoimage.h"
#include <iostream>
#include "../PhotomatixMisc/ResourceUtils.h"
#include "../PhotomatixMisc/appproperties.h"
#include "../PhotomatixMisc/localtext.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/PhotomatixMetaLib.h"
#include "../PhotomatixMisc/dbgprintf.h"
#include "../PhotomatixMisc/stringutils.h"

#include <dirent.h>

using namespace std;
using namespace PhotomatixControls;
using namespace PhotomatixImaging;
using namespace PhotomatixMisc;


namespace Photomatix
{
	PreviewWindow::PreviewWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& gladeBuilder):
		Gtk::Window(cobject),
		builder(gladeBuilder),
		currentZoomLevel(0)
	{		
	
		this->blockUpdates = false;
		
		// Get widgets
		builder->get_widget("lblRefreshing", lblRefreshing);
		builder->get_widget("btnZoomOut", btnZoomOut);
		builder->get_widget("btnZoomIn", btnZoomIn);
		builder->get_widget("btnProcess", btnProcess);
		builder->get_widget("cbxProcessingMethod", cbxProcessingMethod);
		/*builder->get_widget("rbnExposureFusion", rbnExposureFusion);		
		builder->get_widget("rbnDetailsEnhancer", rbnDetailsEnhancer);
		builder->get_widget("rbnContrastOptimizer", rbnContrastOptimizer);
		builder->get_widget("rbnToneBalancer", rbnToneBalancer);
		builder->get_widget("rbnToneCompressor", rbnToneCompressor);*/
		
		builder->get_widget("cbxPreset", cbxPreset);
		
		builder->get_widget("layout-main", layoutMain);
		builder->get_widget("settings-box", settingsBox);
		builder->get_widget("presets-thumbnails-box", presetsBox);
		builder->get_widget("preview-box", previewBox);
		builder->get_widget("zoom-controls", zoomControls);
		
		builder->get_widget("button-built-in", btnPresetsBuiltIn);
		builder->get_widget("button-my-presets", btnPresetsCustom);
		
		builder->get_widget("scrolledwindow1", scrolledWindow);
		
		Glib::RefPtr <Gdk::Screen> screen = Gdk::Screen::get_default();
		Gdk::Rectangle workArea = screen->get_monitor_workarea();
		if(workArea.get_height() > 800)
			scrolledWindow->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
		
		
		// Assign signals/events
		this->cbxProcessingMethod->signal_changed().connect(sigc::mem_fun(*this, &PreviewWindow::OnProcessingMethodChanged));
		
		this->btnZoomIn->signal_clicked().connect(sigc::mem_fun(*this, &PreviewWindow::OnZoomInButtonClicked));
		this->btnZoomOut->signal_clicked().connect(sigc::mem_fun(*this, &PreviewWindow::OnZoomOutButtonClicked));
		this->btnProcess->signal_clicked().connect(sigc::mem_fun(*this, &PreviewWindow::OnProcessButtonClicked));
		
		this->presetChangedSignal = this->cbxPreset->signal_changed().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetSelectedIndexChanged));
		
		
		this->btnPresetsBuiltIn->set_active(PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected());
		this->btnPresetsCustom->set_active(!PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected());
		
		this->btnPresetsBuiltInSignalChanged = this->btnPresetsBuiltIn->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsBuiltInClicked));
		this->btnPresetsCustomSignalChanged =this->btnPresetsCustom->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsCustomClicked));
		
		
		// Create settings controls
		Glib::RefPtr<Gtk::Builder> refBuilder = Gtk::Builder::create();
		refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("details-enhancer-settings").c_str());
		//refBuilder->get_widget_derived(name, widget);
		refBuilder->get_widget_derived("contents", this->deSettingsControl);
		this->deSettingsControl->reparent(*this);
		this->deSettingsSignal = this->deSettingsControl->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewWindow::OnSettingsChanged));
		
		refBuilder = Gtk::Builder::create();
		refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("tone-balancer-settings").c_str());
		refBuilder->get_widget_derived("contents", this->tbSettingsControl);
		this->tbSettingsControl->reparent(*this);
		this->tbSettingsSignal = this->tbSettingsControl->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewWindow::OnSettingsChanged));
		
		refBuilder = Gtk::Builder::create();
		refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("tone-compressor-settings").c_str());
		refBuilder->get_widget_derived("contents", this->tcSettingsControl);
		this->tcSettingsControl->reparent(*this);
		this->tcSettingsSignal = this->tcSettingsControl->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewWindow::OnSettingsChanged));
		
		refBuilder = Gtk::Builder::create();
		refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("contrast-optimizer-settings").c_str());
		refBuilder->get_widget_derived("contents", this->coSettingsControl);
		this->coSettingsControl->reparent(*this);
		this->coSettingsSignal = this->coSettingsControl->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewWindow::OnSettingsChanged));
		
		refBuilder = Gtk::Builder::create();
		refBuilder->add_from_file(PhotomatixUtils::GetGladeResourcePath("exposure-fusion-settings").c_str());
		refBuilder->get_widget_derived("contents", this->efSettingsControl);
		this->efSettingsControl->reparent(*this);
		this->efSettingsSignal = this->efSettingsControl->signal_settings_changed.connect(sigc::mem_fun(*this, &PreviewWindow::OnSettingsChanged));
		
		this->AddControl("settings-control", this->deSettingsControl);
		
		
		this->move(0, 0);
	}

	PreviewWindow::~PreviewWindow()
	{

	}
	
	void PreviewWindow::OnProcessingMethodChanged()
	{
		if(!this->blockUpdates)
			this->cbxPreset->set_active(-1);
			
		
		int selectedMethodIdx = this->cbxProcessingMethod->get_active_row_number();
		// Details Enhancer
		if(selectedMethodIdx == 0) {
			this->RemoveControl("settings-control", this->tbSettingsControl);
			this->RemoveControl("settings-control", this->tcSettingsControl);
			this->RemoveControl("settings-control", this->coSettingsControl);
			this->RemoveControl("settings-control", this->efSettingsControl);
			
			this->AddControl("settings-control", this->deSettingsControl);
			
			this->signal_settings_changed.emit(this->deSettingsControl->GetCurrentSettings());
		}
		// Contrast Optimizer
		else if(selectedMethodIdx == 1) {
			this->RemoveControl("settings-control", this->deSettingsControl);
			this->RemoveControl("settings-control", this->tbSettingsControl);
			this->RemoveControl("settings-control", this->tcSettingsControl);
			this->RemoveControl("settings-control", this->efSettingsControl);
			
			this->AddControl("settings-control", this->coSettingsControl);
			
			this->signal_settings_changed.emit(this->coSettingsControl->GetCurrentSettings());
		}
		// Tone Balancer
		else if(selectedMethodIdx == 2) {
			this->RemoveControl("settings-control", this->deSettingsControl);
			this->RemoveControl("settings-control", this->tcSettingsControl);
			this->RemoveControl("settings-control", this->coSettingsControl);
			this->RemoveControl("settings-control", this->efSettingsControl);
			
			this->AddControl("settings-control", this->tbSettingsControl);
			
			this->signal_settings_changed.emit(this->tbSettingsControl->GetCurrentSettings());
		}
		// Tone Compressor
		else if(selectedMethodIdx == 3) {
			this->RemoveControl("settings-control", this->deSettingsControl);
			this->RemoveControl("settings-control", this->tbSettingsControl);
			this->RemoveControl("settings-control", this->coSettingsControl);
			this->RemoveControl("settings-control", this->efSettingsControl);
			
			this->AddControl("settings-control", this->tcSettingsControl);
			
			this->signal_settings_changed.emit(this->tcSettingsControl->GetCurrentSettings());
		} else if(selectedMethodIdx == 4) {
			this->RemoveControl("settings-control", this->coSettingsControl);
			this->RemoveControl("settings-control", this->tbSettingsControl);
			this->RemoveControl("settings-control", this->tcSettingsControl);
			this->RemoveControl("settings-control", this->deSettingsControl);
			this->AddControl("settings-control", this->efSettingsControl);
			
			if(!this->blockUpdates)
				this->signal_settings_changed.emit(this->efSettingsControl->GetCurrentSettings());
		}
	}
	
	void PreviewWindow::SetThumbnailsTabsEnability(bool enabled)
	{
		this->btnPresetsBuiltIn->set_sensitive(enabled);
		this->btnPresetsCustom->set_sensitive(enabled);
	}
	
	void PreviewWindow::Init()
	{
		// Initialize combo box
		this->InitPresetComboBox();
	}
	
	void PreviewWindow::InitPresetComboBox()
	{
		// Create source files list model
		this->presetFileList = Gtk::ListStore::create(this->presetFileColumns);
		this->cbxPreset->set_model(this->presetFileList);
		//this->cbxPreset->set_id_column(0);
		
		
		this->cbxPreset->set_row_separator_func(sigc::mem_fun(*this, &PreviewWindow::IsRowItemSeparator));
		
		Gtk::TreeModel::Row row = *(this->presetFileList->append());
		row[this->presetFileColumns.shortName] = LocalText::GetSavePresetTitle();
		row[this->presetFileColumns.fullPath] = "";
		
		row = *(this->presetFileList->append());
		row[this->presetFileColumns.shortName] = LocalText::GetLoadPresetTitle();
		row[this->presetFileColumns.fullPath] = "";
		
		row = *(this->presetFileList->append());
		row[this->presetFileColumns.shortName] = "-";
		row[this->presetFileColumns.fullPath] = "";
		
		// Add built-in presets
		vector<string> presetNames = PhotomatixUtils::split(LocalText::GetTextBuiltInPresetNamesShortList(), ',');//  {"Detailed", "Balanced", "Realistic", "Photographic", "Natural", "Painterly", "Vibrant", "Enhanced", "Deep", "Surreal", "Grunge", "Creative", "Realistic 2", "Soft", "Smooth", "Monochrome"};
		vector<string> files = {"-1", "-3", "-4", "-2", "-10", "Painterly.xmp", "Vibrant.xmp", "Enhanced.xmp", "CompressorDeep.xmp", "Surreal.xmp", "Grunge.xmp", "Creative.xmp", "Realistic2.xmp", "Soft.xmp", "Smooth.xmp", "DefaultBW.xmp"};
		
		for(int i = 0; i < files.size(); ++i)
		{
			// Skip "Natural" preset in case of single image processing
			if(presetNames[i] == "Natural" && this->imagesNumber == 1)
				continue;
			
			row = *(this->presetFileList->append());
			row[this->presetFileColumns.shortName] = presetNames[i];
			string fileName;
			if(files[i][0] == '-')
				fileName = files[i];
			else
			{
#ifdef Release
				fileName = string("/usr/share/photomatix/BuiltinPresets/") + files[i];
#else
				fileName = string("BuiltinPresets/") + files[i];
#endif
			}
			
			row[this->presetFileColumns.fullPath] = fileName;
		}
		
		
		// List of built-in presets
		DIR *dir;
		class dirent *ent;
		class stat st;
		
		string directory = PhotomatixMisc::AppProperties::GetUserPath() + "presets";
		struct stat buf;
		// Check if directory exists
		if (stat(directory.c_str(), &st) == 0)
		{
			dir = opendir(directory.c_str());
			while ((ent = readdir(dir)) != NULL) {
				const string fileName = ent->d_name;
				const string fullFileName = directory + "/" + fileName;
				
				if (fileName[0] == '.')
					continue;
				
				// Check if file still exists
				if (stat(fullFileName.c_str(), &st) == -1)
					continue;
				
				// Check if it's really a file
				const bool is_directory = (st.st_mode & S_IFDIR) != 0;
				if (!is_directory)
				{
					// Check if contains valid xmp settings
					int method = read_xmp(fullFileName.c_str(), -1, NULL);
					if(method == 1 || method == 2 || method == 3 || (method == 10 && this->imagesNumber > 1))
					{
						int index = fileName.find_last_of('.');
						string presetName = fileName.substr(0, index);
						
						PresetListEntry preset;
						preset.shortName = presetName;
						preset.fullPath = fullFileName;
						
						this->allPresetFiles.push_back(preset);
					}
				}
			}
			closedir(dir);
		}
		// If directory doesn't exist - create it
		else
		{
			mkdir(directory.c_str(), 0700);
		}
		
		if(this->allPresetFiles.size())
		{
			row = *(this->presetFileList->append());
			row[this->presetFileColumns.shortName] = "-";
			row[this->presetFileColumns.fullPath] = "";
			
			std::sort(allPresetFiles.begin(), allPresetFiles.end());
			
			for(int i = 0; i < this->allPresetFiles.size(); ++i)
			{
				row = *(this->presetFileList->append());
				row[this->presetFileColumns.shortName] = this->allPresetFiles[i].shortName;
				row[this->presetFileColumns.fullPath] = this->allPresetFiles[i].fullPath;
			}
		}
		
		this->cbxPreset->set_size_request(190, 20);
		
		Gtk::CellRendererText cell;// = gtk.CellRendererText()
		cell.property_ellipsize().set_value(Pango::ELLIPSIZE_END);
		//cell.pack_start(this->presetFileColumns.shortName);
		//cell.props.ellipsize = pango.ELLIPSIZE_END
		//ls = gtk.ListStore( object)
		//cb = gtk.ComboBox( ls)
		
		//this->cbxPreset->pack_start(cell, true);
		
		//Gtk::CellRendererText cell2;// = gtk.CellRendererText()
		//cell2.property_ellipsize().set_value(Pango::ELLIPSIZE_END);
		
		//this->presetFileColumns.shortName.pack_start(cell, false);
		//this->presetFileColumns.shortName.pack_start(cell2, false);
		
		//this->cbxPreset->pack_start(this->presetFileColumns.shortName);
		this->cbxPreset->pack_start(cell, false);
		//this->cbxPreset->pack_start(cell2, false);
		
		this->cbxPreset->add_attribute(cell, "text", 0);
	}
	
	void PreviewWindow::OnPresetSelectedIndexChanged()
	{
		int index = this->cbxPreset->get_active_row_number();
		Glib::ustring fileName = "";
		Gtk::TreeModel::iterator iter = this->cbxPreset->get_active();
		if(iter) {
			Gtk::TreeModel::Row row = *iter;
			if(row) {
				fileName = row[this->presetFileColumns.fullPath];
			}
		}
		
		
		this->signal_preset_changed.emit(index, fileName);
		
		//this->cbxPreset->set_active(-1);
	}
	
	bool PreviewWindow::IsRowItemSeparator(const Glib::RefPtr< Gtk::TreeModel >& model, const Gtk::TreeModel::iterator& iter)
	{
		Gtk::TreeModel::Row row = *iter;
		string value;
		row->get_value(0, value);
		if(value == "-")
			return true;
		
		return false;
	}
	
	void PreviewWindow::SetPresetIndex(int index, bool silent)
	{
		if(silent) this->presetChangedSignal.disconnect();
		
		this->cbxPreset->set_active(index);
		
		if(silent) this->presetChangedSignal = this->cbxPreset->signal_changed().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetSelectedIndexChanged));
	}
	
	int PreviewWindow::GetPresetIndex()
	{
		return this->cbxPreset->get_active_row_number();
	}
	
	void PreviewWindow::SetProcessingMethod(int method)
	{		
		if(method == 1) this->cbxProcessingMethod->set_active(0);
		else if(method == 2) this->cbxProcessingMethod->set_active(3);
		else if(method == 3) this->cbxProcessingMethod->set_active(1);
		else if(method == 4) this->cbxProcessingMethod->set_active(2);
		else if(method == 10) this->cbxProcessingMethod->set_active(4);
	}
	
	void PreviewWindow::OnSettingsChanged(XmpSettings* settings)
	{
		this->cbxPreset->set_active(-1);
		
		this->signal_settings_changed.emit(settings);
	}
	
	void PreviewWindow::OnProcessButtonClicked()
	{
		this->signal_final_processing.emit();
	}
	
	void PreviewWindow::SetImagesNum(int imagesNum)
	{
		this->imagesNumber = imagesNum;
		if(this->imagesNumber > 1) {
			this->cbxProcessingMethod->append(LocalText::GetTextFusionNatural());
		}
	}
	
	void PreviewWindow::SetPreviewUpdating(bool updating)
	{
		PhotomatixMisc::dbgprintf("PreviewWindow::SetPreviewUpdating(): updating=%d\n", updating ? 1 : 0);
		
		this->lblRefreshing->set_visible(updating);
		
		// Set enability of controls
		this->btnZoomIn->set_sensitive(!updating && (this->currentZoomLevel != 0));
		this->btnZoomOut->set_sensitive(!updating && (this->currentZoomLevel != this->zoomInLevels + this->zoomOutLevels));
		this->btnProcess->set_sensitive(!updating);
	}
	
	bool PreviewWindow::on_delete_event(GdkEventAny* event)
	{
		this->signal_preview_closed.emit();
		return false;
	}

	void PreviewWindow::SetPreviewImage(std::shared_ptr <PhotomatixImaging::PhotoImage> photo)
	{
		PhotomatixMisc::dbgprintf("Start PreviewWindow::SetPreviewImage()\n");
		
		bool start = false;
		if(!this->displayableImageControl)
		{
			this->displayableImageControl = shared_ptr<ScrollableDisplayableImageControl>(new ScrollableDisplayableImageControl());
			this->displayableImageControl->set_halign(Gtk::ALIGN_CENTER);
			this->displayableImageControl->set_valign(Gtk::ALIGN_CENTER);
			start = true;
		}
		
		PhotomatixMisc::dbgprintf("PreviewWindow::SetPreviewImage() before displayableImageControl->SetPhoto()\n");
		
		this->displayableImageControl->SetPhoto(photo);
		
		Gtk::Box* box=0;
		this->builder->get_widget("preview-image-box", box);
		
		if(box && start)
		{
			//this->displayableImageControl = shared_ptr<DisplayableImageControl>(new DisplayableImageControl());
			
			box->pack_start(*this->displayableImageControl);
			//this->displayableImageControl->ShowScrollbars(true);
		}
		
		PhotomatixMisc::dbgprintf("PreviewWindow::SetPreviewImage() before show_all_children()\n");
		
		this->show_all_children();
		
		Glib::RefPtr <Gdk::Screen> screen = Gdk::Screen::get_default();
		Gdk::Rectangle workArea = screen->get_monitor_workarea();
		
		int newWidth, newHeight;
		
		// Calculate maximum size of preview image
		int titleBarHeight = std::max <int>(28, this->get_height() - this->layoutMain->get_height());
		int reduceWidth = this->layoutMain->get_margin_left() + this->settingsBox->get_width() + this->layoutMain->get_spacing() * 2 + this->presetsBox->get_width() + this->layoutMain->get_margin_right();
		int reduceHeight = this->layoutMain->get_margin_top() + this->previewBox->get_margin_top() + this->zoomControls->get_height() + this->previewBox->get_spacing() + this->layoutMain->get_margin_bottom() + titleBarHeight * 2;
		
		newWidth = std::min <int>(photo->GetWidth(), workArea.get_width() - reduceWidth);
		newHeight= std::min <int>(photo->GetHeight(), workArea.get_height() - reduceHeight);
		
		PhotomatixMisc::dbgprintf("PreviewWindow::SetPreviewImage() before displayableImageControl->SetZoomFactor()\n");
		
		this->displayableImageControl->SetZoomFactor(1.0f);
		//this->displayableImageControl->set_size_request(photo->GetWidth(), photo->GetHeight());
		this->displayableImageControl->set_size_request(newWidth, newHeight);
		
		// Needs to be not resizable because otherwise the window won't shrink
		/// TODO: maybe it would be enough to first set_resizable(false) and after resizing takes place
		/// set it back to true?
		this->set_resizable(true);
		
		PhotomatixMisc::dbgprintf("End PreviewWindow::SetPreviewImage()\n");
	}
	
	void PreviewWindow::OnZoomInButtonClicked()
	{
		if (this->currentZoomLevel > 0)
		{
			--this->currentZoomLevel;
			
			this->btnZoomOut->set_sensitive(true);
			
			if(this->currentZoomLevel == 0)
				this->btnZoomIn->set_sensitive(false);
			
			this->signal_zoom_level_changed.emit(this->currentZoomLevel);
		}
	}
	
	void PreviewWindow::OnZoomOutButtonClicked()
	{
		if (this->currentZoomLevel < zoomInLevels + zoomOutLevels)
		{
			++this->currentZoomLevel;
			
			this->btnZoomIn->set_sensitive(true);
			
			if(this->currentZoomLevel == zoomInLevels + zoomOutLevels)
				this->btnZoomOut->set_sensitive(false);
			
			this->signal_zoom_level_changed.emit(this->currentZoomLevel);
		}
	}
	
	void PreviewWindow::OnPresetsBuiltInClicked()
	{
		// Already selected button was clicked (make sure to reselect it)
		if(!this->btnPresetsBuiltIn->get_active() && !this->btnPresetsCustom->get_active())
		{
			this->btnPresetsBuiltInSignalChanged.disconnect();
			this->btnPresetsBuiltIn->set_active(true);
			this->btnPresetsBuiltInSignalChanged = this->btnPresetsBuiltIn->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsBuiltInClicked));
			
			return;
		}
		
		// Deselect the other button
		this->btnPresetsCustomSignalChanged.disconnect();
		this->btnPresetsCustom->set_active(false);
		this->btnPresetsCustomSignalChanged = this->btnPresetsCustom->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsCustomClicked));
		
		
		this->signal_presets_selected.emit(true);
	}

	void PreviewWindow::OnPresetsCustomClicked()
	{
		// Already selected button was clicked (make sure to reselect it)
		if(!this->btnPresetsBuiltIn->get_active() && !this->btnPresetsCustom->get_active())
		{
			this->btnPresetsCustomSignalChanged.disconnect();
			this->btnPresetsCustom->set_active(true);
			this->btnPresetsCustomSignalChanged = this->btnPresetsCustom->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsCustomClicked));
			
			return;
		}
		
		// Deselect the other button
		this->btnPresetsBuiltInSignalChanged.disconnect();
		this->btnPresetsBuiltIn->set_active(false);
		this->btnPresetsBuiltInSignalChanged = this->btnPresetsBuiltIn->signal_toggled().connect(sigc::mem_fun(*this, &PreviewWindow::OnPresetsBuiltInClicked));
		
		
		this->signal_presets_selected.emit(false);
	}
	
	void PreviewWindow::AddControl(const std::string& parentControl, Gtk::Widget* widget)
	{
		Gtk::Box* box = 0;
		builder->get_widget(parentControl, box);
		if(box)
			box->pack_start(*widget);
	}
	
	void PreviewWindow::RemoveControl(const std::string& parentControl, Gtk::Widget* widget)
	{
		Gtk::Box* box = 0;
		builder->get_widget(parentControl, box);
		if(box)
			box->remove(*widget);
	}
	
	void PreviewWindow::UpdateSliders(XmpSettings* settings)
	{
		// Details Enhancer
		if(settings->method == 1) {
			this->deSettingsSignal.block();
				this->deSettingsControl->SetSettings(settings);
			this->deSettingsSignal.unblock();
		}
		// Tone Compressor
		else if(settings->method == 2) {
			this->tcSettingsSignal.block();
				this->tcSettingsControl->SetSettings(settings);
			this->tcSettingsSignal.unblock();
		}
		// Contrast Optimizer
		else if(settings->method == 3) {
			this->coSettingsSignal.block();
				this->coSettingsControl->SetSettings(settings);
			this->coSettingsSignal.unblock();
		}
		// Tone Balancer
		else if(settings->method == 4) {
			this->tbSettingsSignal.block();
				this->tbSettingsControl->SetSettings(settings);
			this->tbSettingsSignal.unblock();
		}
		// Fusion Natural
		else if(settings->method == 10) {
			this->efSettingsSignal.block();
				this->efSettingsControl->SetSettings(settings);
			this->efSettingsSignal.unblock();
		}
	}

	void PreviewWindow::SetPreviewZoomLevel(int level)
	{
		if(level >= 0 && level <= zoomInLevels + zoomOutLevels) {
			this->currentZoomLevel = level;
			
			if(this->currentZoomLevel == 0)
				this->btnZoomIn->set_sensitive(false);
			
			if(this->currentZoomLevel == zoomInLevels + zoomOutLevels)
				this->btnZoomOut->set_sensitive(false);
				
			this->signal_zoom_level_changed.emit(this->currentZoomLevel);
		}
	}
	
	void PreviewWindow::SetPreset(Preset* preset)
	{
		string& presetName = preset->GetDisplayName();
		
		Gtk::TreeModel::iterator iter = this->cbxPreset->get_model()->children().begin();
		Gtk::TreeModel::iterator iterEnd = this->cbxPreset->get_model()->children().end();
		int index = 0;
		bool presetSet = false;
		while(iter != iterEnd)
		{
			Gtk::TreeModel::Row row = *iter;
			if(row) {
				Glib::ustring displayName = row[this->presetFileColumns.shortName];
				if(displayName == presetName)
				{
					this->SetPresetIndex(index, true);
					presetSet = true;
					break;
				}
			}
			
			++index;
			++iter;
		}
		
		if(!presetSet)
			this->cbxPreset->set_active(-1);
	}

}