#ifndef PREVIEWWINDOW_H
#define PREVIEWWINDOW_H


#include "scrollabledisplayableimagecontrol.h"
#include <gtkmm.h>
#include "../PhotomatixControls/detailsenhancersettings.h"
#include "../PhotomatixControls/tonebalancersettings.h"
#include "../PhotomatixControls/tonecompressorsettings.h"
#include "../PhotomatixControls/contrastoptimizersettings.h"
#include "../PhotomatixControls/exposurefusionsettings.h"


namespace Photomatix
{
	class PreviewWindow: public Gtk::Window
	{
	protected:
		struct PresetListEntry
		{
			std::string shortName;
			std::string fullPath;
			
			bool operator < (const PresetListEntry& rhs) const
			{
				return (shortName < rhs.shortName);
			}
		};
		
		std::vector <PresetListEntry> allPresetFiles;
		Glib::RefPtr<Gtk::ListStore> presetFileList;
		
		/**
		 * Data model for our file list.  The data model for this file
		 * list contains two fields: a full path to the location of the
		 * file, and the short filename that is displayed to the user
		 */
		class PresetFileColumns : public Gtk::TreeModel::ColumnRecord
		{
		public:
			
			PresetFileColumns() {add(shortName); add(fullPath);}
			
			/** short name that is displayed to the user */
			Gtk::TreeModelColumn<Glib::ustring> shortName;
			/** full path that is stored hidden to access the file */
			Gtk::TreeModelColumn<Glib::ustring> fullPath;
		} presetFileColumns;
		
		sigc::connection deSettingsSignal;
		sigc::connection tbSettingsSignal;
		sigc::connection tcSettingsSignal;
		sigc::connection coSettingsSignal;
		sigc::connection efSettingsSignal;
		sigc::connection rbnExposureFusionToggled;
		sigc::connection presetChangedSignal;
		
		sigc::connection btnPresetsBuiltInSignalChanged;
		sigc::connection btnPresetsCustomSignalChanged;
		
		/// Displayable image control used for displaying a photo
		std::shared_ptr <PhotomatixControls::ScrollableDisplayableImageControl> displayableImageControl;
		
		/// Glade builder used to access widgets
		Glib::RefPtr<Gtk::Builder> builder;
		
		Gtk::Box* layoutMain;
		Gtk::Box* settingsBox;
		Gtk::Box* presetsBox;
		Gtk::Box* previewBox;
		Gtk::Grid* zoomControls;
		
		Gtk::Label* lblRefreshing;
		Gtk::Button* btnZoomIn;
		Gtk::Button* btnZoomOut;
		Gtk::Button* btnProcess;
		
		Gtk::ToggleButton* btnPresetsBuiltIn;
		Gtk::ToggleButton* btnPresetsCustom;
		
		Gtk::ScrolledWindow* scrolledWindow;
		
		Gtk::ComboBoxText *cbxProcessingMethod;
		/*Gtk::RadioButton* rbnExposureFusion;
		Gtk::RadioButton* rbnDetailsEnhancer;
		Gtk::RadioButton* rbnContrastOptimizer;
		Gtk::RadioButton* rbnToneBalancer;
		Gtk::RadioButton* rbnToneCompressor;*/
		Gtk::ComboBox *cbxPreset;
		
		/// Settings controls
		PhotomatixControls::DetailsEnhancerSettings* deSettingsControl;
		PhotomatixControls::ToneBalancerSettings* tbSettingsControl;
		PhotomatixControls::ToneCompressorSettings* tcSettingsControl;
		PhotomatixControls::ContrastOptimizerSettings* coSettingsControl;
		PhotomatixControls::ExposureFusionSettings* efSettingsControl;
		
		int currentZoomLevel;
		int zoomInLevels;
		int zoomOutLevels;
		int imagesNumber;
		bool blockUpdates;
		
		
		void OnSettingsChanged(PhotomatixImaging::XmpSettings* settings);
		void OnZoomInButtonClicked();
		void OnZoomOutButtonClicked();
		void OnProcessButtonClicked();
		void OnProcessingMethodChanged();
		
		void OnPresetsBuiltInClicked();
		void OnPresetsCustomClicked();
		
		void OnPresetSelectedIndexChanged();
		bool IsRowItemSeparator(const Glib::RefPtr<Gtk::TreeModel>&, const Gtk::TreeModel::iterator&);
		
		/// Used to capture close event to do additional stuff.
		bool on_delete_event(GdkEventAny* event);
		
		
		void InitPresetComboBox();
		
	public:
		PreviewWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
		virtual ~PreviewWindow();
		
		void SetPreviewImage(std::shared_ptr <PhotomatixImaging::PhotoImage> photo);
		void SetProcessingMethod(int method);
		void SetPreviewUpdating(bool updating);
		void ZoomFactorChanged() {}
		void SetImagesNum(int imagesNum);
		
		void SetPreset(PhotomatixImaging::Preset* preset);
		void UpdateSliders(PhotomatixImaging::XmpSettings* settings);
		
		void AddControl(const std::string& parentControl, Gtk::Widget* widget);
		void RemoveControl(const std::string& parentControl, Gtk::Widget* widget);
		
		void SetZoomLevels(int zoomIn, int zoomOut) {this->zoomInLevels = zoomIn; this->zoomOutLevels = zoomOut; this->currentZoomLevel = this->zoomInLevels;}
		void SetPreviewZoomLevel(int level);
		int GetPreviewZoomLevel() const {return this->currentZoomLevel;}
		
		void SetBlockUpdates(bool value) {this->blockUpdates = value;}
		void SetThumbnailsTabsEnability(bool enabled);
		
		void SetPresetIndex(int index, bool silent);
		int GetPresetIndex();
		
		void Init();
		
		sigc::signal <void, int> signal_zoom_level_changed;
		sigc::signal <void> signal_preview_closed;
		sigc::signal <void> signal_final_processing;
		sigc::signal <void, PhotomatixImaging::XmpSettings*> signal_settings_changed;
		sigc::signal <void, int, std::string> signal_preset_changed;
		sigc::signal <void, bool> signal_presets_selected;
	};
}

	
#endif // PREVIEWWINDOW_H
