#ifndef PROCESSPHOTOCONTROLLER_H
#define PROCESSPHOTOCONTROLLER_H


#include <memory>
#include <string>

#include "saveimageform.h"


namespace PhotomatixImaging
{
	class PhotoImage;
}

namespace Photomatix
{
	// Some forward declarations
	class DisplayableImageForm;
	class HdrController;
	class PreviewController;
	class FinishingTouchController;
	
	
	/** Controller responsible for processing photos.
	*/
	class ProcessPhotoController
	{
	private:
		std::shared_ptr <HdrController> hdrController;
		std::shared_ptr <PreviewController> previewController;
		std::shared_ptr <FinishingTouchController> finishingTouchController;
		std::shared_ptr <PhotomatixImaging::PhotoImage> finalImage;
		
	protected:
		DisplayableImageForm* window;					///< Image window managed by this controller
		
		SaveImageForm saveFileDialog;
		
		
		
		void OnPreviewClosed();
		void OnPreviewFinished(std::shared_ptr<PhotomatixImaging::PhotoImage> outputImage);
		void OnSaveDialogFilterChanged();
		void OnFinishingTouchFinished(std::shared_ptr<PhotomatixImaging::PhotoImage> outputImage);
		void OnFinishingTouchClosed(std::shared_ptr<PhotomatixImaging::PhotoImage> outputImage);
		void OnFinishingTouchPreviewUpdated(std::shared_ptr<PhotomatixImaging::PhotoImage> previewImage);
		void OnSaveImageRequested();
		
		void CreateImageWindow();
		void SaveFinalImage(std::shared_ptr<PhotomatixImaging::PhotoImage> outputImage);
		
	public:
		ProcessPhotoController();
		virtual ~ProcessPhotoController();
		
		void ShowImageWindow();
		void ImageWindowOpenFile(const std::string& fileName);
		
		/// TODO: might be better to create it on ProcessPhotoController side
		std::shared_ptr <HdrController> GetHdrController() {return this->hdrController;}
		DisplayableImageForm* GetImageWindow() const {return this->window;}
		void SetImageWindow(DisplayableImageForm* imageWindow) {this->window = imageWindow;}
		
		/** Event handlers */
		void OnCreateInstanceRequested(std::string fileName);
		void OnToneMappingRequested();
		//@}
	};
}

#endif // PROCESSPHOTOCONTROLLER_H
