#include "progressform.h"
#include "linuxprogressnotifier.h"

#include <iostream>


namespace Photomatix
{
	ProgressForm::ProgressForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		Gtk::Dialog(cobject),
		progressNotifier(new LinuxProgressNotifier())
	{
		this->progressNotifier->signal_progress.connect(sigc::mem_fun(*this, &ProgressForm::OnProgress));
		
		builder->get_widget("lblMessage", this->lblMessage);
		builder->get_widget("prgProgress", this->prgProgress);
		builder->get_widget("btnStop", this->btnStop);
		if(this->btnStop)
			this->btnStop->signal_clicked().connect(sigc::mem_fun(*this, &ProgressForm::OnButtonStopClicked));
		
		this->set_size_request(400, 100);
	}

	ProgressForm::~ProgressForm()
	{

	}
	
	void ProgressForm::OnProgress()
	{
		PhotomatixImaging::ProgressInfo info = progressNotifier->GetProgress();
		if(info.message.length() > 0)
		{
			this->lblMessage->set_text(info.message);
		}
		this->prgProgress->set_fraction((double)info.progress / info.total);
		
		Gtk::Main::iteration(false);
		
		std::cout << info.message << " " << info.progress << "/" << info.total << std::endl;
	}
	
	void ProgressForm::OnButtonStopClicked()
	{
		this->signal_stop_button_clicked.emit();
	}
}