#include <iostream>
#include <sys/stat.h>

#include <fastformat/fastformat.hpp>

#include "load_bracketed_dialog.h"
#include "hdrgenerator.h"
#include "openimageform.h"
#include "photoimage.h"
#include "singlerawconversiondialog.h"
#include "../PhotomatixMisc/appproperties.h"
#include "GladeUtils.h"
#include "hdrgenerator.h"
#include "localtext.h"
#include "appproperties.h"


using namespace std;
using namespace PhotomatixImaging;
using namespace PhotomatixMisc;


namespace Photomatix
{
	SingleRawConversionDialog::SingleRawConversionDialog(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		Gtk::Dialog(cobject)
	{
		whiteBalanceSettings[0] = EWhiteBalance::WBAsShot;
		whiteBalanceSettings[1] = EWhiteBalance::WBAuto;
		whiteBalanceSettings[2] = EWhiteBalance::WBDaylight;
		whiteBalanceSettings[3] = EWhiteBalance::WBCloudy;
		whiteBalanceSettings[4] = EWhiteBalance::WBShade;
		whiteBalanceSettings[5] = EWhiteBalance::WBTungsten;
		whiteBalanceSettings[6] = EWhiteBalance::WBFluorescent;
		whiteBalanceSettings[7] = EWhiteBalance::WBCustom;
		
		
		builder->get_widget("btnCancel", this->btnCancel);
		builder->get_widget("btnOk", this->btnOk);
		builder->get_widget("cbxColorSpace", this->cbxColorSpace);
		builder->get_widget("cbxWhiteBalance", this->cbxWhiteBalance);
		builder->get_widget("chkReduceNoise", this->chkReduceNoise);
		builder->get_widget("label1", this->label1);
		builder->get_widget("label2", this->label2);
		builder->get_widget("label3", this->label3);
		builder->get_widget("sldReduceNoiseStrength", this->sldReduceNoiseStrength);
		builder->get_widget("txtCustomTemperature", this->txtCustomTemperature);
	
		builder->get_widget("btnOk", btnOk);
		btnOk->signal_clicked().connect(sigc::mem_fun(*this, &SingleRawConversionDialog::OnOkButtonClicked)); 
		
		this->chkReduceNoise->signal_toggled().connect(sigc::mem_fun(*this, &SingleRawConversionDialog::OnReduceNoiseToggled));
		this->cbxWhiteBalance->signal_changed().connect(sigc::mem_fun(*this, &SingleRawConversionDialog::OnWhiteBalanceSelectedIndexChanged));
		
		this->txtCustomTemperature->signal_focus_out_event().connect(sigc::mem_fun(*this, &SingleRawConversionDialog::OnCustomTemperatureLostFocus));
		
		this->cbxWhiteBalance->set_active(0);
		this->cbxColorSpace->set_active(0);
		
		this->sldReduceNoiseStrength->set_range(50.0f, 150.0f);
		this->sldReduceNoiseStrength->set_value(100.0f);
		
		this->set_resizable(false);
		
		this->ReadStickySettings();
	}
	
	void SingleRawConversionDialog::OnReduceNoiseToggled()
	{
		this->sldReduceNoiseStrength->set_sensitive(this->chkReduceNoise->get_active());
		this->label3->set_sensitive(this->chkReduceNoise->get_active());
	}
	
	bool SingleRawConversionDialog::OnCustomTemperatureLostFocus(GdkEventFocus* evt)
	{
		if(this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()] != EWhiteBalance::WBCustom)
			return false;
		
		int curTemperature = atoi(this->txtCustomTemperature->get_text().c_str());
		// Limit available temperature to the range of 2000 - 50000 (one used by Lightroom 4)
		if(curTemperature < 2000 || curTemperature > 50000)
		{
			if(curTemperature < 2000) curTemperature = 2000;
			if(curTemperature > 50000) curTemperature = 50000;
			
			stringstream temperature;
			temperature << curTemperature;
			this->txtCustomTemperature->set_text(temperature.str());
		}
		
		return false;
	}

	SingleRawConversionDialog::~SingleRawConversionDialog()
	{
	}
	
	void SingleRawConversionDialog::OnWhiteBalanceSelectedIndexChanged()
	{
		int index = this->cbxWhiteBalance->get_active_row_number();
		EWhiteBalance curWB = this->whiteBalanceSettings[index];
		if(curWB != EWhiteBalance::WBAsShot && curWB != EWhiteBalance::WBAuto)
		{
			stringstream whiteBalance;
			whiteBalance << curWB;
			this->txtCustomTemperature->set_text(whiteBalance.str());
		}
		
		if(curWB == EWhiteBalance::WBCustom)
		{
			stringstream whiteBalance;
			whiteBalance << (int)EWhiteBalance::WBDaylight;
			this->txtCustomTemperature->set_text(whiteBalance.str());
			this->txtCustomTemperature->set_editable(true);
			this->txtCustomTemperature->set_sensitive(true);
		}
		else
		{
			this->txtCustomTemperature->set_editable(false);
			this->txtCustomTemperature->set_sensitive(false);
		}
	}
	
	void SingleRawConversionDialog::OnOkButtonClicked()
	{
		this->SaveStickySettings();
		this->hide();
	}

	void SingleRawConversionDialog::ReadStickySettings()
	{
		this->chkReduceNoise->set_active(AppProperties::GetSingleRawNoiseReductionEnabled());
		this->sldReduceNoiseStrength->set_value(AppProperties::GetSingleRawNoiseReductionStrength());
		this->cbxWhiteBalance->set_active(AppProperties::GetSingleRawWhiteBalance());
		this->cbxColorSpace->set_active(AppProperties::GetSingleRawColorProfile());
	}

	void SingleRawConversionDialog::SaveStickySettings()
	{
		AppProperties::SetSingleRawNoiseReductionEnabled(this->chkReduceNoise->get_active());
		AppProperties::SetSingleRawNoiseReductionStrength(this->sldReduceNoiseStrength->get_value());
		AppProperties::SetSingleRawWhiteBalance(this->cbxWhiteBalance->get_active_row_number());
		AppProperties::SetSingleRawColorProfile(this->cbxColorSpace->get_active_row_number());
	}
}