#include "contrastoptimizersettings.h"
#include <iostream>
#include <iomanip>

using namespace Photomatix;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	ContrastOptimizerSettings::ContrastOptimizerSettings(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		ProcessingSettingsControl(cobject, builder)
	{
		// Set up sliders
		builder->get_widget("sldStrength", this->sldStrength);		
		builder->get_widget("sldToneCompression", this->sldToneCompression);
		builder->get_widget("sldLightingEffect", this->sldLightingEffect);
		builder->get_widget("sldWhiteClip", this->sldWhiteClip);
		builder->get_widget("sldBlackClip", this->sldBlackClip);
		builder->get_widget("sldMidtone", this->sldMidtone);
		builder->get_widget("sldColorSaturation", this->sldColorSaturation);
		builder->get_widget("sldColorTemperature", this->sldColorTemperature);
		builder->get_widget("sldLuminance", this->sldLuminance);
		
		builder->get_widget("lblStrength", this->lblStrength);		
		builder->get_widget("lblToneCompression", this->lblToneCompression);
		builder->get_widget("lblLightingEffect", this->lblLightingEffect);
		builder->get_widget("lblWhiteClip", this->lblWhiteClip);
		builder->get_widget("lblBlackClip", this->lblBlackClip);
		builder->get_widget("lblMidtone", this->lblMidtone);
		builder->get_widget("lblColorSaturation", this->lblColorSaturation);
		builder->get_widget("lblColorTemperature", this->lblColorTemperature);
		builder->get_widget("lblLuminance", this->lblLuminance);
		
		
		this->sldStrength->set_range(0, 100);
		this->sldStrength->set_digits(0);
		this->sldToneCompression->set_range(-10.0, 10.0);
		this->sldLightingEffect->set_range(0, 100);
		this->sldLightingEffect->set_digits(0);
		this->sldWhiteClip->set_range(0.0, 10.0);
		this->sldBlackClip->set_range(0.0, 10.0);
		this->sldMidtone->set_range(-10.0, 10.0);
		this->sldColorSaturation->set_range(-10.0, 10.0);
		this->sldColorTemperature->set_range(-10.0, 10.0);
		this->sldLuminance->set_range(-10.0, 10.0);
		
		this->sldStrength->set_value(50);
		this->sldLightingEffect->set_value(20);
		this->sldWhiteClip->set_value(4.0);
		this->sldBlackClip->set_value(3.0);
		
		this->sldStrength->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldToneCompression->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLightingEffect->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldWhiteClip->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlackClip->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldMidtone->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldColorSaturation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldColorTemperature->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLuminance->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		
		this->sldStrength->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldStrength));
		this->sldToneCompression->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldToneCompression));
		this->sldLightingEffect->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldLightingEffect));
		this->sldWhiteClip->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldWhiteClip));
		this->sldBlackClip->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldBlackClip));
		this->sldMidtone->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldMidtone));
		this->sldColorSaturation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldColorSaturation));
  		this->sldColorTemperature->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldColorTemperature));
		this->sldLuminance->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ContrastOptimizerSettings::OnSliderValueChanging), this->sldLuminance));
		
		// Update all sliders' values
		this->OnSliderValueChanging(this->sldStrength);
		this->OnSliderValueChanging(this->sldToneCompression);
		this->OnSliderValueChanging(this->sldLightingEffect);
		this->OnSliderValueChanging(this->sldWhiteClip);
		this->OnSliderValueChanging(this->sldBlackClip);
		this->OnSliderValueChanging(this->sldMidtone);
		this->OnSliderValueChanging(this->sldColorSaturation);
		this->OnSliderValueChanging(this->sldColorTemperature);
		this->OnSliderValueChanging(this->sldLuminance);
	}

	ContrastOptimizerSettings::~ContrastOptimizerSettings()
	{
	}
	
	XmpSettings ContrastOptimizerSettings::GetSettings()
	{
		float ClipPowerOf2 = logf(0.025f) / logf(0.3f);
		float invPower = 1.0f / ClipPowerOf2;

		XmpSettings settings;
		settings.parameters[0] = this->sldStrength->get_value();
		settings.parameters[1] = this->sldToneCompression->get_value();
		settings.parameters[2] = this->sldLightingEffect->get_value();
		settings.parameters[3] = this->sldWhiteClip->get_value();//10.0f * powf(this->sldWhiteClip->get_value() / 10.0f, invPower);
		settings.parameters[4] = this->sldBlackClip->get_value();
		settings.parameters[8] = this->sldMidtone->get_value();
		settings.parameters[9] = this->sldLuminance->get_value();
		settings.parameters[12]= this->sldColorTemperature->get_value();
		settings.parameters[13]= this->sldColorSaturation->get_value();
		
		settings.method = 3;
		
		return settings;
	}
	
	void ContrastOptimizerSettings::SetSettings(XmpSettings* settings)
	{
		float ClipPowerOf2 = logf(0.025f) / logf(0.3f);
		float invPower = 1.0f / ClipPowerOf2;
		
		this->sldStrength->set_value(settings->parameters[0]);
		this->sldToneCompression->set_value(settings->parameters[1]);
		this->sldLightingEffect->set_value(settings->parameters[2]);
		float value = 100.0f * powf(settings->parameters[3] / 10.0f, invPower);
		this->sldWhiteClip->set_value(settings->parameters[3]);
		this->sldBlackClip->set_value(settings->parameters[4]);
		this->sldMidtone->set_value(settings->parameters[8]);
		this->sldLuminance->set_value(settings->parameters[9]);
		this->sldColorTemperature->set_value(settings->parameters[12]);
		this->sldColorSaturation->set_value(settings->parameters[13]);
	}
	
	void ContrastOptimizerSettings::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldStrength) this->lblStrength->set_text(str.str());
		else if(sender == this->sldToneCompression) this->lblToneCompression->set_text(str.str());
		else if(sender == this->sldLightingEffect) this->lblLightingEffect->set_text(str.str());
		else if(sender == this->sldWhiteClip) this->lblWhiteClip->set_text(str.str());
		else if(sender == this->sldBlackClip) this->lblBlackClip->set_text(str.str());
		else if(sender == this->sldMidtone) this->lblMidtone->set_text(str.str());
		else if(sender == this->sldColorSaturation) this->lblColorSaturation->set_text(str.str());
		else if(sender == this->sldColorTemperature) this->lblColorTemperature->set_text(str.str());
		else if(sender == this->sldLuminance) this->lblLuminance->set_text(str.str());
	}

}