#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"

#include <iostream>
#include <algorithm>

#include "displayableimage.h"
#include "../PhotomatixImaging/photoimage.h"

#include "cairomm/context.h"
#include "gdkmm/general.h"
#include <gtkmm/scrollbar.h>
#include <gtkmm/adjustment.h>


using namespace std;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	bool DisplayableDrawingArea::on_draw(const Cairo::RefPtr< Cairo::Context >& cr)
	{
		if (!this->pixbuf)
			return false;
		
		// Apply transformation matrix
		cr->scale(this->scale, this->scale);
		
		// Render the pixel buffer
		Gdk::Cairo::set_source_pixbuf(cr, pixbuf, 0, 0);
		cr->paint();
		
		if(!cropRect.has_zero_area()) {
			cr->scale(1.0f / scale, 1.0f / scale);
			//int w = this->get_allocated_width() / scale;
			//int h = this->get_allocated_height() / scale;
			int w = this->get_allocated_width();
			int h = this->get_allocated_height();
			
			cr->set_source_rgba(0, 0, 0, 0.3f);
			
			cr->rectangle(0, 0, cropRect.get_x() * scale, h);
			cr->fill();
			cr->rectangle((cropRect.get_x() + cropRect.get_width()) * scale, 0, w - (cropRect.get_x() + cropRect.get_width()) * scale, h);
			cr->fill();
			cr->rectangle(cropRect.get_x() * scale, 0, cropRect.get_width() * scale, cropRect.get_y() * scale);
			cr->fill();
			cr->rectangle(cropRect.get_x() * scale, (cropRect.get_y() + cropRect.get_height()) * scale, cropRect.get_width() * scale, h - (cropRect.get_y() + cropRect.get_height()) * scale);
			cr->fill();
			
			
			cr->set_line_width(1);
			cr->set_source_rgb(1.0f, 1.0f, 1.0f);
			cr->rectangle(cropRect.get_x() * scale, cropRect.get_y() * scale, cropRect.get_width() * scale, cropRect.get_height() * scale);
			std::valarray< double > dashes(2);
			dashes[0] = 4.0;
			dashes[1] = 4.0;

			cr->set_dash(dashes, 0);
			
			cr->stroke();
		}
		
		if(hasBorder) {
			cr->scale(1.0f / scale, 1.0f / scale);
			cr->set_line_width(3);
			cr->set_source_rgba(0, 0, 0, 1.0f);
			cr->rectangle(0, 0, this->get_width(), this->get_height());
			cr->stroke();
		}
		
		return true;
	}
	
	void DisplayableDrawingArea::ShowCropRectangle(Gdk::Rectangle cropRect)
	{
		this->cropRect = cropRect;
		this->queue_draw();
	}
	
	void DisplayableDrawingArea::SetBorder(bool border)
	{
		this->hasBorder = border;
		this->queue_draw();
	}
	
	
	
	DisplayableImage::DisplayableImage():
		image(NULL)
	{		
		this->exposureLevel = 1.0f;
		this->setUp = false;
	}

	DisplayableImage::DisplayableImage(const string& fileName):
		image(NULL)
	{	
		this->Open(fileName);
		
		this->exposureLevel = 1.0f;
		this->setUp = false;
	}
	
	DisplayableImage::~DisplayableImage()
	{
	}
	
	void DisplayableImage::ExposureUp()
	{
		this->exposureLevel *= 0.5f;
		this->RemoveChildControl();
		this->SetPhoto(this->photo);
	}
	
	void DisplayableImage::ExposureDown()
	{
		this->exposureLevel *= 2.0f;
		this->RemoveChildControl();
		this->SetPhoto(this->photo);
	}
	
	void DisplayableImage::UpdatePixBuf()
	{
		if(photo->GetBitsPerSample() == 8)
		{
			if(photo->GetSamplesPerPixel() == 3)
				memcpy(this->pixBuf->get_pixels(), this->photo->GetImageBuffer()->data(), this->photo->GetWidth() * this->photo->GetHeight() * 3);
			else if(photo->GetSamplesPerPixel() == 1)
			{
				// TODO: at the moment greyscale images are filled by iterating over the array but this is extremely slow - I guess there is another way to do this
				for(uint i = 0; i < this->photo->GetWidth() * this->photo->GetHeight(); ++i)
				{
					this->pixBuf->get_pixels()[i * 3 + 0] = this->photo->GetImageBuffer()->data()[i];
					this->pixBuf->get_pixels()[i * 3 + 1] = this->photo->GetImageBuffer()->data()[i];
					this->pixBuf->get_pixels()[i * 3 + 2] = this->photo->GetImageBuffer()->data()[i];
				}
			}
		}
		else
		{
			from16to8copy(this->photo->GetWidth(), this->photo->GetHeight(), this->photo->GetSamplesPerPixel(),
						  this->photo->GetWidth() * this->photo->GetSamplesPerPixel() * 2,
						  this->photo->GetImageBuffer()->data(), this->pixBuf->get_pixels());
			// TODO: at the moment greyscale images are filled by iterating over the array but this is extremely slow - I guess there is another way to do this
			if(this->photo->GetSamplesPerPixel() == 1)
			{
				int count = this->photo->GetWidth() * this->photo->GetHeight() * 3;
				for(uint i = this->photo->GetWidth() * this->photo->GetHeight(); i > 0; --i)
				{
					this->pixBuf->get_pixels()[count - 0] = this->pixBuf->get_pixels()[i];
					this->pixBuf->get_pixels()[count - 1] = this->pixBuf->get_pixels()[i];
					this->pixBuf->get_pixels()[count - 2] = this->pixBuf->get_pixels()[i];
					count -= 3;
				}
			}
		}
	}
	
	void DisplayableImage::SetPhoto(std::shared_ptr< PhotomatixImaging::PhotoImage > photo)
	{
		this->photo = photo;
		
		this->pixBuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, photo->GetWidth(), photo->GetHeight());
		
		int pixelBufRowStride = photo->GetWidth() * 3;
		while(pixelBufRowStride % 4 != 0) ++pixelBufRowStride;
		
		unsigned char ratios[256 * 256];
		for (int pixel = 0; pixel < 256; ++pixel)
		{
			for (int ratio = 0; ratio < 256; ++ratio)
			{
				ratios[pixel * 256 + ratio] = (unsigned char)((pixel * ratio) >> 8);
			}
		}
		
		float exps2[256];
		for (int i = 0; i < 256; ++i)
		{
			exps2[i] = powf(2.0f, i-128);
		}
		
		if(photo->GetBitsPerSample() == 32)
		{
			copyScaledBitmapFromBufferOptimized(photo->GetWidth(), photo->GetHeight(), photo->GetWidth(), photo->GetHeight(), pixelBufRowStride, photo->GetRowStride(), photo->GetSamplesPerPixel(), photo->GetBitsPerSample(), 0, 0, 0, 0, 1.0f, this->photo->GetPixelAverage() * exposureLevel, this->pixBuf->get_pixels(), this->photo->GetImageBuffer()->data(), ratios, exps2, 0, 0, "");
			bgr2rgb(photo->GetWidth(), photo->GetHeight(), photo->GetRowStride(), photo->GetSamplesPerPixel(), /*photo->GetBitsPerSample()*/8, this->pixBuf->get_pixels());
		}
		else
			this->UpdatePixBuf();
		
		if(this->image)
		{
			this->RemoveChildControl();
		}
		
		this->image = new DisplayableDrawingArea();
		this->image->add_events(Gdk::BUTTON_PRESS_MASK);
		this->image->add_events(Gdk::BUTTON_RELEASE_MASK);
		this->image->add_events(Gdk::POINTER_MOTION_MASK);
		this->image->signal_mouse_down.connect(sigc::mem_fun(*this, &DisplayableImage::OnMouseDown));
		this->image->signal_mouse_up.connect(sigc::mem_fun(*this, &DisplayableImage::OnMouseUp));
		this->image->signal_mouse_moved.connect(sigc::mem_fun(*this, &DisplayableImage::OnMouseMoved));
		image->SetPixBuf(this->pixBuf);
	}
	
	void DisplayableImage::OnMouseDown(GdkEventButton* evt)
	{
		this->signal_mouse_down.emit(evt);
	}
	
	void DisplayableImage::OnMouseUp(GdkEventButton* evt)
	{
		this->signal_mouse_up.emit(evt);
	}
	
	void DisplayableImage::OnMouseMoved(GdkEventMotion* evt)
	{
		this->signal_mouse_moved.emit(evt);
	}
	
	void DisplayableImage::Open(const string& fileName, bool openRAWs, int colorTemperature, int colorSpace)
	{
		this->photo = make_shared <PhotoImage>(fileName, true, colorSpace, colorTemperature);
		if((openRAWs || this->photo->GetImageFormat() != EImageFormatType::Raw) && this->photo->GetImageFormat() != EImageFormatType::Hdr)
		{
			// TODO: note that pixBuf cannot have more than 8 bits per sample at the moment but this may be possible in the future updates of GTKMM
			this->pixBuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, photo->GetWidth(), photo->GetHeight());
			
			this->UpdatePixBuf();
			
			if(photo->GetSamplesPerPixel() == 3)
				bgr2rgb(photo->GetWidth(), photo->GetHeight(), photo->GetRowStride(), photo->GetSamplesPerPixel(), /*photo->GetBitsPerSample()*/8, this->pixBuf->get_pixels());
		}		
			
		this->image = new DisplayableDrawingArea();
		image->SetPixBuf(this->pixBuf);
	}
	
	void DisplayableImage::ShowCropRectangle(Gdk::Rectangle cropRect)
	{
		this->image->ShowCropRectangle(cropRect);
	}
	
	void DisplayableImage::SetBorder(bool hasBorder)
	{
		this->image->SetBorder(hasBorder);
	}
}