#include "tonecompressorsettings.h"
#include <gtkmm-3.0/gtkmm/label.h>
#include <iostream>
#include <iomanip>

using namespace Photomatix;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	ToneCompressorSettings::ToneCompressorSettings(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		ProcessingSettingsControl(cobject, builder)
	{
		// Set up sliders
		builder->get_widget("sldBrightness", this->sldBrightness);		
		builder->get_widget("sldTonalRangeCompression", this->sldTonalRangeCompression);
		builder->get_widget("sldContrastAdaptation", this->sldContrastAdaptation);
		builder->get_widget("sldWhitePoint", this->sldWhitePoint);
		builder->get_widget("sldBlackPoint", this->sldBlackPoint);
		builder->get_widget("sldTemperature", this->sldTemperature);
		builder->get_widget("sldSaturation", this->sldSaturation);
		
		builder->get_widget("lblBrightnessVal", this->lblBrightnessVal);
		builder->get_widget("lblTonalRangeCompression", this->lblTonalRangeCompression);
		builder->get_widget("lblContrastAdaptation", this->lblContrastAdaptation);
		builder->get_widget("lblWhitePoint", this->lblWhitePoint);
		builder->get_widget("lblBlackPoint", this->lblBlackPoint);
		builder->get_widget("lblTemperature", this->lblTemperature);
		builder->get_widget("lblSaturation", this->lblSaturation);
		
		
		this->sldBrightness->set_range(-10.0, 10.0);
		this->sldTonalRangeCompression->set_range(-10.0, 10.0);
		this->sldContrastAdaptation->set_range(-10.0, 10.0);
		this->sldWhitePoint->set_range(0.0, 10.0);
		this->sldBlackPoint->set_range(0.0, 10.0);
		this->sldTemperature->set_range(-10.0, 10.0);
		this->sldSaturation->set_range(-10.0, 10.0);
		
		this->sldBrightness->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldTonalRangeCompression->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldContrastAdaptation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldWhitePoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlackPoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldTemperature->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSaturation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		
		this->sldBrightness->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldBrightness));
		this->sldTonalRangeCompression->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldTonalRangeCompression));
		this->sldContrastAdaptation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldContrastAdaptation));
		this->sldWhitePoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldWhitePoint));
		this->sldBlackPoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldBlackPoint));
		this->sldTemperature->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldTemperature));
		this->sldSaturation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneCompressorSettings::OnSliderValueChanging), this->sldSaturation));
		
		// Update all sliders' values
		this->OnSliderValueChanging(this->sldBrightness);
		this->OnSliderValueChanging(this->sldTonalRangeCompression);
		this->OnSliderValueChanging(this->sldContrastAdaptation);
		this->OnSliderValueChanging(this->sldWhitePoint);
		this->OnSliderValueChanging(this->sldBlackPoint);
		this->OnSliderValueChanging(this->sldTemperature);
		this->OnSliderValueChanging(this->sldSaturation);
	}

	ToneCompressorSettings::~ToneCompressorSettings()
	{
	}
	
	XmpSettings ToneCompressorSettings::GetSettings()
	{
		XmpSettings settings;
		settings.parameters[0] = this->sldBrightness->get_value();
		settings.parameters[1] = this->sldTonalRangeCompression->get_value();
		settings.parameters[2] = this->sldContrastAdaptation->get_value();
		settings.parameters[3] = this->sldWhitePoint->get_value();
		settings.parameters[4] = this->sldBlackPoint->get_value();
		settings.parameters[12]= this->sldTemperature->get_value();
		settings.parameters[13]= this->sldSaturation->get_value();
		
		settings.method = 2;
		
		return settings;
	}
	
	void ToneCompressorSettings::SetSettings(XmpSettings* settings)
	{
		this->sldBrightness->set_value(settings->parameters[0]);
		this->sldTonalRangeCompression->set_value(settings->parameters[1]);
		this->sldContrastAdaptation->set_value(settings->parameters[2]);
		this->sldWhitePoint->set_value(settings->parameters[3]);
		this->sldBlackPoint->set_value(settings->parameters[4]);
		this->sldTemperature->set_value(settings->parameters[12]);
		this->sldSaturation->set_value(settings->parameters[13]);
	}
	
	void ToneCompressorSettings::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldBrightness) this->lblBrightnessVal->set_text(str.str());
		else if(sender == this->sldTonalRangeCompression) this->lblTonalRangeCompression->set_text(str.str());
		else if(sender == this->sldContrastAdaptation) this->lblContrastAdaptation->set_text(str.str());
		else if(sender == this->sldWhitePoint) this->lblWhitePoint->set_text(str.str());
		else if(sender == this->sldBlackPoint) this->lblBlackPoint->set_text(str.str());
		else if(sender == this->sldTemperature) this->lblTemperature->set_text(str.str());
		else if(sender == this->sldSaturation) this->lblSaturation->set_text(str.str());
	}

}