#ifndef APPPROPERTIES_H
#define APPPROPERTIES_H

#ifdef WEBSERVER
#include <string>
#include <map>
#include <memory>
#include <boost/lexical_cast.hpp>
#else
#include <gtkmm.h>
//#include <giomm/settingsschemasource.h>
#endif

namespace PhotomatixMisc
{
	/// Class for storing global properties (like eg. registration data).
	class AppProperties
	{
	private:
#ifdef WEBSERVER
		class AppSettings
		{
		public:
		  AppSettings() { data["number-of-processors"] = "1"; }
		  void set_int(const std::string &name, int value) { data[name] = boost::lexical_cast<std::string>(value); }
		  void set_double(const std::string &name, double value) { data[name] = boost::lexical_cast<std::string>(value); }
		  void set_string(const std::string &name, const std::string &value) { data[name] = value; }
		  void set_boolean(const std::string &name, bool value) { data[name] = value ? "true" : "false"; }
		  
		  int get_int(const std::string &name) { return data[name] == "" ? 0 : boost::lexical_cast<int>(data[name]); }
		  double get_double(const std::string &name) { return data[name] == "" ? 0.0 : boost::lexical_cast<double>(data[name]); }
		  std::string get_string(const std::string &name) { return data[name]; }
		  bool get_boolean(const std::string &name) { return data[name] == "true" ? true : false; }  
		private:
		  std::map<std::string, std::string> data;
		};
		static std::shared_ptr<AppSettings> settings;
#else
	  
		static Glib::RefPtr<Gio::Settings> settings;
#endif
		
	public:
		AppProperties();
		virtual ~AppProperties();
		
		static bool IsRegistered();
		
		static bool Init()
		{
#ifdef WEBSERVER
			// TODO: Fetch settings from somewhere?
#else
			if(g_settings_schema_source_lookup(g_settings_schema_source_get_default(), "apps.photomatix", true) != 0) {
				AppProperties::settings = Gio::Settings::create("apps.photomatix");
				return true;
			} else {
				return false;
			}
#endif
		}
		
		static void SetNumberOfProcessors(int value)
		{
			settings->set_int("number-of-processors", value);
		}
		static int GetNumberOfProcessors()
		{
			return settings->get_int("number-of-processors");
		}

		static void SetLastOpenDirectory(const std::string& value)
		{
			settings->set_string("last-open-directory", value);
		}
		static std::string GetLastOpenDirectory()
		{
			return settings->get_string("last-open-directory");
		}
		static void SetGenerateHdrAlignmentEnabled(bool value)
		{
			settings->set_boolean("generate-hdr-alignment-enabled", value);
		}
		static bool GetGenerateHdrAlignmentEnabled()
		{
			return settings->get_boolean("generate-hdr-alignment-enabled");
		}
		static void SetGenerateHdrAlignmentPerspective(bool value)
		{
			settings->set_boolean("generate-hdr-alignment-perspective", value);
		}
		static bool GetGenerateHdrAlignmentPerspective()
		{
			return settings->get_boolean("generate-hdr-alignment-perspective");
		}
		static void SetGenerateHdrAlignmentType(int value)
		{
			settings->set_int("generate-hdr-alignment-type", value);
		}
		static int GetGenerateHdrAlignmentType()
		{
			return settings->get_int("generate-hdr-alignment-type");
		}
		static void SetGenerateHdrAlignmentCrop(bool value)
		{
			settings->set_boolean("generate-hdr-alignment-crop", value);
		}
		static bool GetGenerateHdrAlignmentCrop()
		{
			return settings->get_boolean("generate-hdr-alignment-crop");
		}
		static void SetGenerateHdrAlignmentMaxShiftPercent(double value)
		{
			settings->set_double("generate-hdr-alignment-max-shift-percent", value);
		}
		static double GetGenerateHdrAlignmentMaxShiftPercent()
		{
			return settings->get_double("generate-hdr-alignment-max-shift-percent");
		}
		static void SetGenerateHdrDeghostingEnabled(bool value)
		{
			settings->set_boolean("generate-hdr-deghosting-enabled", value);
		}
		static bool GetGenerateHdrDeghostingEnabled()
		{
			return settings->get_boolean("generate-hdr-deghosting-enabled");
		}
		static void SetGenerateHdrDeghostingStrength(int value)
		{
			settings->set_int("generate-hdr-deghosting-strength", value);
		}
		static int GetGenerateHdrDeghostingStrength()
		{
			return settings->get_int("generate-hdr-deghosting-strength");
		}
		static void SetGenerateHdrColorProfile(int value)
		{
			settings->set_int("generate-hdr-color-profile", value);
		}
		static int GetGenerateHdrColorProfile()
		{
			return settings->get_int("generate-hdr-color-profile");
		}
		static void SetGenerateHdrWhiteBalance(int value)
		{
			settings->set_int("generate-hdr-white-balance", value);
		}
		static int GetGenerateHdrWhiteBalance()
		{
			return settings->get_int("generate-hdr-white-balance");
		}
		static void SetGenerateHdrNoiseReductionEnabled(bool value)
		{
			settings->set_boolean("generate-hdr-noise-reduction-enabled", value);
		}
		static bool GetGenerateHdrNoiseReductionEnabled()
		{
			return settings->get_boolean("generate-hdr-noise-reduction-enabled");
		}
		static void SetGenerateHdrNoiseReductionStrength(int value)
		{
			settings->set_int("generate-hdr-noise-reduction-strength", value);
		}
		static int GetGenerateHdrNoiseReductionStrength()
		{
			return settings->get_int("generate-hdr-noise-reduction-strength");
		}
		static void SetGenerateHdrNoiseReductionType(int value)
		{
			settings->set_int("generate-hdr-noise-reduction-type", value);
		}
		static int GetGenerateHdrNoiseReductionType()
		{
			return settings->get_int("generate-hdr-noise-reduction-type");
		}
		static void SetSingleRawNoiseReductionEnabled(bool value)
		{
			settings->set_boolean("single-raw-noise-reduction-enabled", value);
		}
		static bool GetSingleRawNoiseReductionEnabled()
		{
			return settings->get_boolean("single-raw-noise-reduction-enabled");
		}
		static void SetSingleRawNoiseReductionStrength(int value)
		{
			settings->set_int("single-raw-noise-reduction-strength", value);
		}
		static int GetSingleRawNoiseReductionStrength()
		{
			return settings->get_int("single-raw-noise-reduction-strength");
		}
		static void SetSingleRawWhiteBalance(int value)
		{
			settings->set_int("single-raw-white-balance", value);
		}
		static int GetSingleRawWhiteBalance()
		{
			return settings->get_int("single-raw-white-balance");
		}
		static void SetSingleRawColorProfile(int value)
		{
			settings->set_int("single-raw-color-profile", value);
		}
		static int GetSingleRawColorProfile()
		{
			return settings->get_int("single-raw-color-profile");
		}
		static void SetSaveJpegQuality(int value)
		{
			settings->set_int("save-jpeg-quality", value);
		}
		static int GetSaveJpegQuality()
		{
			return settings->get_int("save-jpeg-quality");
		}
		static void SetLicenseKey(const std::string& value)
		{
			settings->set_string("license-key", value);
		}
		static std::string GetLicenseKey()
		{
			return settings->get_string("license-key");
		}
		static void SetReduceNoiseLdr(bool value)
		{
			settings->set_boolean("reduce-noise-ldr", value);
		}
		static bool GetReduceNoiseLdr()
		{
			return settings->get_boolean("reduce-noise-ldr");
		}
		static void SetUseStickyTmSettings(bool value)
		{
			settings->set_boolean("use-sticky-tm-settings", value);
		}
		static bool GetUseStickyTmSettings()
		{
			return settings->get_boolean("use-sticky-tm-settings");
		}
		static void SetJpegQuality(int value)
		{
			settings->set_int("jpeg-quality", value);
		}
		static int GetJpegQuality()
		{
			return settings->get_int("jpeg-quality");
		}
		static void SetConcatenateSourceFilenames(bool value)
		{
			settings->set_boolean("concatenate-source-filenames", value);
		}
		static bool GetConcatenateSourceFilenames()
		{
			return settings->get_boolean("concatenate-source-filenames");
		}
		static void SetAppendSuffix(bool value)
		{
			settings->set_boolean("append-suffix", value);
		}
		static bool GetAppendSuffix()
		{
			return settings->get_boolean("append-suffix");
		}
		static void SetAutomaticSuffix(bool value)
		{
			settings->set_boolean("automatic-suffix", value);
		}
		static bool GetAutomaticSuffix()
		{
			return settings->get_boolean("automatic-suffix");
		}
		static void SetCustomSuffix(const std::string& value)
		{
			settings->set_string("custom-suffix", value);
		}
		static std::string GetCustomSuffix()
		{
			return settings->get_string("custom-suffix");
		}
		static void SetUseLastOpenDirForSave(bool value)
		{
			settings->set_boolean("use-last-open-dir-for-save", value);
		}
		static bool GetUseLastOpenDirForSave()
		{
			return settings->get_boolean("use-last-open-dir-for-save");
		}
		static void SetBypassDraggedFilesDialog(bool value)
		{
			settings->set_boolean("bypass-dragged-files-dialog", value);
		}
		static bool GetBypassDraggedFilesDialog()
		{
			return settings->get_boolean("bypass-dragged-files-dialog");
		}
		static void SetBypassAllDialogsBeforeTm(bool value)
		{
			settings->set_boolean("bypass-all-dialogs-before-tm", value);
		}
		static bool GetBypassAllDialogsBeforeTm()
		{
			return settings->get_boolean("bypass-all-dialogs-before-tm");
		}
		static void SetPreviewZoomLevel(int value)
		{
			settings->set_int("preview-zoom-level", value);
		}
		static int GetPreviewZoomLevel()
		{
			return settings->get_int("preview-zoom-level");
		}
		static void SetSaveDialogFilter(int value)
		{
			settings->set_int("save-dialog-filter", value);
		}
		static int GetSaveDialogFilter()
		{
			return settings->get_int("save-dialog-filter");
		}
		static void SetBatchBracketsNumber(int value)
		{
			settings->set_int("batch-brackets-number", value);
		}
		static int GetBatchBracketsNumber()
		{
			return settings->get_int("batch-brackets-number");
		}
		static void SetBatchAlignEnabled(bool value)
		{
			settings->set_boolean("batch-align-enabled", value);
		}
		static bool GetBatchAlignEnabled()
		{
			return settings->get_boolean("batch-align-enabled");
		}
		static void SetBatchAlignTripod(bool value)
		{
			settings->set_boolean("batch-align-tripod", value);
		}
		static bool GetBatchAlignTripod()
		{
			return settings->get_boolean("batch-align-tripod");
		}
		static void SetBatchAlignCrop(bool value)
		{
			settings->set_boolean("batch-align-crop", value);
		}
		static bool GetBatchAlignCrop()
		{
			return settings->get_boolean("batch-align-crop");
		}
		static void SetBatchAlignPerspectiveCorrection(bool value)
		{
			settings->set_boolean("batch-align-perspective-correction", value);
		}
		static bool GetBatchAlignPerspectiveCorrection()
		{
			return settings->get_boolean("batch-align-perspective-correction");
		}
		static void SetBatchFileFormat(int value)
		{
			settings->set_int("batch-file-format", value);
		}
		static int GetBatchFileFormat()
		{
			return settings->get_int("batch-file-format");
		}
		static void SetBatchRemoveHdr(bool value)
		{
			settings->set_boolean("batch-remove-hdr", value);
		}
		static bool GetBatchRemoveHdr()
		{
			return settings->get_boolean("batch-remove-hdr");
		}
		static void SetBatchRemoveGhosts(bool value)
		{
			settings->set_boolean("batch-remove-ghosts", value);
		}
		static bool GetBatchRemoveGhosts()
		{
			return settings->get_boolean("batch-remove-ghosts");
		}
		static void SetBatchReduceNoise(bool value)
		{
			settings->set_boolean("batch-reduce-noise", value);
		}
		static bool GetBatchReduceNoise()
		{
			return settings->get_boolean("batch-reduce-noise");
		}
		static void SetBatchReduceNoiseType(int value)
		{
			settings->set_int("batch-reduce-noise-type", value);
		}
		static int GetBatchReduceNoiseType()
		{
			return settings->get_int("batch-reduce-noise-type");
		}
		static void SetBatchReduceNoiseStrength(int value)
		{
			settings->set_int("batch-reduce-noise-strength", value);
		}
		static int GetBatchReduceNoiseStrength()
		{
			return settings->get_int("batch-reduce-noise-strength");
		}
		static void SetBatchColorSpace(int value)
		{
			settings->set_int("batch-color-space", value);
		}
		static int GetBatchColorSpace()
		{
			return settings->get_int("batch-color-space");
		}
		static void SetBatchWhiteBalance(int value)
		{
			settings->set_int("batch-white-balance", value);
		}
		static int GetBatchWhiteBalance()
		{
			return settings->get_int("batch-white-balance");
		}
		static void SetBatchWhiteBalanceTemperature(int value)
		{
			settings->set_int("batch-white-balance-temperature", value);
		}
		static int GetBatchWhiteBalanceTemperature()
		{
			return settings->get_int("batch-white-balance-temperature");
		}
		static void SetBatchSettingsTm1(bool value)
		{
			settings->set_boolean("batch-settings-tm1", value);
		}
		static bool GetBatchSettingsTm1()
		{
			return settings->get_boolean("batch-settings-tm1");
		}
		static void SetBatchSettingsTm2(bool value)
		{
			settings->set_boolean("batch-settings-tm2", value);
		}
		static bool GetBatchSettingsTm2()
		{
			return settings->get_boolean("batch-settings-tm2");
		}
		static void SetBatchSettingsTm3(bool value)
		{
			settings->set_boolean("batch-settings-tm3", value);
		}
		static bool GetBatchSettingsTm3()
		{
			return settings->get_boolean("batch-settings-tm3");
		}
		static void SetBatchSettingsFusion(bool value)
		{
			settings->set_boolean("batch-settings-fusion", value);
		}
		static bool GetBatchSettingsFusion()
		{
			return settings->get_boolean("batch-settings-fusion");
		}
		
		static std::string GetUserPath();
		static void SetBatchLastOpenedDir(const std::string& value)
		{
			settings->set_string("batch-last-opened-dir", value);
		}
		static std::string GetBatchLastOpenedDir()
		{
			return settings->get_string("batch-last-opened-dir");
		}
		
		static void SetBuiltInPresetsSelected(bool value)
		{
			settings->set_boolean("builtin-presets-selected", value);
		}
		static bool GetBuiltInPresetsSelected()
		{
			return settings->get_boolean("builtin-presets-selected");
		}
		static void SetBatchSettingsToneBalancer(bool value)
		{
			settings->set_boolean("batch-settings-tone-balancer", value);
		}
		static bool GetBatchSettingsToneBalancer()
		{
			return settings->get_boolean("batch-settings-tone-balancer");
		}
		static void SetShowFinishingTouch(bool value)
		{
			settings->set_boolean("show-finishing-touch", value);
		}
		static bool GetShowFinishingTouch()
		{
			return settings->get_boolean("show-finishing-touch");
		}
		static void SetLastSaveDirectory(const std::string& value)
		{
			settings->set_string("last-save-directory", value);
		}
		static std::string GetLastSaveDirectory()
		{
			return settings->get_string("last-save-directory");
		}
		static void SetLoadPhotosX(int value)
		{
			settings->set_int("load-photos-x", value);
		}
		static int GetLoadPhotosX()
		{
			return settings->get_int("load-photos-x");
		}
		static void SetLoadPhotosY(int value)
		{
			settings->set_int("load-photos-y", value);
		}
		static int GetLoadPhotosY()
		{
			return settings->get_int("load-photos-y");
		}
	};
}

#endif // APPPROPERTIES_H
