#ifndef LOCALTEXT_H
#define LOCALTEXT_H

#include <map>
#include <string>


class TiXmlNode;


namespace PhotomatixMisc
{
	class LocalText
	{
	private:
		static std::string KEY_NOT_FOUND;		
		
		static std::map <std::string, std::string> texts;
		
		static void Parse(TiXmlNode* pParent);
		LocalText();
		
	public:
		LocalText(LocalText const&) = delete;
		LocalText& operator=(LocalText const&) = delete;
		
		static void LoadResourceFile(const std::string& fileName);
		
		static bool IsGermanVersion();
		
		
		std::string& GetResourceString(const std::string& key);
		
		static std::string& GetTextMSGCONVTM1() {return LocalText::texts["MSG_CONV_TM1"];}
		
		static std::string& GetTextMSGCONVTM2() {return LocalText::texts["MSG_CONV_TM2"];}
		
		static std::string& GetTextMSGCONVTMCONTRASTOPTMIZER() {return LocalText::texts["MSG_CONV_TM_CONTRAST_OPTMIZER"];}
		
		static std::string& GetTextMSGCONVTMTONEBALANCER() {return LocalText::texts["MSG_CONV_TM_TONE_BALANCER"];}
		
		static std::string& GetTextBlendingProcessingStatus() {return LocalText::texts["BlendingProcessingStatus"];}
		
		static std::string& GetTextTXTCOMBIHDR() {return LocalText::texts["TXT_COMBI_HDR"];}
		
		static std::string& GetTextReducingNoiseOnImage() {return LocalText::texts["ReducingNoiseOnImage"];}
		
		static std::string& GetTextTXTALIGN() {return LocalText::texts["TXT_ALIGN"];}
		
		static std::string& GetTextTXTLOAD2() {return LocalText::texts["TXT_LOAD2"];}
		
		static std::string& GetTextTXTLOAD3() {return LocalText::texts["TXT_LOAD3"];}
		
		static std::string& GetTextTXTRAW() {return LocalText::texts["TXT_RAW"];}
		
		static std::string& GetTextMSGBATCHALIGN() {return LocalText::texts["MSG_BATCH_ALIGN"];}
		
		static std::string& GetTextMsgBatchReducingNoise() {return LocalText::texts["MsgBatchReducingNoise"];}
		
		static std::string& GetTextMSGBATCHHDRI() {return LocalText::texts["MSG_BATCH_HDRI"];}
		
		static std::string& GetTextFILEBATCHRESFOLDER() {return LocalText::texts["FILE_BATCH_RESFOLDER"];}
		
		static std::string& GetTextMSGSAVE() {return LocalText::texts["MSG_SAVE"];}
		
		static std::string& GetTextRequiresAtLeastTwoImages() {return LocalText::texts["RequiresAtLeastTwoImages"];}
		
		static std::string& GetTextReduceNoiseBeforeToneMapping() {return LocalText::texts["TextReduceNoiseBeforeToneMapping"];}
		
		static std::string& GetTextMSGERRDEPTH() {return LocalText::texts["MSG_ERR_DEPTH"];}
		
		static std::string& GetTextMSGERRCHANNELS() {return LocalText::texts["MSG_ERR_CHANNELS"];}
		
		static std::string& GetTextMSGERRFORMAT() {return LocalText::texts["MSG_ERR_FORMAT"];}
		
		static std::string& GetTextMSGERRDIM() {return LocalText::texts["MSG_ERR_DIM"];}
		
		static std::string& GetTextMsgErrProcessNoImages() {return LocalText::texts["MsgErrProcessNoImages"];}
		
		static std::string& GetTextMSGCONVEND4() {return LocalText::texts["MSG_CONV_END4"];}
		
		static std::string& GetOpenDialogTitle() {return LocalText::texts["OpenDialogTitle"];}
		
		static std::string& GetSaveDialogTitle() {return LocalText::texts["SaveDialogTitle"];}
		
		static std::string& GetLoadPresetTitle() {return LocalText::texts["LoadPresetTitle"];}
		
		static std::string& GetSavePresetTitle() {return LocalText::texts["SavePresetTitle"];}
		
		static std::string& GetTextFusionNatural() {return LocalText::texts["FusionNatural"];}
		
		static std::string& GetTextBuiltInPresetNamesShortList() {return LocalText::texts["BuiltinPresetNamesShort"];}
		
		static std::string& GetTextBuiltInPresetNamesFullList() {return LocalText::texts["BuiltinPresetNamesFull"];}
		
		static std::string& GetTextAllImagesFilter() {return LocalText::texts["AllImagesFilter"];}
		
		static std::string& GetUserManualURL() {return LocalText::texts["UserManualURL"];}

		static std::string& GetTextCancel() {return LocalText::texts["TXT_CANCEL"];}

		static std::string& GetTextYes() {return LocalText::texts["TXT_YES"];}

		static std::string& GetTextNo() {return LocalText::texts["TXT_NO"];}
		
		static std::string& GetTextShowOptions() {return LocalText::texts["TXT_SHOWOPTIONS"];}
		
		static std::string& GetTextHideOptions() {return LocalText::texts["TXT_HIDEOPTIONS"];}
		
		static std::string& GetTextProcessingWithDetailsEnhancer() {return LocalText::texts["TextProcessingWithDetailsEnhancer"];}
		
		static std::string& GetTextProcessingWithToneCompressor() {return LocalText::texts["TextProcessingWithToneCompressor"];}
		
		static std::string& GetTextProcessingWithContrastOptimizer() {return LocalText::texts["TextProcessingWithContrastOptimizer"];}
		
		static std::string& GetTextProcessingWithToneBalancer() {return LocalText::texts["TextProcessingWithToneBalancer"];}
		
		static std::string& GetTextProcessingWithFusionNatural() {return LocalText::texts["TextProcessingWithFusionNatural"];}
		
		static std::string& GetTextRegistrationSuccess() {return LocalText::texts["TextRegistrationSuccess"];}
		
		static std::string& GetTextRegistrationFail() {return LocalText::texts["TextRegistrationFail"];}
		
		static std::string& GetTextRegistrationFailMsg() {return LocalText::texts["TextRegistrationFailMsg"];}
		
		static std::string& GetUrlOrderPage() {return LocalText::texts["UrlOrderPage"];}

	};
}

#endif // LOCALTEXT_H
